Package: likelihood.model
Type: Package
Title: Likelihood-Based Statistical Inference in the Fisherian
        Tradition
Version: 1.0.1
Authors@R: 
    person("Alexander", "Towell", , "lex@metafunctor.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6443-9897"))
Description: Facilitates building likelihood models in the Fisherian tradition
    following Richard Royall (1997, ISBN:978-0412044113) "Statistical Evidence: A
    Likelihood Paradigm". Defines generic methods for working with likelihoods
    (loglik(), score(), hess_loglik(), fim()) and provides functions for pure
    likelihood-based inference (support(), relative_likelihood(),
    likelihood_interval(), profile_loglik()).
License: MIT + file LICENSE
Encoding: UTF-8
Depends: algebraic.mle (>= 2.0.0)
Imports: algebraic.dist, generics, stats, numDeriv, boot
URL: https://github.com/queelius/likelihood.model,
        https://queelius.github.io/likelihood.model/
BugReports: https://github.com/queelius/likelihood.model/issues
Suggests: mvtnorm, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-24 08:17:51 UTC; spinoza
Author: Alexander Towell [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6443-9897>)
Maintainer: Alexander Towell <lex@metafunctor.com>
Repository: CRAN
Date/Publication: 2026-05-24 09:20:02 UTC
Built: R 4.5.2; ; 2026-05-24 13:51:37 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(assumptions,exponential_lifetime)
S3method(bias,fisher_boot)
S3method(bias,fisher_mle)
S3method(coef,fisher_mle)
S3method(confint,fisher_boot)
S3method(confint,fisher_mle)
S3method(deviance,fisher_mle)
S3method(evidence,likelihood_model)
S3method(fim,exponential_lifetime)
S3method(fim,likelihood_model)
S3method(fit,exponential_lifetime)
S3method(fit,likelihood_model)
S3method(hess_loglik,exponential_lifetime)
S3method(hess_loglik,likelihood_model)
S3method(likelihood_interval,fisher_mle)
S3method(logLik,fisher_mle)
S3method(loglik,exponential_lifetime)
S3method(loglik,likelihood_model)
S3method(mse,fisher_mle)
S3method(nobs,fisher_mle)
S3method(nparams,fisher_mle)
S3method(obs,fisher_mle)
S3method(observed_fim,fisher_mle)
S3method(params,fisher_mle)
S3method(print,fisher_boot)
S3method(print,fisher_mle)
S3method(print,likelihood_interval)
S3method(print,likelihood_model)
S3method(print,lrt_result)
S3method(print,profile_loglik)
S3method(print,summary_fisher_mle)
S3method(profile_loglik,fisher_mle)
S3method(rdata,exponential_lifetime)
S3method(rdata,likelihood_model)
S3method(relative_likelihood,fisher_mle)
S3method(sampler,fisher_boot)
S3method(sampler,fisher_mle)
S3method(sampler,likelihood_model)
S3method(score,exponential_lifetime)
S3method(score,likelihood_model)
S3method(score_val,fisher_mle)
S3method(se,fisher_mle)
S3method(summary,fisher_mle)
S3method(support,fisher_mle)
S3method(vcov,fisher_mle)
export(assumptions)
export(evidence)
export(exponential_lifetime)
export(fim)
export(fisher_boot)
export(fisher_mle)
export(fit)
export(hess_loglik)
export(is_likelihood_model)
export(likelihood_interval)
export(loglik)
export(lrt)
export(profile_loglik)
export(rdata)
export(relative_likelihood)
export(score)
export(support)
importFrom(algebraic.dist,nparams)
importFrom(algebraic.dist,obs)
importFrom(algebraic.dist,params)
importFrom(algebraic.dist,sampler)
importFrom(algebraic.mle,bias)
importFrom(algebraic.mle,mse)
importFrom(algebraic.mle,observed_fim)
importFrom(algebraic.mle,score_val)
importFrom(algebraic.mle,se)
importFrom(boot,boot)
importFrom(boot,boot.ci)
importFrom(generics,fit)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,cov)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,uniroot)
importFrom(stats,vcov)
importFrom(utils,modifyList)

# likelihood.model

Likelihood-based statistical inference in the Fisherian tradition for R.

## Why likelihood?

Classical statistics often jumps from data to p-values or posterior distributions. The likelihood approach takes a different path: the likelihood function *is* the complete summary of what the data say about the parameters. No priors, no long-run frequency guarantees -- just the evidence in the data.

This package makes it easy to:

- **Specify** likelihood models for standard distributions or custom observation types
- **Estimate** parameters via maximum likelihood, with analytical or numerical derivatives
- **Quantify evidence** using the likelihood function directly -- support functions, relative likelihood, and likelihood intervals
- **Handle censored data** (left, right, interval) as a natural part of the likelihood framework

## Installation

```r
# From CRAN
install.packages("likelihood.model")

# Development version from GitHub
devtools::install_github("queelius/likelihood.model")
```

## Quick start

### Fit a distribution

```r
library(likelihood.model)

# Any R distribution works: "norm", "weibull", "exp", "gamma", ...
model <- likelihood_name("weibull", ob_col = "time", censor_col = "status")
mle <- fit(model)(df, par = c(shape = 1, scale = 1))

coef(mle)      # parameter estimates
se(mle)        # standard errors
confint(mle)   # confidence intervals
```

### Handle censored data properly

Ignoring censoring biases your estimates. `likelihood.model` handles it correctly:

```r
df <- data.frame(
  time   = c(1.2, 3.4, 5.0, 2.1, 5.0),
  status = c("exact", "exact", "right", "exact", "right")
)

model <- likelihood_name("weibull", ob_col = "time", censor_col = "status")
mle <- fit(model)(df, par = c(shape = 1, scale = 1))
```

Right-censored observations contribute through the survival function rather than the density -- the likelihood framework handles this seamlessly.

### Fisherian inference

Instead of confidence intervals that rely on repeated-sampling arguments, likelihood intervals identify parameter values well-supported by the data:

```r
# Likelihood interval: {theta : L(theta)/L(theta_hat) >= 1/8}
li <- likelihood_interval(mle, df, model, k = 8)

# How well does the data support theta1 over theta2?
evidence(model, df, theta1 = c(2, 3), theta2 = c(1, 5))
```

## Three ways to build a model

| Approach | Use when | Example |
|----------|----------|---------|
| `likelihood_name()` | Standard R distribution | `likelihood_name("norm", "x")` |
| Specialized model | You need analytical derivatives or closed-form MLE | `weibull_uncensored("x")`, `exponential_lifetime("t")` |
| `likelihood_contr_model` | Heterogeneous observation types (mixed exact/censored) | Custom likelihood contributions |

### `likelihood_name()` -- the quick path

Wraps any R distribution that has `d<name>` and `p<name>` functions. Supports exact, left-censored, right-censored, and interval-censored observations automatically.

```r
model <- likelihood_name("norm", ob_col = "x", censor_col = "censor")
```

### Specialized models -- the fast path

When you have analytical derivatives, estimation is 10-100x faster than numerical differentiation:

```r
# Weibull with hand-derived score and Hessian
model <- weibull_uncensored("x")

# Exponential with closed-form MLE (bypasses optim entirely)
model <- exponential_lifetime("t", censor_col = "status")
mle <- fit(model)(df)  # no initial guess needed
```

### `likelihood_contr_model` -- the flexible path

For complex models with different observation types, define type-specific log-likelihood contributions:

```r
model <- likelihood_contr_model$new(
  obs_type = function(df) df$type,
  logliks = list(exact = loglik_exact, censored = loglik_censored)
)
```

## The likelihood calculus

Every likelihood model provides a consistent set of operations:

```r
loglik(model)       # log-likelihood function
score(model)        # score function (gradient)
hess_loglik(model)  # Hessian matrix
fim(model)          # Fisher information matrix
```

If you provide only `loglik()`, the package computes `score()` and `hess_loglik()` via numerical differentiation (Richardson extrapolation). Provide analytical versions for better speed and accuracy.

## Inference toolkit

### Maximum likelihood estimation

```r
mle <- fit(model)(df, par = c(1, 1))
coef(mle)           # point estimates
vcov(mle)           # variance-covariance matrix
confint(mle)        # Wald confidence intervals
summary(mle)        # full summary
```

### Bootstrap

```r
boot <- sampler(model, df = df, par = c(1, 1))(n = 1000)
se(boot)            # bootstrap standard errors
confint(boot)       # bootstrap confidence intervals (BCa)
bias(boot)          # bootstrap bias estimate
```

### Model comparison

```r
lrt(null_model, alt_model, df,            # likelihood ratio test
    null_par = p0, alt_par = p1, dof = 1)
aic(mle)                                   # Akaike information criterion
bic(mle)                                   # Bayesian information criterion
```

### Fisherian likelihood inference

```r
support(mle, theta, df, model)             # S(theta) = logL(theta) - logL(theta_hat)
relative_likelihood(mle, theta, df, model) # R(theta) = L(theta) / L(theta_hat)
likelihood_interval(mle, df, model, k = 8) # {theta : R(theta) >= 1/k}
profile_loglik(mle, df, model, param = 1)  # profile out nuisance parameters
evidence(model, df, theta1, theta2)        # log-likelihood ratio
```

## Ecosystem

`likelihood.model` interoperates with two companion packages:

- **[algebraic.mle](https://github.com/queelius/algebraic.mle)** -- MLE result objects with a unified interface (`params`, `rmap`, `marginal`)
- **[algebraic.dist](https://github.com/queelius/algebraic.dist)** -- probability distribution objects for simulation and analysis

## Learn more

- [Getting started](https://queelius.github.io/likelihood.model/articles/getting-started.html) -- worked examples from basic MLE to Fisherian inference
- [Named distributions](https://queelius.github.io/likelihood.model/articles/likelihood-name-model.html) -- censoring, interval data, model comparison
- [Likelihood contributions](https://queelius.github.io/likelihood.model/articles/likelihood-contributions.html) -- custom models for series systems and mixed observation types
- [Exponential lifetime](https://queelius.github.io/likelihood.model/articles/exponential-lifetime.html) -- closed-form MLE, analytical derivatives, Monte Carlo validation
- [algebraic.mle integration](https://queelius.github.io/likelihood.model/articles/algebraic-mle-integration.html) -- the three-package ecosystem
- [Reference](https://queelius.github.io/likelihood.model/reference/) -- full API documentation

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)

## ----load, message=FALSE, warning=FALSE---------------------------------------
library(likelihood.model)

## ----uncensored---------------------------------------------------------------
set.seed(42)
true_lambda <- 2.5
df <- data.frame(t = rexp(200, rate = true_lambda))

model <- exponential_lifetime("t")
assumptions(model)

## ----fit-uncensored-----------------------------------------------------------
mle <- fit(model)(df)

cat("MLE:", coef(mle), "(true:", true_lambda, ")\n")
cat("SE:", se(mle), "\n")
cat("95% CI:", confint(mle)[1, ], "\n")
cat("Score at MLE:", score_val(mle), "(exactly zero by construction)\n")

## ----verify-mle---------------------------------------------------------------
n <- nrow(df)
total_time <- sum(df$t)
manual_mle <- n / total_time

cat("fit() result:   ", coef(mle), "\n")
cat("Manual n/T:     ", manual_mle, "\n")
cat("Match:", all.equal(unname(coef(mle)), manual_mle), "\n")

## ----censored-data------------------------------------------------------------
set.seed(42)
true_lambda <- 2.0
censor_time <- 0.5

# Generate latent failure times
x <- rexp(300, rate = true_lambda)
censored <- x > censor_time

df_cens <- data.frame(
  t = pmin(x, censor_time),
  status = ifelse(censored, "right", "exact")
)

cat("Sample size:", nrow(df_cens), "\n")
cat("Censoring rate:", round(mean(censored) * 100, 1), "%\n")
cat("Exact observations (d):", sum(!censored), "\n")
cat("Total time (T):", round(sum(df_cens$t), 2), "\n")

## ----fit-censored-------------------------------------------------------------
model_cens <- exponential_lifetime("t", censor_col = "status")
assumptions(model_cens)

mle_cens <- fit(model_cens)(df_cens)

cat("\nMLE:", coef(mle_cens), "(true:", true_lambda, ")\n")
cat("SE:", se(mle_cens), "\n")
cat("95% CI:", confint(mle_cens)[1, ], "\n")

## ----censoring-bias-----------------------------------------------------------
# Wrong: treat all observations as exact
model_wrong <- exponential_lifetime("t")
mle_wrong <- fit(model_wrong)(df_cens)

cat("Comparison:\n")
cat("  True lambda:          ", true_lambda, "\n")
cat("  MLE (correct):        ", coef(mle_cens), "\n")
cat("  MLE (ignoring censor):", coef(mle_wrong), "\n")
cat("\nIgnoring censoring overestimates the failure rate!\n")

## ----derivatives--------------------------------------------------------------
set.seed(99)
df_small <- data.frame(t = rexp(50, rate = 3))
model_small <- exponential_lifetime("t")
lambda_test <- 2.5

# Analytical score
score_fn <- score(model_small)
analytical_score <- score_fn(df_small, lambda_test)

# Numerical score (via numDeriv)
ll_fn <- loglik(model_small)
numerical_score <- numDeriv::grad(
  function(p) ll_fn(df_small, p), lambda_test
)

cat("Score at lambda =", lambda_test, ":\n")
cat("  Analytical:", analytical_score, "\n")
cat("  Numerical: ", numerical_score, "\n")
cat("  Match:", all.equal(unname(analytical_score), numerical_score), "\n")

# Analytical Hessian
hess_fn <- hess_loglik(model_small)
analytical_hess <- hess_fn(df_small, lambda_test)

numerical_hess <- numDeriv::hessian(
  function(p) ll_fn(df_small, p), lambda_test
)

cat("\nHessian at lambda =", lambda_test, ":\n")
cat("  Analytical:", analytical_hess[1, 1], "\n")
cat("  Numerical: ", numerical_hess[1, 1], "\n")
cat("  Match:", all.equal(analytical_hess[1, 1], numerical_hess[1, 1]), "\n")

## ----fim----------------------------------------------------------------------
fim_fn <- fim(model_small)
n_obs <- nrow(df_small)
lambda_hat <- coef(fit(model_small)(df_small))

fim_analytical <- fim_fn(lambda_hat, n_obs)

cat("FIM at MLE (lambda =", lambda_hat, "):\n")
cat("  Analytical n/lambda^2:", n_obs / lambda_hat^2, "\n")
cat("  fim() result:         ", fim_analytical[1, 1], "\n")
cat("  Match:", all.equal(n_obs / unname(lambda_hat)^2, fim_analytical[1, 1]), "\n")

## ----fisherian----------------------------------------------------------------
set.seed(42)
df_fish <- data.frame(t = rexp(100, rate = 2.0))
model_fish <- exponential_lifetime("t")
mle_fish <- fit(model_fish)(df_fish)

# Support function: S(lambda) = logL(lambda) - logL(lambda_hat)
# Support at MLE is always 0
s_mle <- support(mle_fish, coef(mle_fish), df_fish, model_fish)
cat("Support at MLE:", s_mle, "\n")

# At a different value, support is negative
s_alt <- support(mle_fish, c(lambda = 1.5), df_fish, model_fish)
cat("Support at lambda=1.5:", s_alt, "\n")

# Relative likelihood: R(lambda) = L(lambda)/L(lambda_hat) = exp(S)
rl_alt <- relative_likelihood(mle_fish, c(lambda = 1.5), df_fish, model_fish)
cat("Relative likelihood at lambda=1.5:", rl_alt, "\n")

## ----likelihood-interval------------------------------------------------------
li <- likelihood_interval(mle_fish, df_fish, model_fish, k = 8)
print(li)

cat("\nWald 95% CI for comparison:\n")
print(confint(mle_fish))

## ----profile, fig.width=6, fig.height=4---------------------------------------
prof <- profile_loglik(mle_fish, df_fish, model_fish, param = 1, n_grid = 100)

plot(prof$lambda, prof$support, type = "l", lwd = 2,
     xlab = expression(lambda), ylab = "Support",
     main = "Profile Support Function")
abline(h = -log(8), lty = 2, col = "red")
abline(v = coef(mle_fish), lty = 3, col = "blue")
legend("topright",
       legend = c("Support S(lambda)", "1/8 cutoff", "MLE"),
       lty = c(1, 2, 3), col = c("black", "red", "blue"), lwd = c(2, 1, 1))

## ----monte-carlo, cache=TRUE--------------------------------------------------
set.seed(42)
true_lambda <- 3.0
n_obs <- 100
n_sim <- 1000

model_mc <- exponential_lifetime("t")
gen <- rdata(model_mc)

# Simulate n_sim datasets and fit each
mle_vals <- replicate(n_sim, {
  sim_df <- gen(true_lambda, n_obs)
  coef(fit(model_mc)(sim_df))
})

cat("Monte Carlo results (", n_sim, "simulations, n=", n_obs, "):\n")
cat("  True lambda:       ", true_lambda, "\n")
cat("  Mean of MLEs:      ", mean(mle_vals), "\n")
cat("  Bias:              ", mean(mle_vals) - true_lambda, "\n")
cat("  Empirical SE:      ", sd(mle_vals), "\n")
cat("  Theoretical SE:    ", true_lambda / sqrt(n_obs), "\n")
cat("  (SE = lambda/sqrt(n) from Fisher information)\n")

## ----session------------------------------------------------------------------
sessionInfo()

## ----cleanup, include=FALSE---------------------------------------------------
options(old_opts)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)

## ----install, eval=FALSE------------------------------------------------------
# # Install from GitHub
# if (!require(devtools)) install.packages("devtools")
# devtools::install_github("queelius/likelihood.model")

## ----load, message=FALSE, warning=FALSE---------------------------------------
library(likelihood.model)

## ----exponential-example------------------------------------------------------
# Generate exponential survival data
set.seed(99)
df_exp <- data.frame(t = rexp(200, rate = 3.0))

# Create model and fit -- no initial guess needed!
model_exp <- exponential_lifetime("t")

# View model assumptions
assumptions(model_exp)

# Fit the model
mle_exp <- fit(model_exp)(df_exp)

# View results
summary(mle_exp)

## ----exponential-results------------------------------------------------------
# Parameter estimates
cat("Estimated parameters:\n")
print(coef(mle_exp))

# Confidence intervals (Wald-based)
cat("\n95% Confidence Intervals:\n")
print(confint(mle_exp))

# Standard errors
cat("\nStandard Errors:\n")
print(se(mle_exp))

# Log-likelihood value
cat("\nLog-likelihood:", as.numeric(logLik(mle_exp)), "\n")

# AIC for model selection
cat("AIC:", AIC(mle_exp), "\n")

# The score at the MLE is exactly zero (by construction)
cat("Score at MLE:", score_val(mle_exp), "\n")

## ----exponential-censored-----------------------------------------------------
# Generate data with right-censoring at time 0.5
set.seed(99)
true_lambda <- 3.0
x <- rexp(200, rate = true_lambda)
censored <- x > 0.5
df_cens <- data.frame(
  t = pmin(x, 0.5),
  status = ifelse(censored, "right", "exact")
)

cat("Censoring rate:", mean(censored) * 100, "%\n")

model_cens <- exponential_lifetime("t", censor_col = "status")
mle_cens <- fit(model_cens)(df_cens)

cat("MLE (censored):", coef(mle_cens), "(true:", true_lambda, ")\n")
cat("95% CI:", confint(mle_cens)[1, ], "\n")

## ----score-check--------------------------------------------------------------
model <- exponential_lifetime("t")
df <- data.frame(t = rexp(100, rate = 2.0))

# Fit MLE
mle <- fit(model)(df)

# Evaluate score at MLE
score_func <- score(model)
score_at_mle <- score_func(df, coef(mle))

cat("Score at MLE (should be near zero):\n")
print(score_at_mle)

# The score is also stored in the MLE object
cat("\nScore stored in MLE object:\n")
print(score_val(mle))

## ----fisherian-example--------------------------------------------------------
# Fit a model
model <- exponential_lifetime("t")
df <- data.frame(t = rexp(200, rate = 2.0))
mle <- fit(model)(df)

# Support function: log relative likelihood
# S(theta) = logL(theta) - logL(theta_hat)
# Support at MLE is always 0
s_at_mle <- support(mle, coef(mle), df, model)
cat("Support at MLE:", s_at_mle, "\n")

# Support at alternative parameter values (negative = less supported)
s_alt <- support(mle, c(lambda = 1.0), df, model)
cat("Support at lambda=1.0:", s_alt, "\n")

# Relative likelihood = exp(support)
rl <- relative_likelihood(mle, c(lambda = 1.0), df, model)
cat("Relative likelihood at lambda=1.0:", rl, "\n")

## ----likelihood-intervals-----------------------------------------------------
# Compute 1/8 likelihood interval (roughly equivalent to 95% CI)
# This is the set of theta where R(theta) >= 1/8
li <- likelihood_interval(mle, df, model, k = 8)
print(li)

# Compare with Wald confidence interval
cat("\nWald 95% CI:\n")
print(confint(mle))

## ----evidence-example---------------------------------------------------------
model <- exponential_lifetime("t")
set.seed(42)
df <- data.frame(t = rexp(100, rate = 2.0))

# Evidence for lambda=2 vs lambda=1
ev <- evidence(model, df, c(lambda = 2), c(lambda = 1))
cat("Evidence for lambda=2 vs lambda=1:", ev, "\n")

# Positive evidence supports the first hypothesis
if (ev > log(8)) {
  cat("Strong evidence favoring lambda=2\n")
}

## ----bootstrap-example, cache=TRUE--------------------------------------------
model <- exponential_lifetime("t")
set.seed(42)
df <- data.frame(t = rexp(100, rate = 2.0))

# Create bootstrap sampler
boot_sampler <- sampler(model, df = df, par = c(lambda = 2))

# Generate bootstrap samples (100 replicates for speed)
boot_result <- boot_sampler(n = 100)

# Compare asymptotic vs bootstrap standard errors
mle <- fit(model)(df)
asymp_se <- se(mle)
boot_se <- se(boot_result)

cat("Standard Error Comparison:\n")
cat("           Asymptotic  Bootstrap\n")
cat(sprintf("Lambda:    %10.4f  %9.4f\n", asymp_se[1], boot_se[1]))

# Bootstrap bias estimate
cat("\nBootstrap Bias Estimate:\n")
print(bias(boot_result))

## ----session------------------------------------------------------------------
sessionInfo()

## ----cleanup, include=FALSE---------------------------------------------------
options(old_opts)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)

## ----load, message=FALSE, warning=FALSE---------------------------------------
library(likelihood.model)
library(algebraic.mle)

## ----basic-fit----------------------------------------------------------------
set.seed(42)
true_lambda <- 2.0
n <- 200
df <- data.frame(t = rexp(n, rate = true_lambda))

model <- exponential_lifetime("t")
mle_result <- fit(model)(df)

# algebraic.mle generics -- these work because fisher_mle inherits from mle
params(mle_result)
nparams(mle_result)

## ----observed-fim-------------------------------------------------------------
observed_fim(mle_result)

## ----fim-pd-------------------------------------------------------------------
cat("Observed FIM:", observed_fim(mle_result)[1,1], "\n")
cat("Positive:", observed_fim(mle_result)[1,1] > 0, "\n")

## ----vcov-vs-fim--------------------------------------------------------------
vcov(mle_result)
1 / observed_fim(mle_result)

## ----rmap---------------------------------------------------------------------
# Transform rate -> mean lifetime
mean_life_mle <- rmap(mle_result, function(p) {
  c(mean_lifetime = 1 / p[1])
})

params(mean_life_mle)
se(mean_life_mle)

## ----rmap-compare-------------------------------------------------------------
true_mean <- 1 / true_lambda
cat("True mean lifetime:", true_mean, "\n")
cat("Estimated mean lifetime:", params(mean_life_mle), "\n")
cat("95% CI:", confint(mean_life_mle), "\n")

## ----rmap-multi---------------------------------------------------------------
# Derive mean, variance, and median of the exponential distribution
derived_mle <- rmap(mle_result, function(p) {
  lam <- p[1]
  c(
    mean   = 1 / lam,
    var    = 1 / lam^2,
    median = log(2) / lam
  )
})

params(derived_mle)
se(derived_mle)

## ----expectation--------------------------------------------------------------
set.seed(123)
# E[lambda^2] under the asymptotic distribution
e_lam_sq <- expectation(mle_result, function(p) p[1]^2,
                         control = list(n = 10000L))
cat("E[lambda^2]:", e_lam_sq, "\n")
cat("lambda^2 at MLE:", params(mle_result)[1]^2, "\n")

# Probability that lambda > 1.5 (under asymptotic distribution)
pr_lam_gt <- expectation(mle_result, function(p) as.numeric(p[1] > 1.5),
                          control = list(n = 10000L))
cat("P(lambda > 1.5):", pr_lam_gt, "\n")

## ----mse----------------------------------------------------------------------
mse(mle_result)
all.equal(mse(mle_result), vcov(mle_result))

## ----bootstrap, cache=TRUE----------------------------------------------------
set.seed(42)
boot_sampler <- sampler(model, df = df, par = c(lambda = 2))
boot_result <- boot_sampler(n = 200)

# Same algebraic.mle generics work
params(boot_result)
nparams(boot_result)
se(boot_result)
bias(boot_result)

## ----boot-compare-------------------------------------------------------------
cat("Asymptotic 95% CI:\n")
confint(mle_result)

cat("\nBootstrap percentile 95% CI:\n")
confint(boot_result, type = "perc")

## ----dist-check, include=FALSE------------------------------------------------
has_dist <- requireNamespace("algebraic.dist", quietly = TRUE)

## ----dist-section, eval=has_dist----------------------------------------------
library(algebraic.dist)

## ----dist-compare, eval=has_dist----------------------------------------------
set.seed(42)

cat("MLE:", params(mle_result), "\n")
cat("SE:", se(mle_result), "\n")

# Theoretical asymptotic distribution of the MLE
asymp_var <- true_lambda^2 / n
asymp_dist <- normal(mu = true_lambda, var = asymp_var)
cat("\nTheoretical asymptotic distribution:\n")
cat("  Mean:", params(asymp_dist)[1], "\n")
cat("  Variance:", params(asymp_dist)[2], "\n")

# Compare: sample from the MLE's estimated distribution
mle_sampler <- sampler(mle_result)
set.seed(1)
mle_samples <- mle_sampler(5000)

# vs. sample from the theoretical distribution
dist_sampler <- sampler(asymp_dist)
set.seed(1)
dist_samples <- dist_sampler(5000)

cat("\nMLE sampler mean:", mean(mle_samples), "\n")
cat("Theoretical sampler mean:", mean(dist_samples), "\n")
cat("MLE sampler sd:", sd(mle_samples), "\n")
cat("Theoretical sampler sd:", sd(dist_samples), "\n")

## ----cleanup, include=FALSE---------------------------------------------------
options(old_opts)

#' @title likelihood.model: Likelihood-Based Inference in the Fisherian Tradition
#' @description
#' The `likelihood.model` package provides a framework for likelihood-based
#' inference. The package is organized in layers:
#'
#' **Core Concept** (`core-generics.R`):
#' The `likelihood_model` "concept" -- an abstract interface that any model
#' can implement. At minimum, implement [loglik()]. Optionally provide
#' [score()] and [hess_loglik()] for analytical derivatives; defaults use
#' numerical differentiation via `numDeriv`.
#'
#' **Core Infrastructure**:
#' \itemize{
#'   \item [fisher_mle] / [fisher_boot]: Result objects from MLE fitting, with
#'     methods for [coef()], [vcov()], [confint()], [se()], [stats::AIC()], [stats::BIC()],
#'     [summary()].
#'   \item [fit()]: Default MLE solver using [optim()]. Models can override this
#'     with closed-form solutions (see [exponential_lifetime] for an example).
#'   \item Fisherian inference: [support()], [relative_likelihood()],
#'     [likelihood_interval()], [profile_loglik()], [evidence()] -- pure
#'     likelihood-based inference without probability statements.
#'   \item [lrt()]: Likelihood ratio test for nested models.
#' }
#'
#' **Example Implementation**:
#' [exponential_lifetime]: Exponential with right-censoring support.
#' Demonstrates closed-form MLE (no optim needed), analytical FIM,
#' and [rdata()] for Monte Carlo validation.
#'
#' For contribution-based models with heterogeneous observation types,
#' see the companion package \pkg{likelihood.contr}.
#'
"_PACKAGE"

#' Check if an object is a likelihood model
#'
#' Tests whether an object satisfies the `likelihood_model` concept by
#' checking if `"likelihood_model"` is in its class hierarchy. To be a
#' likelihood model, at a minimum the object must implement [loglik()].
#' For optimal results, it may also provide [score()] and [hess_loglik()].
#'
#' @param x An object to test
#' @return Logical indicating whether x is a likelihood model
#' @export
is_likelihood_model <- function(x) {
  inherits(x, "likelihood_model")
}

#' Log-likelihood method
#'
#' This function returns the log-likelihood function of a model
#'
#' @param model The likelihood model
#' @param ... Additional arguments
#' @return A log-likelihood function to compute the log-likelihood given a data
#' frame and parameters.
#' @export
loglik <- function(model, ...) {
  UseMethod("loglik")
}

#' Default loglik method
#'
#' Provides a clear error when a model class does not implement `loglik`.
#'
#' @param model A likelihood model
#' @param ... Additional arguments
#' @return Never returns; always errors.
#' @export
loglik.likelihood_model <- function(model, ...) {
  stop(sprintf(
    "Class '%s' must implement loglik() to satisfy the likelihood_model concept",
    class(model)[1]
  ))
}

#' Score method
#'
#' This function returns the score function of a model
#'
#' @param model The likelihood model
#' @param ... Additional arguments
#'
#' @return A function to compute the score given a data frame and parameters
#' @export
score <- function(model, ...) {
  UseMethod("score")
}

#' Default score method
#'
#' In case a `score` method is not provided, this function will be used.
#' It computes the score by numerical differentiation of the log-likelihood.
#' @param model The likelihood model
#' @param control Custom arguments to pass to `numDeriv::grad`.
#' @param ... Additional arguments (to pass into `loglik`)
#' @return A function to compute the score given a data frame and parameters
#' @importFrom numDeriv grad
#' @export
score.likelihood_model <- function(model, control = list(), ...) {
  defaults <- list(
    method = "Richardson",
    side = NULL,
    method.args = list(r = 6)
  )
  control <- modifyList(defaults, control)
  ll <- loglik(model, ...)
  function(df, par, ...) {
    stopifnot(!is.null(df), !is.null(par))
    grad(
      func = ll,
      x = par,
      method = control$method,
      side = control$side,
      method.args = control$method.args,
      df = df,
      ...
    )
  }
}

#' Hessian of log-likelihood method
#'
#' This function returns the hessian of the log-likelihood function of a model
#'
#' @param model The likelihood model
#' @param ... Additional arguments
#'
#' @return A function to compute the hessian of the log-likelihood given a
#' data frame and parameters
#' @export
hess_loglik <- function(model, ...) {
  UseMethod("hess_loglik")
}

#' Default method to compute the hessian of the log-likelihood.
#'
#' In case a `hess_loglik` method is not provided, this function will be
#' used.
#' It computes the hessian of the log-likelihood function using numerical
#' differentiation.
#'
#' @param model The likelihood model
#' @param ... Additional arguments (to pass into `loglik`)
#' @param control Custom arguments to pass to `numDeriv::hessian`.
#' @return A function(df, par, ...) that computes the Hessian matrix of the
#'   log-likelihood evaluated at `par` given data `df`.
#' @importFrom numDeriv hessian
#' @export
hess_loglik.likelihood_model <- function(model, control = list(), ...) {
  defaults <- list(
    method = "Richardson",
    side = NULL,
    method.args = list(r = 6)
  )
  control <- modifyList(defaults, control)
  ll <- loglik(model, ...)
  function(df, par, ...) {
    stopifnot(!is.null(df), !is.null(par))
    hessian(
      func = ll,
      x = par,
      method = control$method,
      side = control$side,
      method.args = control$method.args,
      df = df,
      ...
    )
  }
}

#' Fisher information matrix method
#'
#' This function calculates the Fisher information matrix (FIM), an expectation
#' over the data-generating process (DGP). The FIM is a crucial concept in
#' statistics because it provides information about the precision of estimates
#' and the amount of information that data carries about an unknown parameter.
#'
#' FIM is a function of the parameters, and is used to compute the standard
#' errors of the parameters. It is also used to compute the covariance matrix of
#' the parameters, which is in turn used to compute standard errors of the
#' parameters.
#'
#' Additionally, FIM is used to compute the Cramer-Rao lower bound (CRLB),
#' the inverse of the FIM. CRLB represents the lower limit of the variance that
#' an unbiased estimator can attain. This is used to compute the asymptotic
#' relative efficiency (ARE) of an estimator of the parameters, which is the
#' ratio of the variance of the estimator to the CRLB.
#'
#' The function computes FIM(x)(theta), the FIM of the likelihood model `x`, is
#' based on the following formulas:
#'
#'     FIM(x)(theta) = E[-loglik_hessian(x)(ob,theta)]
#'     FIM(x)(theta) = E[score(x)(ob,theta) %*% t(score(x)(ob,theta))]
#'
#' where the expectation is taken with respect to ob ~ DGP. The first formula
#' is the expected hessian of the log-likelihood function, and the second formula
#' is the expected outer product of the score function. The two formulas are
#' equivalent.
#'
#' @param model A likelihood model
#' @param ... Additional arguments
#' @return Function that computes the FIM given a parameter vector and sample size
#' @export
fim <- function(model, ...) {
  UseMethod("fim")
}

#' Default FIM method using Monte Carlo estimation
#'
#' Computes the Fisher information matrix by Monte Carlo simulation using
#' the negative expected Hessian approach. For each of `n_samples` replicates,
#' generates a single-observation dataset via `rdata`, computes
#' `-hess_loglik(single_obs, theta)`, and averages. The result is
#' `n_obs * I_1(theta)` where `I_1(theta)` is the per-observation FIM.
#'
#' This default requires the model to implement `rdata` and `hess_loglik`
#' (or `loglik`, since `hess_loglik` falls back to numerical differentiation).
#'
#' Extra arguments via `...` to the returned FIM function are forwarded
#' to `rdata` only (i.e., they parameterize the data-generating process:
#' censoring time, masking probability, observation functor, etc.) and
#' are NOT passed to the likelihood evaluator `hess_loglik`. This honors
#' the separation between the DGP layer and the likelihood layer: the
#' likelihood is computed on data, not on the DGP knobs that produced it.
#' Forwarding DGP kwargs into the likelihood layer was both an axiom
#' violation (under masking condition C3 the likelihood does not depend
#' on the masking probability) and a partial-matching footgun (a kwarg
#' named `p` would collide with the formal `par`).
#'
#' @param model A likelihood model
#' @param ... Additional arguments (currently unused; reserved for future
#'   extension at closure-construction time)
#' @return Function that takes (theta, n_obs, n_samples, ...) and returns
#'   FIM matrix. The inner `...` is forwarded to `rdata` only.
#' @export
fim.likelihood_model <- function(model, ...) {
  hess_fn <- hess_loglik(model, ...)
  rdata_fn <- rdata(model)

  function(theta, n_obs, n_samples = 1000, ...) {
    p <- length(theta)
    fim_sum <- matrix(0, nrow = p, ncol = p)
    n_ok <- 0L

    for (b in seq_len(n_samples)) {
      sample <- tryCatch({
        data_b <- rdata_fn(theta, n = 1, ...)
        H <- -hess_fn(data_b, theta)
        if (any(!is.finite(H))) list(ok = FALSE) else list(ok = TRUE, val = H)
      }, error = function(e) list(ok = FALSE))

      if (sample$ok) {
        fim_sum <- fim_sum + sample$val
        n_ok <- n_ok + 1L
      }
    }

    if (n_ok == 0L) {
      stop("All Monte Carlo samples failed in FIM estimation")
    }
    if (n_ok < n_samples) {
      warning(sprintf("FIM estimation: %d of %d samples failed",
                      n_samples - n_ok, n_samples))
    }

    fim_mat <- n_obs * fim_sum / n_ok
    if (!is.null(names(theta))) {
      dimnames(fim_mat) <- list(names(theta), names(theta))
    }
    fim_mat
  }
}

#' Random data generation method
#'
#' Returns a function that generates random data from the model's
#' data-generating process (DGP) at a given parameter value.
#'
#' This is used by the default `fim` method for Monte Carlo estimation
#' of the Fisher information matrix.
#'
#' @param model A likelihood model
#' @param ... Additional arguments
#' @return Function that takes (theta, n, ...) and returns a data frame
#' @export
rdata <- function(model, ...) {
  UseMethod("rdata")
}

#' Default rdata method
#'
#' Provides a clear error when a model class does not implement `rdata`.
#' The `rdata` method is required by the default [fim.likelihood_model()]
#' for Monte Carlo FIM estimation.
#'
#' @param model A likelihood model
#' @param ... Additional arguments
#' @return Never returns; always errors.
#' @export
rdata.likelihood_model <- function(model, ...) {
  stop(sprintf(
    "Class '%s' does not implement rdata(); needed by fim() for Monte Carlo estimation",
    class(model)[1]
  ))
}

#' Retrieve the assumptions the likelihood model makes about the data.
#'
#' @param model The likelihood model
#' @param ... Additional arguments
#'
#' @return A list of assumptions
#' @export
assumptions <- function(model, ...) {
  UseMethod("assumptions")
}

#' Print method for likelihood models
#'
#' @param x A likelihood model
#' @param show.loglik Logical; if TRUE, print the log-likelihood function
#' @param ... Additional arguments (ignored)
#' @return The likelihood model object, invisibly
#' @export
print.likelihood_model <- function(x, show.loglik = FALSE, ...) {
  cat("Likelihood model:", class(x)[1], "\n")
  cat("---------------\n")
  if (!is.null(x$ob_col)) cat("Observation column:", x$ob_col, "\n")
  if (!is.null(x$censor_col)) cat("Censoring column:", x$censor_col, "\n")
  cat("Assumptions:\n")
  for (assumption in assumptions(x)) {
    cat(" -", trimws(assumption), "\n")
  }
  if (show.loglik) {
    cat("---------------\n")
    cat("Log-likelihood function:\n")
    print(loglik(x))
  }
  invisible(x)
}

library(testthat)
library(likelihood.model)

# Helper function to get parameter estimates from MLE result
# fisher_mle stores estimates in $par, accessible via coef()
get_mle_params <- function(mle_result) {
  # Use coef() for fisher_mle objects
  result <- coef(mle_result)
  if (!is.null(result)) {
    return(result)
  }
  # Fallback: try $par directly
  if (!is.null(mle_result$par)) {
    return(mle_result$par)
  }
  return(NULL)
}

# =============================================================================
# IS_LIKELIHOOD_MODEL TESTS
# =============================================================================

test_that("is_likelihood_model correctly identifies likelihood models", {
  # exponential_lifetime is a likelihood model
  model_exp <- exponential_lifetime("t")
  expect_true(is_likelihood_model(model_exp))

  # Non-likelihood model objects return FALSE
  expect_false(is_likelihood_model(list()))
  expect_false(is_likelihood_model(data.frame()))
  expect_false(is_likelihood_model(NULL))
  expect_false(is_likelihood_model("not a model"))
  expect_false(is_likelihood_model(42))
})

# =============================================================================
# FIT.LIKELIHOOD_MODEL TESTS
# =============================================================================

test_that("fit.likelihood_model finds correct MLE for exponential data", {
  set.seed(999)
  true_rate <- 2.0
  n <- 200
  df <- data.frame(t = rexp(n, rate = true_rate))

  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  estimated <- get_mle_params(result)

  # Closed-form MLE: lambda_hat = n / sum(t)
  expect_equal(unname(estimated), n / sum(df$t), tolerance = 1e-12)

  # Should be close to true value
  expect_lt(abs(estimated[1] - true_rate), 0.5)
})

# =============================================================================
# SAMPLER.LIKELIHOOD_MODEL TESTS
# =============================================================================

test_that("sampler.likelihood_model generates bootstrap samples", {
  skip_if_not_installed("boot")

  set.seed(1002)
  df <- data.frame(t = rexp(200, rate = 2))

  model <- exponential_lifetime("t")

  # Create sampler
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))

  # Generate bootstrap samples
  boot_result <- model_sampler(n = 10)

  # Should return a fisher_boot object
  expect_true(inherits(boot_result, "fisher_boot"))
  expect_true(inherits(boot_result, "fisher_mle"))

  # Bootstrap estimates should be stored in replicates
  expect_equal(nrow(boot_result$replicates), 10)
  expect_equal(ncol(boot_result$replicates), 1)  # rate only

  # coef() should return original MLE
  expect_equal(length(coef(boot_result)), 1)
})

test_that("sampler.likelihood_model bootstrap distribution is centered near MLE", {
  skip_if_not_installed("boot")

  set.seed(1003)
  true_rate <- 2.0
  df <- data.frame(t = rexp(200, rate = true_rate))

  model <- exponential_lifetime("t")

  # First fit to get MLE
  mle_result <- fit(model)(df)
  mle_params <- get_mle_params(mle_result)

  # Create sampler and generate bootstrap samples
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_result <- model_sampler(n = 50)

  # Bootstrap mean should be close to MLE
  boot_mean <- colMeans(boot_result$replicates)
  expect_equal(unname(boot_mean[1]), unname(mle_params[1]), tolerance = 0.5)
})

# =============================================================================
# LRT (LIKELIHOOD RATIO TEST) TESTS
# =============================================================================

test_that("lrt requires likelihood models as inputs", {
  df <- data.frame(t = rexp(10, rate = 1))

  expect_error(lrt(list(), exponential_lifetime("t"),
                   df, null_par = c(1), alt_par = c(2)))
})

test_that("lrt computes correct test statistic", {
  set.seed(1004)
  df <- data.frame(t = rexp(100, rate = 1))

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)

  null_par <- c(1.0)
  alt_par <- c(1.05)

  null_ll <- ll_func(df, null_par)
  alt_ll <- ll_func(df, alt_par)

  result <- lrt(model, model, df,
                null_par = null_par, alt_par = alt_par, dof = 1)

  expected_stat <- -2 * (null_ll - alt_ll)
  expect_equal(result$stat, expected_stat, tolerance = 1e-10)
  expect_equal(result$dof, 1)
  expect_true(result$p.value >= 0 && result$p.value <= 1)
})

test_that("lrt p-value is computed correctly", {
  set.seed(1005)
  df <- data.frame(t = rexp(200, rate = 2))

  model <- exponential_lifetime("t")

  null_par <- c(2.0)
  mle <- fit(model)(df)
  alt_par <- coef(mle)

  result <- lrt(model, model, df,
                null_par = null_par, alt_par = alt_par, dof = 1)

  # Test statistic should be non-negative
  expect_true(result$stat >= 0)
  # p-value should be in [0, 1]
  expect_true(result$p.value >= 0 && result$p.value <= 1)
})

test_that("lrt auto-calculates degrees of freedom", {
  set.seed(1006)
  df <- data.frame(t = rexp(50, rate = 2))

  model <- exponential_lifetime("t")

  result <- lrt(model, model, df,
                null_par = c(1),
                alt_par = c(1, 0.5),
                dof = NULL)

  expect_equal(result$dof, 1)
  expect_true(!is.null(result$stat))
  expect_true(!is.null(result$p.value))
})

# =============================================================================
# EDGE CASES AND ERROR HANDLING
# =============================================================================

test_that("loglik handles data with single observation", {
  df <- data.frame(t = 5.0)

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)

  expected_ll <- dexp(5.0, rate = 2, log = TRUE)
  computed_ll <- ll_func(df, c(2))

  expect_equal(computed_ll, expected_ll, tolerance = 1e-10)
})

# =============================================================================
# PRINT.LIKELIHOOD_MODEL TESTS
# =============================================================================

test_that("print.likelihood_model outputs model information", {
  model <- exponential_lifetime("t")

  # Capture output
  output <- capture.output(print(model))

  # Check that key information is printed
  expect_true(any(grepl("Likelihood model", output)))
  expect_true(any(grepl("Observation column", output)))
})

test_that("print.likelihood_model with show.loglik=TRUE shows loglik function", {
  model <- exponential_lifetime("t")

  output <- capture.output(print(model, show.loglik = TRUE))
  expect_true(any(grepl("Log-likelihood function", output)))
})

test_that("print.likelihood_model returns model invisibly", {
  model <- exponential_lifetime("t")

  result <- print(model)
  expect_identical(result, model)
})

# =============================================================================
# LRT WITH NULL DOF TESTS
# =============================================================================

test_that("lrt computes dof automatically when NULL", {
  set.seed(1014)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")

  null_par <- c(1.5)
  alt_par <- c(2.0)

  # When dof is NULL, it should be computed as length(alt_par) - length(null_par)
  result <- lrt(model, model, df,
                null_par = null_par, alt_par = alt_par, dof = NULL)

  # dof should be 0 (1 - 1)
  expect_equal(result$dof, 0)
  expect_true(!is.null(result$stat))
  expect_true(!is.null(result$p.value))
})

# =============================================================================
# FIT WITH SANN METHOD TESTS
# =============================================================================

test_that("fit.likelihood_model works with SANN method (no gradient)", {
  set.seed(1015)
  df <- data.frame(t = rexp(100, rate = 2))

  # Use the generic fit.likelihood_model (not exponential_lifetime's override)
  # by creating a mock model that delegates to exponential loglik
  mock_model <- structure(
    list(ob_col = "t", censor_col = NULL),
    class = c("mock_exp_sann", "likelihood_model")
  )
  loglik.mock_exp_sann <<- function(model, ...) {
    function(df, par, ...) {
      lambda <- par[1]
      if (lambda <= 0) return(-.Machine$double.xmax / 2)
      n <- nrow(df)
      n * log(lambda) - lambda * sum(df[[model$ob_col]])
    }
  }

  solver <- fit(mock_model)
  result <- solver(df, par = c(1), method = "SANN",
                   control = list(maxit = 1000))

  params_val <- get_mle_params(result)
  expect_true(!is.null(params_val))
  expect_true(!any(is.na(params_val)))

  # Parameters should be reasonable
  expect_lt(abs(params_val[1] - 2), 1.5)

  rm(loglik.mock_exp_sann, envir = .GlobalEnv)
})

# =============================================================================
# FIM GENERIC TESTS
# =============================================================================

test_that("fim generic dispatches correctly", {
  # fim is a generic that requires a method
  # Test that it exists and errors appropriately for objects without methods
  expect_error(fim(list()), "no applicable method")
})

# =============================================================================
# FISHER_MLE CLASS TESTS
# =============================================================================

test_that("fisher_mle creates object with correct structure", {
  result <- fisher_mle(
    par = c(mean = 0, sd = 1),
    loglik_val = -50.5,
    hessian = matrix(c(-100, 0, 0, -50), 2, 2),
    nobs = 100
  )

  expect_true(inherits(result, "fisher_mle"))
  expect_true(inherits(result, "mle_fit"))

  # Test base R generic methods

  expect_equal(coef(result), c(mean = 0, sd = 1))
  expect_true(!is.null(vcov(result)))
  expect_equal(nobs(result), 100)

  # logLik should return a logLik object
  ll <- logLik(result)
  expect_true(inherits(ll, "logLik"))
  expect_equal(as.numeric(ll), -50.5)
})

test_that("fisher_mle confint computes Wald confidence intervals", {
  result <- fisher_mle(
    par = c(mean = 5, sd = 2),
    vcov = matrix(c(0.04, 0, 0, 0.02), 2, 2),  # SE = 0.2 and ~0.14
    loglik_val = -100,
    nobs = 100
  )

  ci <- confint(result, level = 0.95)

  # Check structure
  expect_true(is.matrix(ci))
  expect_equal(nrow(ci), 2)
  expect_equal(ncol(ci), 2)

  # Intervals should be centered around estimates
  expect_lt(ci[1, 1], 5)  # lower < estimate
  expect_gt(ci[1, 2], 5)  # upper > estimate
})

test_that("fisher_mle se returns standard errors", {
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    vcov = matrix(c(0.25, 0, 0, 0.16), 2, 2),  # SE = 0.5 and 0.4
    loglik_val = -50
  )

  se_vals <- se(result)
  expect_equal(se_vals, c(0.5, 0.4))
})

test_that("fisher_mle AIC and BIC compute correctly via stats generics", {
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    loglik_val = -100,
    nobs = 50
  )

  # AIC = -2*logL + 2*k = -2*(-100) + 2*2 = 204
  expect_equal(stats::AIC(result), 204)

  # BIC = -2*logL + k*log(n) = 200 + 2*log(50)
  expected_bic <- 200 + 2 * log(50)
  expect_equal(stats::BIC(result), expected_bic)
})

test_that("fisher_mle summary produces correct output", {
  result <- fisher_mle(
    par = c(mean = 5, sd = 2),
    vcov = matrix(c(0.04, 0, 0, 0.02), 2, 2),
    loglik_val = -100,
    nobs = 100
  )

  summ <- summary(result)
  expect_true(inherits(summ, "summary_fisher_mle"))
  expect_true(!is.null(summ$coefficients))
  expect_equal(summ$loglik, -100)
  expect_equal(summ$nobs, 100)
})

test_that("fit.likelihood_model returns fisher_mle object", {
  set.seed(2001)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  expect_true(inherits(result, "fisher_mle"))
  expect_true(inherits(result, "mle_fit"))
  expect_true(result$converged)
  expect_equal(result$nobs, 100)
})

# =============================================================================
# FISHERIAN INFERENCE TESTS
# =============================================================================

test_that("support function computes log relative likelihood", {
  set.seed(2002)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")
  mle_result <- fit(model)(df)

  # Support at MLE should be 0
  s_at_mle <- support(mle_result, coef(mle_result), df, model)
  expect_equal(unname(s_at_mle), 0, tolerance = 1e-10)

  # Support at other values should be negative
  s_away <- support(mle_result, c(lambda = 0.5), df, model)
  expect_true(s_away < 0)
})

test_that("relative_likelihood computes exp(support)", {
  set.seed(2003)
  df <- data.frame(t = rexp(50, rate = 2))

  model <- exponential_lifetime("t")
  mle_result <- fit(model)(df)

  # Relative likelihood at MLE should be 1
  rl_at_mle <- relative_likelihood(mle_result, coef(mle_result), df, model)
  expect_equal(unname(rl_at_mle), 1, tolerance = 1e-10)

  # Relative likelihood elsewhere should be between 0 and 1
  rl_away <- relative_likelihood(mle_result, c(lambda = 0.5), df, model)
  expect_true(rl_away > 0 && rl_away < 1)
})

test_that("likelihood_interval works for single parameter model", {
  set.seed(2004)
  df <- data.frame(t = rexp(200, rate = 2))

  model <- exponential_lifetime("t")
  mle_result <- fit(model)(df)

  # Compute 1/8 likelihood interval
  li <- likelihood_interval(mle_result, df, model, k = 8)

  expect_true(inherits(li, "likelihood_interval"))
  expect_equal(nrow(li), 1)
  expect_equal(ncol(li), 2)

  # Interval should contain MLE
  mle_val <- unname(coef(mle_result))
  expect_true(li[1, 1] < mle_val && mle_val < li[1, 2])
})

test_that("profile_loglik computes profile for single parameter", {
  set.seed(2005)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")
  mle_result <- fit(model)(df)

  # Profile for first parameter (lambda)
  prof <- profile_loglik(mle_result, df, model, param = 1, n_grid = 20)

  expect_true(inherits(prof, "profile_loglik"))
  expect_equal(nrow(prof), 20)
  expect_true("loglik" %in% names(prof))
  expect_true("support" %in% names(prof))
  expect_true("relative_likelihood" %in% names(prof))

  # Maximum profile loglik should be near MLE
  max_idx <- which.max(prof$loglik)
  expect_equal(unname(prof[[1]][max_idx]), unname(coef(mle_result)[1]), tolerance = 0.5)
})

test_that("deviance computes correctly for fisher_mle", {
  result <- fisher_mle(
    par = c(a = 1),
    loglik_val = -50,
    nobs = 100
  )

  # Single model deviance = -2 * logL = 100
  expect_equal(deviance(result), 100)

  # Comparison deviance
  null_result <- fisher_mle(
    par = c(a = 0.5),
    loglik_val = -55,
    nobs = 100
  )

  # Deviance = 2*(logL_full - logL_null) = 2*(-50 - (-55)) = 10
  expect_equal(deviance(result, null_result), 10)
})

test_that("evidence computes log likelihood ratio", {
  set.seed(2006)
  df <- data.frame(t = rexp(50, rate = 2))

  model <- exponential_lifetime("t")

  # Evidence for theta1 vs theta2
  ev <- evidence(model, df, c(lambda = 2), c(lambda = 0.5))

  # Should strongly favor true parameters (2) over (0.5)
  expect_true(ev > 0)
  expect_true(ev > log(8))  # Strong evidence
})

# =============================================================================
# FISHER_BOOT CLASS TESTS
# =============================================================================

test_that("fisher_boot inherits from fisher_mle", {
  skip_if_not_installed("boot")

  set.seed(2007)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_result <- model_sampler(n = 20)

  expect_true(inherits(boot_result, "fisher_boot"))
  expect_true(inherits(boot_result, "fisher_mle"))
  expect_true(inherits(boot_result, "mle_fit"))

  # Base methods should work
  expect_equal(length(coef(boot_result)), 1)
  expect_true(is.matrix(vcov(boot_result)))
})

test_that("fisher_boot bias computes from bootstrap replicates", {
  skip_if_not_installed("boot")

  set.seed(2008)
  df <- data.frame(t = rexp(200, rate = 2))

  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_result <- model_sampler(n = 50)

  # Bias should be computed from replicates
  b <- bias(boot_result)
  expect_equal(length(b), 1)

  # Expected: mean(replicates) - original MLE
  expected_bias <- colMeans(boot_result$replicates) - coef(boot_result)
  expect_equal(b, expected_bias)
})

test_that("sampler generic works on fisher_mle for asymptotic sampling", {
  result <- fisher_mle(
    par = c(mean = 5, sd = 2),
    vcov = matrix(c(0.04, 0, 0, 0.02), 2, 2),
    loglik_val = -100,
    nobs = 100
  )

  samp_fn <- sampler(result)
  samples <- samp_fn(1000)

  expect_equal(dim(samples), c(1000, 2))

  # Samples should be centered near MLE
  expect_equal(mean(samples[, 1]), 5, tolerance = 0.1)
  expect_equal(mean(samples[, 2]), 2, tolerance = 0.1)
})

# =============================================================================
# EDGE CASE AND BUG FIX VALIDATION TESTS
# =============================================================================

test_that("lrt with dof=NULL computes dof from parameter lengths", {
  set.seed(3001)
  df <- data.frame(t = rexp(100, rate = 2))

  model <- exponential_lifetime("t")

  # alt_par has 2 elements, null_par has 1 => dof should be 1
  null_par <- c(2)
  alt_par <- c(2, 0.5)

  result <- lrt(model, model, df,
                null_par = null_par, alt_par = alt_par, dof = NULL)

  expect_equal(result$dof, 1)
  expect_true(is.finite(result$stat))
  expect_true(result$p.value >= 0 && result$p.value <= 1)
})

test_that("evidence returns 0 when comparing same parameters", {
  set.seed(3003)
  df <- data.frame(t = rexp(50, rate = 2))

  model <- exponential_lifetime("t")

  # Evidence for identical parameters should be exactly 0
  ev <- evidence(model, df, c(lambda = 2), c(lambda = 2))
  expect_equal(unname(ev), 0)
})

test_that("fisher_mle with NULL vcov: se returns NA, sampler errors", {
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    loglik_val = -50,
    nobs = 100
    # vcov and hessian both NULL
  )

  # se should return NA values
  se_vals <- se(result)
  expect_equal(length(se_vals), 2)
  expect_true(all(is.na(se_vals)))

  # sampler should error cleanly
  expect_error(sampler(result), "Cannot create sampler: vcov is NULL")
})

# =============================================================================
# EXPONENTIAL_LIFETIME TESTS
# =============================================================================

test_that("exponential_lifetime creates model with correct structure", {
  model <- exponential_lifetime("t")
  expect_true(is_likelihood_model(model))
  expect_equal(model$ob_col, "t")
  expect_null(model$censor_col)
  expect_true("exponential_lifetime" %in% class(model))

  model_c <- exponential_lifetime("t", censor_col = "status")
  expect_equal(model_c$censor_col, "status")
})

test_that("loglik matches manual d*log(lambda) - lambda*T", {
  set.seed(4001)
  lambda <- 2.0
  x <- rexp(100, rate = lambda)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)

  # Manual calculation
  n <- length(x)
  total_time <- sum(x)
  manual_ll <- n * log(lambda) - lambda * total_time

  expect_equal(ll_func(df, lambda), manual_ll, tolerance = 1e-12)
})

test_that("loglik matches sum of dexp(x, rate, log=TRUE)", {
  set.seed(4002)
  lambda <- 1.5
  x <- rexp(80, rate = lambda)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)

  expected_ll <- sum(dexp(x, rate = lambda, log = TRUE))
  expect_equal(ll_func(df, lambda), expected_ll, tolerance = 1e-10)
})

test_that("closed-form MLE equals d/T exactly", {
  set.seed(4003)
  lambda_true <- 3.0
  n <- 200
  x <- rexp(n, rate = lambda_true)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  # Closed-form MLE: lambda_hat = n / sum(x)
  expected_mle <- n / sum(x)
  expect_equal(unname(coef(result)), expected_mle, tolerance = 1e-12)

  # Should be close to true value
  expect_lt(abs(coef(result) - lambda_true), 0.5)

  # MLE should have converged
  expect_true(result$converged)
  expect_equal(result$nobs, n)
})

test_that("analytical score matches numDeriv::grad", {
  set.seed(4004)
  lambda <- 2.5
  x <- rexp(50, rate = lambda)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)
  score_func <- score(model)

  # Test at several parameter values
  for (lam in c(1.0, 2.0, 3.0, 5.0)) {
    analytical <- score_func(df, lam)
    numerical <- numDeriv::grad(function(p) ll_func(df, p), lam)
    expect_equal(unname(analytical), numerical, tolerance = 1e-6,
                 info = paste("Score mismatch at lambda =", lam))
  }
})

test_that("analytical hessian matches numDeriv::hessian", {
  set.seed(4005)
  lambda <- 2.0
  x <- rexp(100, rate = lambda)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  ll_func <- loglik(model)
  hess_func <- hess_loglik(model)

  for (lam in c(1.0, 2.0, 4.0)) {
    analytical <- hess_func(df, lam)
    numerical <- numDeriv::hessian(function(p) ll_func(df, p), lam)
    expect_equal(as.numeric(analytical), as.numeric(numerical), tolerance = 1e-4,
                 info = paste("Hessian mismatch at lambda =", lam))
  }
})

test_that("right-censored MLE is correct", {
  set.seed(4006)
  lambda_true <- 2.0
  n <- 200
  censor_time <- 0.4

  x_raw <- rexp(n, rate = lambda_true)
  x_obs <- pmin(x_raw, censor_time)
  status <- ifelse(x_raw > censor_time, "right", "exact")

  df <- data.frame(t = x_obs, status = status)
  model <- exponential_lifetime("t", censor_col = "status")

  result <- fit(model)(df)
  lambda_hat <- unname(coef(result))

  # Verify closed-form: d / T
  d <- sum(status == "exact")
  total_time <- sum(x_obs)
  expect_equal(lambda_hat, d / total_time, tolerance = 1e-12)

  # Should be reasonable (right-censoring still gives consistent estimator)
  expect_lt(abs(lambda_hat - lambda_true), 0.8)
})

test_that("score is exactly zero at MLE", {
  set.seed(4007)
  x <- rexp(100, rate = 1.5)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  # Score at MLE should be exactly 0 (closed-form solution)
  score_func <- score(model)
  s <- score_func(df, coef(result))
  expect_equal(unname(s), 0, tolerance = 1e-12)

  # Also stored in the MLE object
  expect_equal(unname(score_val(result)), 0, tolerance = 1e-12)
})

test_that("FIM equals n/lambda^2", {
  model <- exponential_lifetime("t")
  fim_func <- fim(model)

  # FIM for n observations at lambda
  for (lam in c(0.5, 1.0, 2.0, 5.0)) {
    for (n_obs in c(10, 100, 1000)) {
      expected_fim <- matrix(n_obs / lam^2, 1, 1)
      computed_fim <- fim_func(lam, n_obs)
      expect_equal(as.numeric(computed_fim), as.numeric(expected_fim),
                   tolerance = 1e-12,
                   info = paste("FIM mismatch at lambda =", lam, "n =", n_obs))
    }
  }
})

test_that("full Fisherian inference works with exponential_lifetime", {
  set.seed(4008)
  lambda_true <- 2.0
  n <- 200
  x <- rexp(n, rate = lambda_true)
  df <- data.frame(t = x)

  model <- exponential_lifetime("t")
  mle_result <- fit(model)(df)

  # Support at MLE should be 0
  s_at_mle <- support(mle_result, coef(mle_result), df, model)
  expect_equal(unname(s_at_mle), 0, tolerance = 1e-10)

  # Support away from MLE should be negative
  s_away <- support(mle_result, c(lambda = 0.5), df, model)
  expect_true(unname(s_away) < 0)

  # Relative likelihood at MLE is 1
  rl_at_mle <- relative_likelihood(mle_result, coef(mle_result), df, model)
  expect_equal(unname(rl_at_mle), 1, tolerance = 1e-10)

  # Likelihood interval should contain MLE
  li <- likelihood_interval(mle_result, df, model, k = 8)
  mle_val <- unname(coef(mle_result))
  expect_true(!is.na(li[1, 1]) && !is.na(li[1, 2]))
  expect_true(li[1, 1] < mle_val && mle_val < li[1, 2])
})

test_that("exponential_lifetime errors with no exact observations", {
  df <- data.frame(t = c(1, 2, 3), status = rep("right", 3))
  model <- exponential_lifetime("t", censor_col = "status")

  expect_error(fit(model)(df), "no exact")
})

test_that("exponential_lifetime rdata generates correct data", {
  model <- exponential_lifetime("t")
  rdata_fn <- rdata(model)

  set.seed(4010)
  df <- rdata_fn(theta = 2.0, n = 100)
  expect_equal(nrow(df), 100)
  expect_true("t" %in% names(df))
  expect_true(all(df$t > 0))

  # With censoring
  model_c <- exponential_lifetime("t", censor_col = "status")
  rdata_fn_c <- rdata(model_c)
  df_c <- rdata_fn_c(theta = 2.0, n = 100, censor_time = 0.5)
  expect_equal(nrow(df_c), 100)
  expect_true("t" %in% names(df_c))
  expect_true("status" %in% names(df_c))
  expect_true(all(df_c$t <= 0.5))
  expect_true(all(df_c$status %in% c("exact", "right")))
})

test_that("assumptions include censoring note when censor_col is set", {
  model <- exponential_lifetime("t")
  a <- assumptions(model)
  expect_false("non-informative right censoring" %in% a)

  model_c <- exponential_lifetime("t", censor_col = "status")
  a_c <- assumptions(model_c)
  expect_true("non-informative right censoring" %in% a_c)
})

# =============================================================================
# ALGEBRAIC.MLE INTERFACE COMPATIBILITY TESTS
# =============================================================================

test_that("params() returns same as coef() on fisher_mle", {
  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    loglik_val = -50,
    hessian = matrix(c(-100, 0, 0, -50), 2, 2),
    nobs = 100
  )

  expect_equal(params(result), coef(result))
  expect_equal(params(result), c(shape = 2.0, scale = 1.5))
})

test_that("nparams() returns correct count", {
  # Multivariate
  result2 <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    loglik_val = -50,
    nobs = 100
  )
  expect_equal(nparams(result2), 2L)

  # Univariate
  result1 <- fisher_mle(
    par = c(lambda = 3.0),
    loglik_val = -30,
    nobs = 50
  )
  expect_equal(nparams(result1), 1L)
})

test_that("observed_fim() returns -hessian (positive definite)", {
  H <- matrix(c(-100, -5, -5, -50), 2, 2)
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    loglik_val = -50,
    hessian = H,
    nobs = 100
  )

  fim_mat <- observed_fim(result)
  expect_equal(fim_mat, -H)

  # Should be positive definite (all eigenvalues > 0)
  evals <- eigen(fim_mat)$values
  expect_true(all(evals > 0))
})

test_that("observed_fim() returns NULL when hessian is NULL", {
  result <- fisher_mle(
    par = c(a = 1),
    loglik_val = -50,
    nobs = 100
    # hessian and vcov both NULL
  )

  expect_null(observed_fim(result))
})

test_that("obs() returns NULL for fisher_mle (by design)", {
  result <- fisher_mle(
    par = c(a = 1),
    loglik_val = -50,
    nobs = 100
  )
  expect_null(obs(result))
})

test_that("mse() equals vcov under zero asymptotic bias", {
  V <- matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2)
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    vcov = V,
    loglik_val = -50,
    nobs = 100
  )

  # Bias is zero for fisher_mle, so MSE = vcov
  expect_equal(mse(result), V)
})

test_that("mse() works for univariate fisher_mle", {
  result <- fisher_mle(
    par = c(lambda = 2.0),
    vcov = matrix(0.04, 1, 1),
    loglik_val = -30,
    nobs = 50
  )

  # Scalar case: MSE = var + 0^2 = var
  expect_equal(mse(result), matrix(0.04, 1, 1))
})

test_that("params()/nparams() work on fit() output from real models", {
  set.seed(5001)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  p <- params(result)
  expect_equal(length(p), 1)
  expect_equal(p, coef(result))
  expect_equal(nparams(result), 1L)
})

test_that("observed_fim() is positive definite on real fit output", {
  set.seed(5002)
  df <- data.frame(t = rexp(200, rate = 2))
  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  fim_mat <- observed_fim(result)
  expect_true(!is.null(fim_mat))
  expect_true(is.matrix(fim_mat))

  evals <- eigen(fim_mat)$values
  expect_true(all(evals > 0),
              info = "Observed FIM should be positive definite at MLE")
})

test_that("rmap() fallthrough works on fisher_mle", {
  skip_if_not_installed("algebraic.mle")

  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  # rmap transforms parameters via delta method
  # Transform: mean lifetime = scale * gamma(1 + 1/shape)
  mapped <- algebraic.mle::rmap(result, function(p) {
    c(mean_life = p[2] * gamma(1 + 1 / p[1]))
  })

  expect_true(inherits(mapped, "mle_fit"))
  expect_equal(length(params(mapped)), 1)
  expect_true(params(mapped) > 0)
})

test_that("marginal() fallthrough works on fisher_mle", {
  skip_if_not_installed("algebraic.mle")

  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  # Extract marginal for first parameter
  m <- algebraic.mle::marginal(result, 1)
  expect_true(inherits(m, "mle_fit"))
  expect_equal(nparams(m), 1L)
  expect_equal(unname(params(m)), 2.0)
})

test_that("expectation() fallthrough works on fisher_mle", {
  skip_if_not_installed("algebraic.mle")

  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  set.seed(5003)
  # Monte Carlo expectation of shape parameter
  # Use qualified name to avoid conflict with testthat::expectation
  # n goes in control list per algebraic.mle API
  e <- algebraic.mle::expectation(result, function(p) p[1],
                                  control = list(n = 5000L))
  expect_equal(e, 2.0, tolerance = 0.1)
})


# =============================================================================
# FISHER_MLE PRINT AND SUMMARY COVERAGE TESTS
# =============================================================================

test_that("print.fisher_mle outputs expected text", {
  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  out <- capture.output(print(result))
  expect_true(any(grepl("Maximum Likelihood Estimate", out)))
  expect_true(any(grepl("Log-likelihood", out)))
  expect_true(any(grepl("Observations:", out)))
})

test_that("print.fisher_mle shows warning when not converged", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50,
    nobs = 10,
    converged = FALSE
  )

  out <- capture.output(print(result))
  expect_true(any(grepl("WARNING.*converge", out)))
})

test_that("print.fisher_mle works without nobs", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50
  )

  out <- capture.output(print(result))
  expect_false(any(grepl("Observations:", out)))
})

test_that("print.summary_fisher_mle outputs expected text", {
  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  out <- capture.output(print(summary(result)))
  expect_true(any(grepl("Maximum Likelihood Estimate", out)))
  expect_true(any(grepl("AIC", out)))
  expect_true(any(grepl("Log-likelihood", out)))
  expect_true(any(grepl("Number of observations", out)))
})

test_that("print.summary_fisher_mle shows warning when not converged", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50,
    nobs = 10,
    converged = FALSE
  )

  out <- capture.output(print(summary(result)))
  expect_true(any(grepl("WARNING.*converge", out)))
})

test_that("print.summary_fisher_mle works without nobs", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50
  )

  out <- capture.output(print(summary(result)))
  expect_false(any(grepl("Number of observations", out)))
})


# =============================================================================
# FISHER_MLE ACCESSOR COVERAGE TESTS
# =============================================================================

test_that("logLik.fisher_mle returns the log-likelihood value", {
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    vcov = matrix(c(0.04, 0, 0, 0.02), 2, 2),
    loglik_val = -123.45,
    nobs = 50
  )
  expect_equal(as.numeric(logLik(result)), -123.45)
})

test_that("confint.fisher_mle works with character parm", {
  result <- fisher_mle(
    par = c(shape = 2.0, scale = 1.5),
    vcov = matrix(c(0.04, 0.01, 0.01, 0.02), 2, 2),
    loglik_val = -50,
    nobs = 100
  )

  ci_all <- confint(result)
  ci_shape <- confint(result, parm = "shape")
  expect_equal(nrow(ci_shape), 1)
  expect_equal(ci_shape[1, ], ci_all["shape", ])
})

test_that("fisher_mle warns when hessian is singular", {
  # Singular hessian: second row is a multiple of first
  H <- matrix(c(-1, -2, -2, -4), 2, 2)
  expect_warning(
    result <- fisher_mle(
      par = c(a = 1, b = 2),
      loglik_val = -50,
      hessian = H,
      nobs = 100
    ),
    "not invertible"
  )
  expect_null(result$vcov)
})

test_that("BIC works via stats::BIC when nobs is available", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50,
    nobs = 100
  )
  # BIC = -2*logL + k*log(n) = 100 + 1*log(100)
  expect_equal(stats::BIC(result), 100 + log(100))
})


# =============================================================================
# FISHER_BOOT TESTS
# =============================================================================

test_that("confint.fisher_boot computes percentile CI", {
  skip_if_not_installed("boot")
  set.seed(6001)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 200)

  ci <- confint(boot_obj, level = 0.95, type = "perc")
  expect_equal(nrow(ci), 1)
  expect_equal(ncol(ci), 2)
  expect_true(ci[1, 1] < ci[1, 2])  # lower < upper
})

test_that("confint.fisher_boot computes basic CI", {
  skip_if_not_installed("boot")
  set.seed(6002)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 200)

  ci <- confint(boot_obj, level = 0.95, type = "basic")
  expect_equal(nrow(ci), 1)
  expect_equal(ncol(ci), 2)
})

test_that("confint.fisher_boot computes norm CI", {
  skip_if_not_installed("boot")
  set.seed(6003)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 200)

  ci <- confint(boot_obj, level = 0.95, type = "norm")
  expect_equal(nrow(ci), 1)
  expect_equal(ncol(ci), 2)
})

test_that("confint.fisher_boot with integer parm", {
  skip_if_not_installed("boot")
  set.seed(6004)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 200)

  ci <- confint(boot_obj, parm = 1, level = 0.95, type = "perc")
  expect_equal(nrow(ci), 1)
})

test_that("print.fisher_boot outputs expected text", {
  skip_if_not_installed("boot")
  set.seed(6005)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 50)

  out <- capture.output(print(boot_obj))
  expect_true(any(grepl("Bootstrap MLE", out)))
  expect_true(any(grepl("Bootstrap replicates:", out)))
  expect_true(any(grepl("Bootstrap SE:", out)))
})

test_that("sampler.fisher_boot resamples from bootstrap distribution", {
  skip_if_not_installed("boot")
  set.seed(6006)
  df <- data.frame(t = rexp(100, rate = 2))
  model <- exponential_lifetime("t")
  model_sampler <- sampler(model, df = df, par = c(lambda = 2))
  boot_obj <- model_sampler(n = 200)

  samp_fn <- sampler(boot_obj)
  samples <- samp_fn(100)
  expect_equal(nrow(samples), 100)
  expect_equal(ncol(samples), 1)
  expect_true(all(is.finite(samples)))
})


# =============================================================================
# SAMPLER.FISHER_MLE UNIVARIATE COVERAGE
# =============================================================================

test_that("sampler.fisher_mle works for univariate case", {
  result <- fisher_mle(
    par = c(rate = 2.0),
    vcov = matrix(0.04, 1, 1),
    loglik_val = -30,
    nobs = 50
  )

  samp_fn <- sampler(result)
  set.seed(6007)
  samples <- samp_fn(500)
  expect_equal(ncol(samples), 1)
  expect_equal(nrow(samples), 500)
  expect_equal(mean(samples), 2.0, tolerance = 0.1)
})

test_that("sampler.fisher_mle errors when vcov is NULL", {
  result <- fisher_mle(
    par = c(rate = 2.0),
    loglik_val = -30,
    nobs = 50
  )
  expect_error(sampler(result), "vcov is NULL")
})


# =============================================================================
# FISHERIAN INFERENCE COVERAGE TESTS
# =============================================================================

test_that("likelihood_interval works for single parameter model", {
  set.seed(6009)
  df <- data.frame(x = rexp(100, rate = 2))
  model <- exponential_lifetime("x")
  result <- fit(model)(df, par = c(rate = 1))

  li <- likelihood_interval(result, data = df, model = model, k = 8)
  expect_true(!is.null(li))
})

test_that("print.likelihood_interval outputs expected text", {
  set.seed(6010)
  df <- data.frame(x = rexp(100, rate = 2))
  model <- exponential_lifetime("x")
  result <- fit(model)(df, par = c(rate = 1))

  li <- likelihood_interval(result, data = df, model = model, k = 8)
  out <- capture.output(print(li))
  expect_true(any(grepl("Likelihood Interval", out)))
})

test_that("profile_loglik errors for > 2 parameters", {
  result <- fisher_mle(
    par = c(a = 1, b = 2, c = 3),
    vcov = diag(3) * 0.01,
    loglik_val = -50,
    nobs = 100
  )
  # Use a mock model for this error test
  mock_model <- structure(list(), class = c("mock_3param", "likelihood_model"))
  loglik.mock_3param <<- function(model, ...) function(df, par, ...) 0
  df <- data.frame(x = 1:10)
  expect_error(
    profile_loglik(result, data = df, model = mock_model, param = 1:3),
    "1 or 2 parameters"
  )
  rm(loglik.mock_3param, envir = .GlobalEnv)
})

test_that("print.profile_loglik outputs expected text", {
  set.seed(6013)
  df <- data.frame(t = rexp(50, rate = 2))
  model <- exponential_lifetime("t")
  result <- fit(model)(df)

  pl <- profile_loglik(result, data = df, model = model,
                       param = 1, n_grid = 10)
  out <- capture.output(print(pl))
  expect_true(any(grepl("Profile Log-Likelihood", out)))
  expect_true(any(grepl("Grid points:", out)))
})

test_that("deviance.fisher_mle errors on non-fisher_mle null model", {
  result <- fisher_mle(
    par = c(a = 1),
    vcov = matrix(0.1, 1, 1),
    loglik_val = -50,
    nobs = 100
  )
  expect_error(deviance(result, null_model = list(loglik = -60)),
               "fisher_mle")
})


# =============================================================================
# FIM AND MOCK MODEL COVERAGE TESTS
# =============================================================================

# Helper: register mock exponential loglik/rdata methods in the global env.
# Returns a model object that falls through to fim.likelihood_model (unlike
# exponential_lifetime, which has its own fim method).
make_mock_exp <- function(class_name) {
  assign(paste0("loglik.", class_name), function(model, ...) {
    function(df, par, ...) {
      lambda <- par[1]
      if (lambda <= 0) return(-.Machine$double.xmax / 2)
      nrow(df) * log(lambda) - lambda * sum(df$x)
    }
  }, envir = .GlobalEnv)

  assign(paste0("rdata.", class_name), function(model, ...) {
    function(theta, n, ...) {
      data.frame(x = rexp(n, rate = theta[1]))
    }
  }, envir = .GlobalEnv)

  structure(list(), class = c(class_name, "likelihood_model"))
}

cleanup_mock_exp <- function(class_name) {
  rm(list = paste0(c("loglik.", "rdata."), class_name), envir = .GlobalEnv)
}

test_that("fim.likelihood_model computes FIM via Monte Carlo (neg Hessian)", {
  mock_model <- make_mock_exp("mock_exp")

  set.seed(6014)
  fim_fn <- fim(mock_model)
  # True FIM for exp(rate=2) with n=50 is n/rate^2 = 50/4 = 12.5
  fim_mat <- fim_fn(theta = c(2), n_obs = 50, n_samples = 500)
  expect_true(is.matrix(fim_mat))
  expect_equal(nrow(fim_mat), 1)
  expect_equal(ncol(fim_mat), 1)
  expect_equal(fim_mat[1, 1], 12.5, tolerance = 2)

  cleanup_mock_exp("mock_exp")
})

test_that("fim.likelihood_model scales linearly with n_obs", {
  mock_model <- make_mock_exp("mock_exp2")

  set.seed(7001)
  fim_fn <- fim(mock_model)
  fim_10 <- fim_fn(theta = c(2), n_obs = 10, n_samples = 2000)
  fim_50 <- fim_fn(theta = c(2), n_obs = 50, n_samples = 2000)

  # FIM should scale as n_obs / lambda^2, so ratio ~ 50/10 = 5
  expect_equal(fim_50[1, 1] / fim_10[1, 1], 5, tolerance = 0.5)

  cleanup_mock_exp("mock_exp2")
})

test_that("fim.likelihood_model adds parameter names from theta", {
  mock_model <- make_mock_exp("mock_exp3")

  set.seed(7002)
  fim_fn <- fim(mock_model)
  fim_mat <- fim_fn(theta = c(rate = 2), n_obs = 10, n_samples = 100)
  expect_equal(rownames(fim_mat), "rate")
  expect_equal(colnames(fim_mat), "rate")

  cleanup_mock_exp("mock_exp3")
})


# =============================================================================
# BIAS.FISHER_MLE TESTS
# =============================================================================

test_that("bias.fisher_mle returns zeros without model", {
  result <- fisher_mle(
    par = c(a = 1, b = 2),
    vcov = diag(2) * 0.01,
    loglik_val = -50,
    nobs = 100
  )
  b <- bias(result)
  expect_equal(b, c(0, 0))

  # With theta but no model, still zeros
  b2 <- bias(result, theta = c(1, 2))
  expect_equal(b2, c(0, 0))
})

test_that("bias.fisher_mle estimates bias via MC when model provided", {
  model <- exponential_lifetime("t")
  set.seed(7020)
  df <- data.frame(t = rexp(200, rate = 2))
  result <- fit(model)(df)

  # MC bias for exponential MLE should be near zero for large n
  b <- bias(result, theta = c(lambda = 2), model = model, n_sim = 200)
  expect_equal(length(b), 1)
  # Bias should be small relative to the parameter value
  expect_true(abs(b[1]) < 0.5)
})


# =============================================================================
# FIM ... FORWARDING + NaN GUARD TESTS
# =============================================================================

test_that("fim.likelihood_model forwards ... to rdata only, not to hess_fn", {
  # CONTRACT (v1.0.1+): the inner `...` of the closure returned by
  # fim(model) flows to rdata only. It does NOT flow to hess_loglik.
  #
  # Rationale:
  #   1. The likelihood is a function of the data alone. Kwargs that
  #      describe the data-generating process (censoring time tau,
  #      masking probability p, observation functor, etc.) are inputs
  #      to rdata. They must not appear in the likelihood, by the
  #      Fisherian separation between sampling and inference, and
  #      specifically by masking condition C3 in this package's models.
  #   2. R partial-argument-matching makes the forwarding-to-hess
  #      behavior actively dangerous: any kwarg whose name shares a
  #      prefix with a hess_loglik formal (e.g., a DGP `p` colliding
  #      with the parameter formal `par`) is silently misrouted and
  #      corrupts the call.
  #
  # This test pins the contract: a likelihood-modifying kwarg
  # `multiplier` passed via the FIM closure's `...` must NOT reach
  # the mock's loglik, so the FIM is unchanged when it is set.
  mock_model <- structure(list(), class = c("mock_dots_fim", "likelihood_model"))

  loglik.mock_dots_fim <<- function(model, ...) {
    function(df, par, multiplier = 1, ...) {
      multiplier * (nrow(df) * log(par[1]) - par[1] * sum(df$x))
    }
  }

  rdata.mock_dots_fim <<- function(model, ...) {
    function(theta, n, ...) {
      data.frame(x = rexp(n, rate = theta[1]))
    }
  }

  set.seed(8002)
  fim_fn <- fim(mock_model)
  fim_default <- fim_fn(theta = c(2), n_obs = 10, n_samples = 500)
  set.seed(8002)
  fim_with_kwarg <- fim_fn(theta = c(2), n_obs = 10,
                           n_samples = 500, multiplier = 2)

  # multiplier did NOT reach the likelihood evaluator: same FIM.
  expect_equal(fim_with_kwarg[1, 1], fim_default[1, 1], tolerance = 1e-8)

  rm(loglik.mock_dots_fim, rdata.mock_dots_fim, envir = .GlobalEnv)
})

test_that("fim.likelihood_model forwards ... to rdata (DGP kwargs)", {
  # Complement to the above: a kwarg that the rdata closure consumes
  # (here, a rate offset that shifts the sampling distribution) DOES
  # reach rdata, which changes the generated data, which changes the
  # data-dependent Hessian, which changes the FIM.
  #
  # Note: the mock's hess_loglik returns matrix(-sum(df$x), 1, 1),
  # an x-dependent Hessian. This is what lets us see the DGP shift
  # propagate through to the FIM. (For a canonical exponential family
  # the FIM at fixed theta does not depend on the DGP via the data,
  # so we use a synthetic non-canonical Hessian for the test.)
  mock_model <- structure(list(), class = c("mock_dgp_kwarg", "likelihood_model"))

  loglik.mock_dgp_kwarg <<- function(model, ...) {
    function(df, par, ...) {
      # Not actually used by hess_loglik below; included for completeness.
      nrow(df) * log(par[1]) - par[1] * sum(df$x)
    }
  }

  hess_loglik.mock_dgp_kwarg <<- function(model, ...) {
    # Data-dependent Hessian: -sum(x). Negative because the FIM is
    # the negated expectation; we want a positive FIM.
    function(df, par, ...) matrix(-sum(df$x), 1, 1)
  }

  rdata.mock_dgp_kwarg <<- function(model, ...) {
    function(theta, n, rate_offset = 0, ...) {
      data.frame(x = rexp(n, rate = theta[1] + rate_offset))
    }
  }

  set.seed(8011)
  fim_fn <- fim(mock_model)
  fim_default <- fim_fn(theta = c(2), n_obs = 10, n_samples = 500)
  set.seed(8011)
  fim_shifted <- fim_fn(theta = c(2), n_obs = 10,
                        n_samples = 500, rate_offset = 3)

  # rate_offset = 0: x ~ Exp(2),    E[sum(x)|n=1] = 0.5,   FIM ~ 10 * 0.5 = 5
  # rate_offset = 3: x ~ Exp(5),    E[sum(x)|n=1] = 0.2,   FIM ~ 10 * 0.2 = 2
  expect_true(fim_default[1, 1] > fim_shifted[1, 1] + 1)

  rm(loglik.mock_dgp_kwarg, hess_loglik.mock_dgp_kwarg, rdata.mock_dgp_kwarg,
     envir = .GlobalEnv)
})

test_that("fim.likelihood_model handles NaN from hess_loglik gracefully", {
  mock_model <- structure(list(), class = c("mock_nan_fim", "likelihood_model"))

  call_count <- 0L
  hess_loglik.mock_nan_fim <<- function(model, ...) {
    function(df, par, ...) {
      # Return NaN matrix every 3rd call
      call_count <<- call_count + 1L
      if (call_count %% 3 == 0) return(matrix(NaN, 1, 1))
      matrix(-nrow(df) / par[1]^2, 1, 1)
    }
  }

  rdata.mock_nan_fim <<- function(model, ...) {
    function(theta, n, ...) {
      data.frame(x = rexp(n, rate = theta[1]))
    }
  }

  set.seed(8003)
  fim_fn <- fim(mock_model)
  # Should still produce a valid FIM despite some NaN samples
  expect_warning(
    fim_mat <- fim_fn(theta = c(2), n_obs = 10, n_samples = 100),
    "samples failed"
  )
  expect_true(is.matrix(fim_mat))
  expect_true(all(is.finite(fim_mat)))

  rm(hess_loglik.mock_nan_fim, rdata.mock_nan_fim, envir = .GlobalEnv)
  call_count <- NULL
})

# =============================================================================
# MSE TESTS
# =============================================================================

test_that("mse.fisher_mle returns Vcov when bias is zero (asymptotic)", {
  model <- exponential_lifetime("t")
  set.seed(8005)
  df <- data.frame(t = rexp(100, rate = 2))
  result <- fit(model)(df)

  # Without MC, bias = 0, so MSE = Vcov
  mse_val <- mse(result, theta = c(lambda = 2))
  expect_equal(mse_val, vcov(result))
})

test_that("mse.fisher_mle forwards model and n_sim to bias for MC-based MSE", {
  model <- exponential_lifetime("t")
  set.seed(8005)
  df <- data.frame(t = rexp(100, rate = 2))
  result <- fit(model)(df)

  # With model, MSE accounts for finite-sample bias
  mse_mc <- mse(result, theta = c(lambda = 2), model = model, n_sim = 100)
  expect_true(is.finite(mse_mc))
})

# =============================================================================
# SAMPLER ... SCOPING TEST
# =============================================================================

test_that("sampler.likelihood_model bootstrap works correctly", {
  model <- exponential_lifetime("t")
  set.seed(8006)
  df <- data.frame(t = rexp(50, rate = 2))

  boot_sampler <- sampler(model, df, par = c(lambda = 2))
  boot_result <- boot_sampler(50)

  expect_s3_class(boot_result, "fisher_boot")
  expect_equal(ncol(boot_result$replicates), 1)
  expect_equal(nrow(boot_result$replicates), 50)
})

# =============================================================================
# DEFAULT METHOD ERROR MESSAGES
# =============================================================================

test_that("loglik.likelihood_model gives clear error for unimplemented class", {
  fake_model <- structure(list(), class = c("fake_model", "likelihood_model"))
  expect_error(loglik(fake_model), "must implement loglik")
})

test_that("rdata.likelihood_model gives clear error", {
  fake_model <- structure(list(), class = c("fake_model2", "likelihood_model"))
  loglik.fake_model2 <<- function(model, ...) function(df, par, ...) 0
  expect_error(rdata(fake_model), "does not implement rdata")
  rm(loglik.fake_model2, envir = .GlobalEnv)
})

# =============================================================================
# LRT RESULT CLASS + PRINT
# =============================================================================

test_that("lrt returns lrt_result class with print method", {
  set.seed(8007)
  df <- data.frame(t = rexp(50, rate = 2))
  model <- exponential_lifetime("t")

  result <- lrt(model, model, df,
                null_par = c(2), alt_par = c(2), dof = 0)

  expect_s3_class(result, "lrt_result")
  expect_true("stat" %in% names(result))
  expect_true("p.value" %in% names(result))

  out <- capture.output(print(result))
  expect_true(any(grepl("Likelihood Ratio Test", out)))
})

# =============================================================================
# EVIDENCE IS A GENERIC
# =============================================================================

test_that("evidence dispatches via S3", {
  set.seed(8008)
  df <- data.frame(t = rexp(50, rate = 2))
  model <- exponential_lifetime("t")

  e <- evidence(model, data = df, theta1 = c(lambda = 2), theta2 = c(lambda = 1))
  expect_true(is.numeric(e))
  expect_equal(length(e), 1)

  # Verify it matches manual computation
  ll <- loglik(model)
  expected <- ll(df, c(lambda = 2)) - ll(df, c(lambda = 1))
  expect_equal(e, expected)
})


# =============================================================================
# COVERAGE RECOVERY TESTS
# =============================================================================
# These tests target multivariate code paths that require a 2+ parameter model.
#
# Mock normal model: loglik.mock_norm, score.mock_norm, hess_loglik.mock_norm
# are registered in the global environment and cleaned up after each test
# block that needs them.

# Helper: set up and tear down mock_norm model
make_mock_norm <- function() {
  model <- structure(list(ob_col = "x"), class = c("mock_norm", "likelihood_model"))

  loglik.mock_norm <<- function(model, ...) {
    function(df, par, ...) {
      mu <- par[1]; sigma <- par[2]
      if (sigma <= 0) return(-.Machine$double.xmax / 2)
      sum(dnorm(df[[model$ob_col]], mu, sigma, log = TRUE))
    }
  }

  model
}

cleanup_mock_norm <- function() {
  to_rm <- intersect(c("loglik.mock_norm"), ls(envir = .GlobalEnv))
  if (length(to_rm)) rm(list = to_rm, envir = .GlobalEnv)
}

# ---------------------------------------------------------------------------
# sampler.fisher_mle: multivariate branch (mvtnorm::rmvnorm) — line 543
# ---------------------------------------------------------------------------

test_that("sampler.fisher_mle uses mvtnorm for 2-param model", {
  skip_if_not_installed("mvtnorm")

  result <- fisher_mle(
    par = c(mu = 5.0, sigma = 2.0),
    vcov = matrix(c(0.04, 0.0, 0.0, 0.02), 2, 2),
    loglik_val = -150,
    nobs = 100
  )

  samp_fn <- sampler(result)
  set.seed(9001)
  samples <- samp_fn(500)

  expect_equal(dim(samples), c(500, 2))
  expect_equal(mean(samples[, 1]), 5.0, tolerance = 0.1)
  expect_equal(mean(samples[, 2]), 2.0, tolerance = 0.1)
})

# ---------------------------------------------------------------------------
# likelihood_interval: multivariate (profile) branch — lines 149-213
# ---------------------------------------------------------------------------

test_that("likelihood_interval computes profile interval for 2-param model", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9002)
  df <- data.frame(x = rnorm(100, mean = 3, sd = 1.5))

  # Fit via fit.likelihood_model (optim-based)
  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  # Compute profile likelihood interval for mu (param = 1)
  li <- likelihood_interval(result, data = df, model = model, k = 8, param = 1)

  expect_true(inherits(li, "likelihood_interval"))
  expect_equal(nrow(li), 1)
  expect_equal(ncol(li), 2)

  # Interval should contain the MLE for mu
  mu_hat <- coef(result)[1]
  expect_true(li[1, 1] < mu_hat && mu_hat < li[1, 2])
})

test_that("likelihood_interval computes profile intervals for both params", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9003)
  df <- data.frame(x = rnorm(80, mean = 2, sd = 1))

  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  # Profile interval for all params (NULL => both)
  li <- likelihood_interval(result, data = df, model = model, k = 8)

  expect_equal(nrow(li), 2)
  mu_hat <- coef(result)[1]
  sigma_hat <- coef(result)[2]
  expect_true(li[1, 1] < mu_hat && mu_hat < li[1, 2])
  expect_true(li[2, 1] < sigma_hat && sigma_hat < li[2, 2])
})

test_that("likelihood_interval handles character param name for 2-param model", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9004)
  df <- data.frame(x = rnorm(60, mean = 1, sd = 2))

  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  # Use character param name
  li <- likelihood_interval(result, data = df, model = model, k = 8,
                            param = "mu")
  expect_equal(nrow(li), 1)
  mu_hat <- coef(result)["mu"]
  expect_true(li[1, 1] < mu_hat && mu_hat < li[1, 2])
})

# ---------------------------------------------------------------------------
# profile_loglik: 1D profile over a 2-param model — lines 310-329
# ---------------------------------------------------------------------------

test_that("profile_loglik 1D profile over nuisance parameter works", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9005)
  df <- data.frame(x = rnorm(80, mean = 3, sd = 1.5))

  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  # Profile over mu (param=1), optimising out sigma
  prof <- profile_loglik(result, data = df, model = model, param = 1, n_grid = 15)

  expect_true(inherits(prof, "profile_loglik"))
  expect_equal(nrow(prof), 15)
  expect_true("loglik" %in% names(prof))
  expect_true("support" %in% names(prof))
  expect_true("relative_likelihood" %in% names(prof))

  # Profile should peak near MLE
  mu_hat <- unname(coef(result)[1])
  max_idx <- which.max(prof$loglik)
  expect_equal(unname(prof[[1]][max_idx]), mu_hat, tolerance = 0.5)
})

# ---------------------------------------------------------------------------
# profile_loglik: 2D profile — lines 332-358
# ---------------------------------------------------------------------------

test_that("profile_loglik 2D profile produces correct shape data frame", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9006)
  df <- data.frame(x = rnorm(60, mean = 2, sd = 1))

  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  prof <- profile_loglik(result, data = df, model = model,
                         param = 1:2, n_grid = 8)

  expect_true(inherits(prof, "profile_loglik"))
  expect_equal(nrow(prof), 8 * 8)   # expand.grid of 8x8 grid
  expect_true("loglik" %in% names(prof))
  expect_true("support" %in% names(prof))
  expect_true("relative_likelihood" %in% names(prof))
})

# ---------------------------------------------------------------------------
# print.profile_loglik: 2D case — covered when param has length 2
# ---------------------------------------------------------------------------

test_that("print.profile_loglik prints correctly for 2D profile", {
  skip_if_not_installed("mvtnorm")
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9007)
  df <- data.frame(x = rnorm(40, mean = 1, sd = 1))

  solver <- fit(model)
  result <- solver(df, par = c(mu = 0, sigma = 1))

  prof <- profile_loglik(result, data = df, model = model,
                         param = 1:2, n_grid = 5)

  out <- capture.output(print(prof))
  expect_true(any(grepl("Profile Log-Likelihood", out)))
  expect_true(any(grepl("Grid points:", out)))
  # Should mention both parameter names
  expect_true(any(grepl("mu", out)))
})

# ---------------------------------------------------------------------------
# bias.fisher_mle: nobs NULL error — line 294
# ---------------------------------------------------------------------------

test_that("bias.fisher_mle errors when nobs is NULL and model is provided", {
  model <- exponential_lifetime("t")

  result <- fisher_mle(
    par = c(lambda = 2.0),
    vcov = matrix(0.04, 1, 1),
    loglik_val = -50
    # nobs intentionally omitted (NULL)
  )

  expect_error(
    bias(result, theta = c(lambda = 2), model = model),
    "nobs not available"
  )
})

# ---------------------------------------------------------------------------
# bias.fisher_mle: warning when most MC replicates fail — lines 314-317
# ---------------------------------------------------------------------------

test_that("bias.fisher_mle warns when most MC replicates fail", {
  # Create a model whose fit() always errors
  bad_model <- structure(
    list(ob_col = "t"),
    class = c("mock_bad_fit", "likelihood_model")
  )

  loglik.mock_bad_fit <<- function(model, ...) {
    function(df, par, ...) {
      lambda <- par[1]
      if (lambda <= 0) return(-.Machine$double.xmax / 2)
      nrow(df) * log(lambda) - lambda * sum(df[[model$ob_col]])
    }
  }

  rdata.mock_bad_fit <<- function(model, ...) {
    function(theta, n, ...) {
      data.frame(t = rexp(n, rate = theta[1]))
    }
  }

  # Override fit to always error
  fit.mock_bad_fit <<- function(object, ...) {
    function(df, par = NULL, ...) {
      stop("Intentional fit failure for testing")
    }
  }

  on.exit({
    rm(list = intersect(
      c("loglik.mock_bad_fit", "rdata.mock_bad_fit", "fit.mock_bad_fit"),
      ls(envir = .GlobalEnv)
    ), envir = .GlobalEnv)
  })

  set.seed(9008)
  result <- fisher_mle(
    par = c(lambda = 2.0),
    vcov = matrix(0.04, 1, 1),
    loglik_val = -50,
    nobs = 50
  )

  # All replicates will fail, so we get a warning + NA return
  expect_warning(
    b <- bias(result, theta = c(lambda = 2), model = bad_model, n_sim = 20),
    "MC replicates succeeded"
  )
  expect_true(all(is.na(b)))
})

# ---------------------------------------------------------------------------
# print.fisher_boot: bias line is covered when boot object has 2+ params
# ---------------------------------------------------------------------------

test_that("print.fisher_boot covers format(bias(x)) for multivariate bootstrap", {
  skip_if_not_installed("boot")

  # Build a 2-param mock model that fit.likelihood_model can use
  model <- make_mock_norm()
  on.exit(cleanup_mock_norm())

  set.seed(9009)
  df <- data.frame(x = rnorm(50, mean = 2, sd = 1))

  boot_sampler <- sampler(model, df, par = c(mu = 0, sigma = 1))
  boot_obj <- boot_sampler(30)

  out <- capture.output(print(boot_obj))
  expect_true(any(grepl("Bootstrap MLE", out)))
  expect_true(any(grepl("Bootstrap bias:", out)))
  # Should show 2 bias values
  expect_equal(length(bias(boot_obj)), 2)
})

# ---------------------------------------------------------------------------
# core-generics.R line 247: "All Monte Carlo samples failed" error in fim
# ---------------------------------------------------------------------------

test_that("fim.likelihood_model errors when all MC samples fail", {
  always_fail_model <- structure(
    list(),
    class = c("mock_always_fail_fim", "likelihood_model")
  )

  loglik.mock_always_fail_fim <<- function(model, ...) {
    function(df, par, ...) 0
  }

  hess_loglik.mock_always_fail_fim <<- function(model, ...) {
    function(df, par, ...) stop("Always fails")
  }

  rdata.mock_always_fail_fim <<- function(model, ...) {
    function(theta, n, ...) data.frame(x = 1)
  }

  on.exit({
    rm(list = intersect(
      c("loglik.mock_always_fail_fim",
        "hess_loglik.mock_always_fail_fim",
        "rdata.mock_always_fail_fim"),
      ls(envir = .GlobalEnv)
    ), envir = .GlobalEnv)
  })

  fim_fn <- fim(always_fail_model)
  expect_error(
    fim_fn(theta = c(1), n_obs = 10, n_samples = 5),
    "All Monte Carlo samples failed"
  )
})

# ---------------------------------------------------------------------------
# exponential_lifetime loglik: negative lambda guard — line 76
# ---------------------------------------------------------------------------

test_that("loglik.exponential_lifetime returns sentinel for non-positive lambda", {
  model <- exponential_lifetime("t")
  ll_fn <- loglik(model)
  df <- data.frame(t = c(1, 2, 3))

  # Negative lambda
  val_neg <- ll_fn(df, c(-1))
  expect_equal(val_neg, -.Machine$double.xmax / 2)

  # Zero lambda
  val_zero <- ll_fn(df, c(0))
  expect_equal(val_zero, -.Machine$double.xmax / 2)
})

# ---------------------------------------------------------------------------
# fit.likelihood_model: auto-switch 1-param to BFGS — line 61
# (the SANN test uses a custom model; this test exercises the 1-param
#  Nelder-Mead -> BFGS switch via fit.likelihood_model directly)
# ---------------------------------------------------------------------------

test_that("fit.likelihood_model auto-switches 1-param Nelder-Mead to BFGS", {
  # Use make_mock_norm but override to 1-param normal (fixed sigma)
  mock_1p <- structure(list(), class = c("mock_1param_bfgs", "likelihood_model"))

  loglik.mock_1param_bfgs <<- function(model, ...) {
    function(df, par, ...) {
      mu <- par[1]
      sum(dnorm(df$x, mu, sd = 1, log = TRUE))
    }
  }

  on.exit(rm(list = intersect("loglik.mock_1param_bfgs", ls(.GlobalEnv)),
             envir = .GlobalEnv))

  set.seed(9010)
  df <- data.frame(x = rnorm(50, mean = 3, sd = 1))

  # Deliberately pass method = "Nelder-Mead" with 1 param; should switch to BFGS
  solver <- fit(mock_1p)
  result <- solver(df, par = c(mu = 0), method = "Nelder-Mead")

  expect_true(inherits(result, "fisher_mle"))
  expect_equal(unname(coef(result)[1]), 3, tolerance = 0.3)
})

#' Likelihood ratio test
#'
#' Computes the likelihood ratio test statistic and p-value for
#' nested models.
#'
#' @param null the likelihood model for the simpler (null) hypothesis nested
#'             within the alternative model
#' @param alt the likelihood model for the more complicated (alternative)
#'            hypothesis
#' @param data a data frame
#' @param null_par parameter values under the null model
#' @param alt_par parameter values under the alternative model
#' @param dof degrees of freedom (computed automatically if NULL)
#' @param ... additional arguments passed to loglik
#' @return An `lrt_result` object with components `stat`, `p.value`, and `dof`
#' @importFrom stats pchisq
#' @export
lrt <- function(null, alt, data, null_par, alt_par, dof = NULL, ...) {
  stopifnot(is_likelihood_model(null), is_likelihood_model(alt),
            is.data.frame(data))
  if (is.null(dof)) {
    dof <- length(alt_par) - length(null_par)
  }
  stopifnot(dof >= 0)
  stat <- -2 * (loglik(null)(df = data, par = null_par, ...) -
                  loglik(alt)(df = data, par = alt_par, ...))
  p.value <- pchisq(stat, df = dof, lower.tail = FALSE)
  structure(
    list(stat = stat, p.value = p.value, dof = dof),
    class = "lrt_result"
  )
}

#' Print method for likelihood ratio test results
#'
#' @param x An lrt_result object
#' @param ... Additional arguments (ignored)
#' @return The lrt_result object, invisibly
#' @export
print.lrt_result <- function(x, ...) {
  cat("Likelihood Ratio Test\n")
  cat("---------------------\n")
  cat("Test statistic:", format(x$stat, digits = 4), "\n")
  cat("Degrees of freedom:", x$dof, "\n")
  cat("P-value:", format(x$p.value, digits = 4), "\n")
  invisible(x)
}

#' @title Exponential lifetime model with right-censoring support
#' @description
#' A likelihood model for the Exponential(lambda) distribution with
#' optional right-censoring. This is a reference implementation
#' demonstrating:
#'
#' - **Closed-form MLE**: Overrides `fit()` to compute lambda_hat = d/T
#'   directly, bypassing `optim` entirely.
#' - **Analytical derivatives**: score, Hessian, and FIM in closed form.
#' - **Right-censoring**: Natural handling via the sufficient statistic
#'   (d, T) where d = number of exact observations and T = total time.
#' - **`rdata()` method**: For Monte Carlo validation and FIM estimation.
#'
#' The log-likelihood is:
#' \deqn{\ell(\lambda) = d \log \lambda - \lambda T}
#' where d is the number of exact (uncensored) observations and T is the
#' total observation time (sum of all times, whether censored or not).
#'
#' @param ob_col The name of the column containing observation times.
#' @param censor_col Optional column name indicating censoring status.
#'   When provided, values should be `"exact"` for uncensored observations
#'   and `"right"` for right-censored observations. When `NULL`, all
#'   observations are treated as exact.
#' @return An `exponential_lifetime` likelihood model object
#' @examples
#' # Uncensored exponential data
#' model <- exponential_lifetime("t")
#' df <- data.frame(t = rexp(100, rate = 2))
#' mle <- fit(model)(df)
#' coef(mle)  # should be close to 2
#'
#' # Right-censored data
#' model_c <- exponential_lifetime("t", censor_col = "status")
#' df_c <- data.frame(
#'   t = c(rexp(80, 2), rep(0.5, 20)),
#'   status = c(rep("exact", 80), rep("right", 20))
#' )
#' mle_c <- fit(model_c)(df_c)
#' coef(mle_c)
#' @export
exponential_lifetime <- function(ob_col, censor_col = NULL) {
  structure(
    list(ob_col = ob_col, censor_col = censor_col),
    class = c("exponential_lifetime", "likelihood_model")
  )
}


#' Assumptions for exponential_lifetime
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return Character vector of assumptions
#' @export
assumptions.exponential_lifetime <- function(model, ...) {
  a <- c("independent", "identically distributed", "exponential distribution")
  if (!is.null(model$censor_col)) {
    a <- c(a, "non-informative right censoring")
  }
  a
}


#' Log-likelihood for exponential_lifetime
#'
#' Returns a function computing ell(lambda) = d * log(lambda) - lambda * T.
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A function(df, par, ...) computing the log-likelihood
#' @export
loglik.exponential_lifetime <- function(model, ...) {
  function(df, par, ...) {
    stats <- .exponential_sufficient_stats(df, model)
    lambda <- par[1]
    if (lambda <= 0) return(-.Machine$double.xmax / 2)
    stats$d * log(lambda) - lambda * stats$total_time
  }
}


#' Score for exponential_lifetime
#'
#' Returns a function computing d/lambda - T.
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A function(df, par, ...) computing the score vector
#' @export
score.exponential_lifetime <- function(model, ...) {
  function(df, par, ...) {
    stats <- .exponential_sufficient_stats(df, model)
    lambda <- par[1]
    stopifnot(lambda > 0)
    c(lambda = stats$d / lambda - stats$total_time)
  }
}


#' Hessian of the log-likelihood for exponential_lifetime
#'
#' Returns a function computing the 1x1 Hessian: -d/lambda^2.
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A function(df, par, ...) computing the Hessian matrix
#' @export
hess_loglik.exponential_lifetime <- function(model, ...) {
  function(df, par, ...) {
    stats <- .exponential_sufficient_stats(df, model)
    lambda <- par[1]
    stopifnot(lambda > 0)
    matrix(-stats$d / lambda^2, nrow = 1, ncol = 1,
           dimnames = list("lambda", "lambda"))
  }
}


#' Closed-form MLE for exponential_lifetime
#'
#' Computes the MLE directly as lambda_hat = d / T, bypassing `optim`.
#' This demonstrates that specialized models can provide exact solutions.
#'
#' @param object An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A solver function that takes (df, par, ...) and returns a
#'   `fisher_mle` object. The `par` argument is accepted but ignored
#'   since the MLE is computed in closed form.
#' @export
fit.exponential_lifetime <- function(object, ...) {
  function(df, par = NULL, ...) {
    stats <- .exponential_sufficient_stats(df, object)

    if (stats$d == 0) {
      stop("Cannot compute MLE: no exact (uncensored) observations")
    }

    lambda_hat <- stats$d / stats$total_time

    fisher_mle(
      par = c(lambda = lambda_hat),
      loglik_val = stats$d * log(lambda_hat) - lambda_hat * stats$total_time,
      hessian = matrix(-stats$d / lambda_hat^2, 1, 1,
                       dimnames = list("lambda", "lambda")),
      score_val = c(lambda = 0),
      nobs = stats$n,
      converged = TRUE
    )
  }
}


#' Fisher information matrix for exponential_lifetime
#'
#' Returns the analytical FIM: n / lambda^2 (1x1 matrix).
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A function(theta, n_obs, ...) computing the FIM
#' @export
fim.exponential_lifetime <- function(model, ...) {
  function(theta, n_obs, ...) {
    lambda <- theta[1]
    stopifnot(lambda > 0)
    matrix(n_obs / lambda^2, nrow = 1, ncol = 1,
           dimnames = list("lambda", "lambda"))
  }
}


#' Random data generation for exponential_lifetime
#'
#' Generates exponential data, optionally with right-censoring at a fixed
#' censoring time.
#'
#' @param model An exponential_lifetime model
#' @param ... Additional arguments (ignored)
#' @return A function(theta, n, censor_time, ...) that returns a data frame
#' @importFrom stats rexp
#' @export
rdata.exponential_lifetime <- function(model, ...) {
  function(theta, n, censor_time = Inf, ...) {
    lambda <- theta[1]
    stopifnot(lambda > 0, n > 0)
    x <- rexp(n, rate = lambda)

    no_censoring <- is.null(model$censor_col) || is.infinite(censor_time)
    if (no_censoring) {
      df <- data.frame(x)
      names(df) <- model$ob_col
      return(df)
    }

    df <- data.frame(
      pmin(x, censor_time),
      ifelse(x > censor_time, "right", "exact")
    )
    names(df) <- c(model$ob_col, model$censor_col)
    df
  }
}


#' Compute sufficient statistics for the exponential model
#'
#' @param df Data frame
#' @param model An exponential_lifetime model
#' @return List with d (number of exact obs), total_time (sum of all times),
#'   n (total observations)
#' @keywords internal
.exponential_sufficient_stats <- function(df, model) {
  x <- df[[model$ob_col]]
  n <- length(x)
  stopifnot(n > 0, all(x >= 0))

  if (is.null(model$censor_col)) {
    d <- n
  } else {
    censor <- df[[model$censor_col]]
    exact <- is.na(censor) | censor == "exact"
    d <- sum(exact)
  }

  list(d = d, total_time = sum(x), n = n)
}

#' Fisherian Likelihood Inference
#'
#' Functions for pure likelihood-based inference in the Fisherian tradition.
#' These emphasize the likelihood function itself as the primary object of
#' inference, without requiring probability statements about parameters.
#'
#' @name fisherian
NULL


#' Support Function (Log Relative Likelihood)
#'
#' Computes the support for parameter value theta relative to the MLE:
#' S(theta) = logL(theta) - logL(theta_hat)
#'
#' The support function is always <= 0, with maximum at the MLE.
#' Values of theta with support > -2 are considered well-supported.
#' Values with support > -log(8) ~ -2.08 correspond roughly to a
#' 95% likelihood interval.
#'
#' @param x A fisher_mle object
#' @param theta Parameter value(s) to evaluate
#' @param data Data frame used for likelihood computation
#' @param model The likelihood model used for fitting
#' @param ... Additional arguments passed to loglik
#' @return Support value(s): logL(theta) - logL(theta_hat)
#' @export
support <- function(x, ...) {
  UseMethod("support")
}

#' @rdname support
#' @export
support.fisher_mle <- function(x, theta, data, model, ...) {
  ll <- loglik(model, ...)
  ll(data, theta, ...) - x$loglik
}


#' Relative Likelihood
#'
#' Computes the relative likelihood (likelihood ratio) for theta:
#' R(theta) = L(theta) / L(theta_hat) = exp(S(theta))
#'
#' The relative likelihood is always between 0 and 1, with maximum 1 at the MLE.
#' Common cutoff values:
#' - R >= 0.15 (k=8): roughly equivalent to 95% confidence
#' - R >= 0.10 (k=10): more conservative
#' - R >= 0.05 (k=20): very conservative
#'
#' @param x A fisher_mle object
#' @param theta Parameter value(s) to evaluate
#' @param data Data frame used for likelihood computation
#' @param model The likelihood model used for fitting
#' @param ... Additional arguments passed to loglik
#' @return Relative likelihood value(s): L(theta)/L(theta_hat)
#' @export
relative_likelihood <- function(x, ...) {
  UseMethod("relative_likelihood")
}

#' @rdname relative_likelihood
#' @export
relative_likelihood.fisher_mle <- function(x, theta, data, model, ...) {
  exp(support(x, theta, data, model, ...))
}


#' Likelihood Interval
#'
#' Computes the likelihood interval for a parameter:
#' \code{LI(k) = \{theta : R(theta) >= 1/k\} = \{theta : S(theta) >= -log(k)\}}
#'
#' Unlike confidence intervals, likelihood intervals make no probability
#' statements about the parameter. They simply identify the set of
#' parameter values that are well-supported by the data.
#'
#' Common choices for k:
#' - k = 8: 1/8 likelihood interval (~95% CI equivalent)
#' - k = 15: 1/15 likelihood interval (~99% CI equivalent)
#' - k = 32: 1/32 likelihood interval (~99.9% CI equivalent)
#'
#' For multivariate parameters, specify `param` to get a profile likelihood
#' interval for that parameter (profiling over the others).
#'
#' @param x A fisher_mle object
#' @param data Data frame used for likelihood computation
#' @param model The likelihood model used for fitting
#' @param k Likelihood ratio cutoff (default 8, giving 1/8 interval)
#' @param param Index or name of parameter for profile interval (NULL for all)
#' @param ... Additional arguments passed to optimization
#' @return Matrix with lower and upper bounds for each parameter
#' @importFrom stats uniroot optim
#' @export
likelihood_interval <- function(x, ...) {
  UseMethod("likelihood_interval")
}

#' @rdname likelihood_interval
#' @export
likelihood_interval.fisher_mle <- function(x, data, model, k = 8,
                                            param = NULL, ...) {
  theta_hat <- coef(x)
  p <- length(theta_hat)
  target_support <- -log(k)
  ll <- loglik(model, ...)
  ll_max <- x$loglik

  if (is.null(param)) {
    param <- seq_len(p)
  } else if (is.character(param)) {
    param <- match(param, names(theta_hat))
  }

  # Get reasonable search bounds from Wald interval
  se_vals <- se(x)
  # Use wider search range than Wald CI
  search_mult <- sqrt(2 * k)

  result <- matrix(NA_real_, nrow = length(param), ncol = 2)
  colnames(result) <- c("lower", "upper")

  for (i in seq_along(param)) {
    idx <- param[i]
    se_i <- se_vals[idx]
    mle_i <- theta_hat[idx]

    if (p == 1) {
      # Univariate case: direct root finding
      support_fn <- function(theta_val) {
        ll(data, theta_val, ...) - ll_max - target_support
      }

      # Find lower bound
      lower_search <- mle_i - search_mult * se_i
      result[i, 1] <- tryCatch({
        uniroot(support_fn, c(lower_search, mle_i),
                extendInt = "yes", tol = 1e-6)$root
      }, error = function(e) NA_real_)

      # Find upper bound
      upper_search <- mle_i + search_mult * se_i
      result[i, 2] <- tryCatch({
        uniroot(support_fn, c(mle_i, upper_search),
                extendInt = "yes", tol = 1e-6)$root
      }, error = function(e) NA_real_)

    } else {
      # Multivariate case: profile likelihood
      # For parameter idx, optimize over all other parameters

      profile_support <- function(theta_i) {
        # Fix parameter idx at theta_i, optimize over others
        other_idx <- setdiff(seq_len(p), idx)
        init_other <- theta_hat[other_idx]

        if (length(other_idx) == 0) {
          # Only one parameter
          theta_full <- theta_i
          names(theta_full) <- names(theta_hat)
          return(ll(data, theta_full, ...) - ll_max)
        }

        # Reparameterize: use log-transform for positive MLE values
        # so the optimizer works in unconstrained space
        pos_mask <- init_other > 0
        init_unconstrained <- init_other
        init_unconstrained[pos_mask] <- log(init_other[pos_mask])

        opt_fn <- function(unconstrained) {
          other_params <- unconstrained
          other_params[pos_mask] <- exp(unconstrained[pos_mask])
          theta_full <- numeric(p)
          theta_full[idx] <- theta_i
          theta_full[other_idx] <- other_params
          names(theta_full) <- names(theta_hat)
          val <- tryCatch(
            suppressWarnings(-ll(data, theta_full, ...)),
            error = function(e) .Machine$double.xmax
          )
          if (!is.finite(val)) return(.Machine$double.xmax)
          val
        }

        opt <- tryCatch(
          suppressWarnings(
            optim(init_unconstrained, opt_fn, method = "BFGS",
                  control = list(maxit = 100))
          ),
          error = function(e) list(value = Inf)
        )

        -opt$value - ll_max
      }

      # Root finding for profile
      profile_root <- function(theta_i) {
        profile_support(theta_i) - target_support
      }

      # Find lower bound
      lower_search <- mle_i - search_mult * se_i
      result[i, 1] <- tryCatch({
        uniroot(profile_root, c(lower_search, mle_i),
                tol = 1e-5)$root
      }, error = function(e) NA_real_)

      # Find upper bound
      upper_search <- mle_i + search_mult * se_i
      result[i, 2] <- tryCatch({
        uniroot(profile_root, c(mle_i, upper_search),
                tol = 1e-5)$root
      }, error = function(e) NA_real_)
    }
  }

  if (!is.null(names(theta_hat))) {
    rownames(result) <- names(theta_hat)[param]
  }

  attr(result, "k") <- k
  attr(result, "relative_likelihood_cutoff") <- 1 / k
  class(result) <- c("likelihood_interval", "matrix")
  result
}

#' Print likelihood interval
#' @param x A likelihood_interval object
#' @param ... Additional arguments (ignored)
#' @return The likelihood_interval object, invisibly
#' @export
print.likelihood_interval <- function(x, ...) {
  k <- attr(x, "k")
  cutoff <- attr(x, "relative_likelihood_cutoff")
  cat(sprintf("1/%d Likelihood Interval (R >= %.3f)\n", k, cutoff))
  cat("-----------------------------------\n")
  print(unclass(x))
  invisible(x)
}


#' Profile Log-Likelihood
#'
#' Computes the profile log-likelihood for a subset of parameters.
#' For each value of the parameters of interest, the remaining
#' (nuisance) parameters are optimized out.
#'
#' The profile likelihood is useful for:
#' - Visualizing the likelihood surface
#' - Computing likelihood intervals
#' - Eliminating nuisance parameters
#'
#' @param x A fisher_mle object
#' @param data Data frame used for likelihood computation
#' @param model The likelihood model used for fitting
#' @param param Index or name of parameter(s) to profile
#' @param grid Optional grid of values to evaluate (vector or matrix)
#' @param n_grid Number of grid points if grid not specified (default 50)
#' @param range_mult Multiplier for grid range based on SE (default 4)
#' @param ... Additional arguments passed to loglik
#' @return A data frame with parameter values and profile log-likelihood
#' @importFrom stats optim
#' @export
profile_loglik <- function(x, ...) {
  UseMethod("profile_loglik")
}

#' @rdname profile_loglik
#' @export
profile_loglik.fisher_mle <- function(x, data, model, param,
                                       grid = NULL, n_grid = 50,
                                       range_mult = 4, ...) {
  theta_hat <- coef(x)
  p <- length(theta_hat)
  ll <- loglik(model, ...)

  if (is.character(param)) {
    param <- match(param, names(theta_hat))
  }

  if (length(param) > 2) {
    stop("Profile likelihood only supported for 1 or 2 parameters")
  }

  se_vals <- se(x)
  nuisance_idx <- setdiff(seq_len(p), param)

  # Evaluate profile log-likelihood at fixed values for the parameters of
  # interest, optimizing over nuisance parameters when present.
  eval_profile <- function(theta_i) {
    if (length(nuisance_idx) == 0) {
      theta_full <- theta_i
      names(theta_full) <- names(theta_hat)
      return(ll(data, theta_full, ...))
    }

    init_nuisance <- theta_hat[nuisance_idx]
    opt_fn <- function(nuisance) {
      theta_full <- numeric(p)
      theta_full[param] <- theta_i
      theta_full[nuisance_idx] <- nuisance
      names(theta_full) <- names(theta_hat)
      -ll(data, theta_full, ...)
    }
    opt <- optim(init_nuisance, opt_fn, method = "BFGS",
                 control = list(maxit = 100))
    -opt$value
  }

  if (length(param) == 1) {
    # 1D profile
    if (is.null(grid)) {
      center <- theta_hat[param]
      spread <- range_mult * se_vals[param]
      grid <- seq(center - spread, center + spread, length.out = n_grid)
    }

    profile_vals <- vapply(grid, eval_profile, numeric(1))

    result <- data.frame(
      param = grid,
      loglik = profile_vals,
      support = profile_vals - x$loglik,
      relative_likelihood = exp(profile_vals - x$loglik)
    )

    if (!is.null(names(theta_hat))) {
      names(result)[1] <- names(theta_hat)[param]
    }

  } else {
    # 2D profile
    if (is.null(grid)) {
      grid1 <- seq(theta_hat[param[1]] - range_mult * se_vals[param[1]],
                   theta_hat[param[1]] + range_mult * se_vals[param[1]],
                   length.out = n_grid)
      grid2 <- seq(theta_hat[param[2]] - range_mult * se_vals[param[2]],
                   theta_hat[param[2]] + range_mult * se_vals[param[2]],
                   length.out = n_grid)
      grid <- expand.grid(param1 = grid1, param2 = grid2)
    }

    profile_vals <- vapply(
      seq_len(nrow(grid)),
      function(i) eval_profile(as.numeric(grid[i, ])),
      numeric(1)
    )

    result <- data.frame(
      grid,
      loglik = profile_vals,
      support = profile_vals - x$loglik,
      relative_likelihood = exp(profile_vals - x$loglik)
    )

    if (!is.null(names(theta_hat))) {
      names(result)[1:2] <- names(theta_hat)[param]
    }
  }

  attr(result, "mle") <- x
  attr(result, "param") <- param
  class(result) <- c("profile_loglik", "data.frame")
  result
}

#' Print profile log-likelihood
#' @param x A profile_loglik object
#' @param ... Additional arguments (ignored)
#' @return The profile_loglik object, invisibly
#' @export
print.profile_loglik <- function(x, ...) {
  param <- attr(x, "param")
  mle <- attr(x, "mle")

  cat("Profile Log-Likelihood\n")
  cat("----------------------\n")
  cat("Parameter(s):", paste(names(coef(mle))[param], collapse = ", "), "\n")
  cat("MLE:", paste(format(coef(mle)[param], digits = 4), collapse = ", "), "\n")
  cat("Grid points:", nrow(x), "\n")
  cat("Max profile loglik:", format(max(x$loglik), digits = 4), "\n")
  invisible(x)
}


#' Deviance for likelihood models
#'
#' Computes the deviance, which is useful for model comparison.
#'
#' When called with a single model, returns -2 * logL (the deviance
#' relative to a saturated model).
#'
#' When called with two models, returns the deviance difference:
#' D = 2 * (logL_full - logL_reduced)
#'
#' Under the null hypothesis that the reduced model is correct,
#' D is asymptotically chi-squared with df = p_full - p_reduced.
#'
#' @param object A fisher_mle object
#' @param null_model Optional reduced/null model for comparison
#' @param ... Additional arguments (ignored)
#' @return Deviance value
#' @export
deviance.fisher_mle <- function(object, null_model = NULL, ...) {
  if (is.null(null_model)) {
    -2 * object$loglik
  } else {
    if (!inherits(null_model, "fisher_mle")) {
      stop("null_model must be a fisher_mle object")
    }
    2 * (object$loglik - null_model$loglik)
  }
}


#' Likelihood-Based Evidence
#'
#' Computes the strength of evidence for theta1 vs theta2:
#' E(theta1, theta2) = logL(theta1) - logL(theta2)
#'
#' Positive values favor theta1, negative values favor theta2.
#'
#' Conventional interpretation (following Royall):
#' - |E| > log(8) ~ 2.08: Strong evidence
#' - |E| > log(32) ~ 3.47: Very strong evidence
#'
#' @param model The likelihood model
#' @param ... Additional arguments
#' @return Evidence value (log likelihood ratio)
#' @export
evidence <- function(model, ...) {
  UseMethod("evidence")
}

#' @rdname evidence
#' @param data Data frame for likelihood computation
#' @param theta1 First parameter value
#' @param theta2 Second parameter value
#' @export
evidence.likelihood_model <- function(model, data, theta1, theta2, ...) {
  ll <- loglik(model, ...)
  ll(data, theta1, ...) - ll(data, theta2, ...)
}

#' Fit a model
#'
#' Re-exported from \pkg{generics}. See [generics::fit()] for details.
#'
#' @importFrom generics fit
#' @export fit
#' @name fit
NULL

#' Default MLE solver for subclasses of `likelihood_model`.
#'
#' Note that `likelihood_model` is not a class, but a concept,
#' that other likelihood models implement. They should add
#' `likelihood_model` to their class definition, and then
#' they can use this function to compute the MLE.
#'
#' This function uses the `optim` function to find the MLE of the
#' parameters of a likelihood model. It uses the `loglik` and `score`
#' methods to compute the log-likelihood and score function, respectively.
#'
#' There are a few interesting options for the `control` argument:
#'
#' * `method`: The optimization method to use. The default is `Nelder-Mead`,
#' which is a derivative-free method. Other options like include `BFGS`
#' are gradient-based methods, which may be preferable if you provide
#' a `score` function (rather than using the default finite-difference). There
#' is also the `SANN` method, which is a simulated annealing method. This
#' method is useful for multimodal likelihood functions, where the MLE
#' may be sensitive to the initial guess. The `SANN` method is a more global
#' method, but it is slower and may require some tweaking. Regardless,
#' if you do use `SANN`, you should follow it up with a local search
#' method like `Nelder-Mead` to refine the solution.
#'
#' @param object The `likelihood_model` object
#' @param ... Additional arguments to pass into the likelihood model's
#' `loglik`, `score`, and `hess_loglik` constructors.
#' @return An MLE solver (function) that returns an MLE object and accepts as
#' arguments:
#' - `df`: The data frame
#' - `par`: The initial guess for the parameters
#' - `control`: Control parameters for the optimization algorithm
#' - `...`: Additional arguments to pass into the likelihood model's
#'          constructed functions from `loglik`, `score`, and `hess_loglik`.
#' @importFrom stats optim
#' @importFrom utils modifyList
#' @export
fit.likelihood_model <- function(object, ...) {
  ll <- loglik(object, ...)
  s <- score(object, ...)
  H <- hess_loglik(object, ...)

  function(df, par,
           method = c("Nelder-Mead", "BFGS", "SANN", "CG", "L-BFGS-B", "Brent"),
           ..., control = list()) {
    stopifnot(!is.null(par))
    defaults <- list(fnscale = -1)
    control <- modifyList(defaults, control)
    method <- match.arg(method)

    if (length(par) == 1 && method == "Nelder-Mead") {
      method <- "BFGS"
    }

    sol <- optim(
      par = par,
      fn = function(par) ll(df, par, ...),
      gr = if (method == "SANN") {
        NULL
      } else {
        function(par) s(df, par, ...)
      },
      hessian = FALSE,
      method = method,
      control = control
    )

    hessian <- H(df, sol$par, ...)
    score_at_mle <- s(df, sol$par, ...)

    fisher_mle(
      par = sol$par,
      loglik_val = sol$value,
      hessian = hessian,
      score_val = score_at_mle,
      nobs = nrow(df),
      converged = (sol$convergence == 0),
      optim_result = sol
    )
  }
}


#' Estimate the sampling distribution of the MLE for a likelihood model.
#'
#' We use the bootstrap method. In other words, we treat the data as an
#' empirical distribution and sample from it to get a new dataset, then
#' we fit the model to that dataset and return the MLE. We do this R
#' times and return the R MLEs.
#'
#' This is the default method, but if you want to use a different method,
#' you should define your own method for your likelihood model.
#'
#' @param x The likelihood model
#' @param df Data frame to bootstrap from
#' @param par Initial parameter values
#' @param ... Additional arguments to pass into the likelihood model
#' @param nthreads The number of threads to use for parallelization
#' @return A function that returns a bootstrapped sampling distribution of an
#' MLE (fisher_boot object).
#' @importFrom boot boot
#' @export
sampler.likelihood_model <- function(x, df, par, ..., nthreads = 1L) {
  solver <- fit(x, ...)
  sol <- solver(df, par, ...)
  init_par <- coef(sol)
  function(n) {
    boot_result <- boot(
      data = df,
      statistic = function(df, ind) {
        coef(solver(df[ind, , drop = FALSE], par = init_par))
      },
      R = n,
      parallel = "multicore",
      ncpus = nthreads
    )
    fisher_boot(boot_result, sol)
  }
}

# ==========================================================================
# Import generics from algebraic.mle for S3 method registration.
# The generics themselves are available to users via Depends: algebraic.mle.
# ==========================================================================
#' @importFrom algebraic.mle se bias score_val observed_fim mse
#' @importFrom algebraic.dist sampler params nparams obs
NULL


#' Maximum Likelihood Estimate (Fisherian)
#'
#' Creates a `fisher_mle` object representing a maximum likelihood estimate
#' with methods for standard inference. This class emphasizes the Fisherian
#' approach to likelihood-based inference.
#'
#' @param par Numeric vector of parameter estimates (may be named)
#' @param vcov Variance-covariance matrix of the estimates
#' @param loglik_val Log-likelihood value at the MLE
#' @param hessian Hessian matrix of the log-likelihood at the MLE
#' @param score_val Optional score vector at the MLE (should be near zero)
#' @param nobs Number of observations used in estimation
#' @param converged Logical indicating if optimization converged
#' @param optim_result Raw result from optim() for diagnostics
#' @return An object of class `c("fisher_mle", "mle_fit")`
#' @importFrom stats coef confint cov printCoefmat qnorm rnorm
#' @export
fisher_mle <- function(par, vcov = NULL, loglik_val, hessian = NULL,
                       score_val = NULL, nobs = NULL, converged = TRUE,
                       optim_result = NULL) {

  if (is.null(vcov) && !is.null(hessian)) {
    info <- -hessian
    vcov <- tryCatch(
      solve(info),
      error = function(e) {
        warning("Hessian not invertible, vcov set to NULL")
        NULL
      }
    )
  }

  structure(
    list(
      par = par,
      vcov = vcov,
      loglik = loglik_val,
      hessian = hessian,
      score = score_val,
      nobs = nobs,
      converged = converged,
      optim = optim_result
    ),
    class = c("fisher_mle", "mle_fit")
  )
}


# --------------------------------------------------------------------------
# Base R generic implementations
# --------------------------------------------------------------------------

#' Extract coefficients from fisher_mle object
#' @param object A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return Named numeric vector of parameter estimates
#' @export
coef.fisher_mle <- function(object, ...) {
  object$par
}

#' Extract variance-covariance matrix from fisher_mle object
#' @param object A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return Variance-covariance matrix
#' @export
vcov.fisher_mle <- function(object, ...) {
  object$vcov
}

#' Extract log-likelihood from fisher_mle object
#' @param object A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return A logLik object
#' @export
logLik.fisher_mle <- function(object, ...) {
  val <- object$loglik
  attr(val, "df") <- length(object$par)
  attr(val, "nobs") <- object$nobs
  class(val) <- "logLik"
  val
}

#' Extract number of observations from fisher_mle object
#' @param object A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return Number of observations
#' @importFrom stats nobs
#' @export
nobs.fisher_mle <- function(object, ...) {
  object$nobs
}

#' Compute confidence intervals for fisher_mle parameters
#'
#' Computes asymptotic Wald confidence intervals based on the
#' estimated variance-covariance matrix.
#'
#' @param object A fisher_mle object
#' @param parm Parameter names or indices (NULL for all)
#' @param level Confidence level (default 0.95)
#' @param ... Additional arguments (ignored)
#' @return Matrix with columns for lower and upper bounds
#' @export
confint.fisher_mle <- function(object, parm = NULL, level = 0.95, ...) {
  cf <- coef(object)
  se <- se(object)

  if (is.null(parm)) {
    parm <- seq_along(cf)
  } else if (is.character(parm)) {
    parm <- match(parm, names(cf))
  }

  a <- (1 - level) / 2
  z <- qnorm(1 - a)

  ci <- cbind(
    cf[parm] - z * se[parm],
    cf[parm] + z * se[parm]
  )

  pct <- paste0(format(100 * c(a, 1 - a), trim = TRUE, digits = 3), "%")
  colnames(ci) <- pct

  if (!is.null(names(cf))) {
    rownames(ci) <- names(cf)[parm]
  }

  ci
}

#' Print fisher_mle object
#' @param x A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return The fisher_mle object, invisibly
#' @export
print.fisher_mle <- function(x, ...) {
  cat("Maximum Likelihood Estimate (Fisherian)\n")
  cat("----------------------------------------\n")
  cat("Coefficients:\n")
  print(coef(x))
  cat("\nLog-likelihood:", x$loglik, "\n")
  if (!is.null(x$nobs)) {
    cat("Observations:", x$nobs, "\n")
  }
  if (!x$converged) {
    cat("WARNING: Optimization did not converge\n")
  }
  invisible(x)
}

#' Summarize fisher_mle object
#' @param object A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return A summary_fisher_mle object
#' @export
summary.fisher_mle <- function(object, ...) {
  cf <- coef(object)
  se <- se(object)
  ci <- confint(object)

  coef_table <- cbind(
    Estimate = cf,
    `Std. Error` = se,
    ci
  )

  structure(
    list(
      coefficients = coef_table,
      loglik = object$loglik,
      nobs = object$nobs,
      converged = object$converged,
      aic = stats::AIC(object)
    ),
    class = "summary_fisher_mle"
  )
}

#' Print summary of fisher_mle
#' @param x A summary_fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return The summary_fisher_mle object, invisibly
#' @export
print.summary_fisher_mle <- function(x, ...) {
  cat("Maximum Likelihood Estimate (Fisherian)\n")
  cat("----------------------------------------\n\n")
  cat("Coefficients:\n")
  printCoefmat(x$coefficients, digits = 4, signif.stars = FALSE)
  cat("\n")
  cat("Log-likelihood:", format(x$loglik, digits = 4), "\n")
  cat("AIC:", format(x$aic, digits = 4), "\n")
  if (!is.null(x$nobs)) {
    cat("Number of observations:", x$nobs, "\n")
  }
  if (!x$converged) {
    cat("\nWARNING: Optimization did not converge\n")
  }
  invisible(x)
}


# --------------------------------------------------------------------------
# Custom accessors for fisher_mle
# --------------------------------------------------------------------------

#' Extract score vector from fisher_mle
#'
#' @param x A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return The score vector at the MLE (should be near zero)
#' @export
score_val.fisher_mle <- function(x, ...) {
  x$score
}

#' Extract standard errors from fisher_mle
#'
#' Computes standard errors as the square root of the diagonal
#' of the variance-covariance matrix.
#'
#' @param x A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return Numeric vector of standard errors, or NA values if vcov is NULL
#' @export
se.fisher_mle <- function(x, ...) {
  if (is.null(x$vcov)) {
    return(rep(NA_real_, length(x$par)))
  }
  sqrt(diag(x$vcov))
}

#' Bias for fisher_mle
#'
#' Estimates the bias of the MLE. Without a model and true parameter value,
#' returns zeros (asymptotic bias is zero under regularity conditions).
#' When both `theta` and `model` are provided, performs a Monte Carlo
#' simulation study to estimate finite-sample bias.
#'
#' @param x A fisher_mle object
#' @param theta True parameter value (for simulation studies)
#' @param model A likelihood model with `rdata` and `fit` methods (optional)
#' @param n_sim Number of Monte Carlo replicates (default 1000)
#' @param ... Additional arguments (ignored)
#' @return Bias estimate vector
#' @export
bias.fisher_mle <- function(x, theta = NULL, model = NULL, n_sim = 1000, ...) {
  p <- length(x$par)

  if (is.null(theta) || is.null(model)) {
    return(rep(0, p))
  }

  if (is.null(x$nobs)) {
    stop("Cannot estimate MC bias: nobs not available in fisher_mle object")
  }

  rdata_fn <- rdata(model)
  fit_fn <- fit(model)
  estimates <- matrix(NA_real_, nrow = n_sim, ncol = p)
  n_ok <- 0L

  for (i in seq_len(n_sim)) {
    est <- tryCatch({
      sim_data <- rdata_fn(theta, n = x$nobs)
      coef(fit_fn(sim_data, par = theta))
    }, error = function(e) NULL)

    if (!is.null(est) && length(est) == p) {
      n_ok <- n_ok + 1L
      estimates[n_ok, ] <- est
    }
  }

  if (n_ok < 10L) {
    warning(sprintf("Only %d of %d MC replicates succeeded; bias estimate unreliable",
                    n_ok, n_sim))
    if (n_ok == 0L) return(rep(NA_real_, p))
  }

  colMeans(estimates[seq_len(n_ok), , drop = FALSE]) - theta
}


# --------------------------------------------------------------------------
# algebraic.mle interface compatibility
#
# fisher_mle inherits from "mle_fit" but uses different field names (e.g.,
# $par instead of $theta.hat, $hessian instead of $info). These methods
# ensure that algebraic.mle generics dispatch correctly to fisher_mle
# objects without falling through to *.mle_fit methods that access the
# wrong fields.
# --------------------------------------------------------------------------

#' Extract parameter estimates from fisher_mle
#'
#' @param x A fisher_mle object
#' @return Named numeric vector of parameter estimates
#' @export
params.fisher_mle <- function(x) {
  x$par
}

#' Number of parameters in fisher_mle
#'
#' @param x A fisher_mle object
#' @return Integer count of parameters
#' @export
nparams.fisher_mle <- function(x) {
  length(x$par)
}

#' Observed Fisher information matrix from fisher_mle
#'
#' Returns the negative Hessian of the log-likelihood evaluated at the
#' MLE, which estimates the Fisher information matrix.
#'
#' @param x A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return A matrix, or NULL if the Hessian was not computed
#' @export
observed_fim.fisher_mle <- function(x, ...) {
  if (is.null(x$hessian)) return(NULL)
  -x$hessian
}

#' Extract observed data from fisher_mle
#'
#' `fisher_mle` objects do not store observed data by design, so this
#' always returns NULL.
#'
#' @param x A fisher_mle object
#' @return Always `NULL`. `fisher_mle` objects do not store the observed data.
#' @export
obs.fisher_mle <- function(x) {
  x$obs  # NULL by design
}

#' Mean squared error for fisher_mle
#'
#' Computes MSE = Var + Bias^2 (scalar) or Vcov + bias %*% t(bias) (matrix).
#' Under regularity conditions, asymptotic bias is zero, so MSE equals the
#' variance-covariance matrix. When `model` is provided, uses Monte Carlo
#' bias estimation via [bias.fisher_mle()].
#'
#' @param x A fisher_mle object
#' @param theta True parameter value (for simulation studies)
#' @param model A likelihood model (optional, enables MC bias estimation)
#' @param n_sim Number of MC replicates for bias estimation (default 1000)
#' @param ... Additional arguments (ignored)
#' @return MSE matrix or scalar
#' @importFrom stats vcov
#' @export
mse.fisher_mle <- function(x, theta = NULL, ..., model = NULL, n_sim = 1000) {
  b <- bias(x, theta, model = model, n_sim = n_sim)
  V <- vcov(x)
  if (length(x$par) == 1L) V + b^2 else V + b %*% t(b)
}


# ==========================================================================
# Bootstrap MLE class: fisher_boot
# ==========================================================================

#' Bootstrap MLE Estimate
#'
#' Creates a `fisher_boot` object representing bootstrap-based inference
#' for maximum likelihood estimates.
#'
#' @param boot_result Result from boot::boot()
#' @param original_mle The original fisher_mle object
#' @return An object of class `c("fisher_boot", "fisher_mle", "mle_fit", "boot")`
#' @export
fisher_boot <- function(boot_result, original_mle) {
  structure(
    list(
      par = boot_result$t0,
      replicates = boot_result$t,
      vcov = cov(boot_result$t),
      loglik = original_mle$loglik,
      hessian = original_mle$hessian,
      nobs = original_mle$nobs,
      R = boot_result$R,
      boot = boot_result,
      converged = TRUE
    ),
    class = c("fisher_boot", "fisher_mle", "mle_fit", "boot")
  )
}

#' Confidence intervals from bootstrap
#'
#' Computes bootstrap confidence intervals using various methods.
#'
#' @param object A fisher_boot object
#' @param parm Parameter names or indices (NULL for all)
#' @param level Confidence level (default 0.95)
#' @param type Type of bootstrap CI: "perc", "bca", "norm", or "basic"
#' @param ... Additional arguments passed to boot::boot.ci
#' @return Matrix with columns for lower and upper bounds
#' @importFrom boot boot.ci
#' @export
confint.fisher_boot <- function(object, parm = NULL, level = 0.95,
                                 type = c("perc", "bca", "norm", "basic"),
                                 ...) {
  type <- match.arg(type)
  cf <- coef(object)

  if (is.null(parm)) {
    parm <- seq_along(cf)
  } else if (is.character(parm)) {
    parm <- match(parm, names(cf))
  }

  ci <- matrix(NA_real_, nrow = length(parm), ncol = 2)
  pct <- paste0(format(100 * c((1 - level) / 2, 1 - (1 - level) / 2),
                       trim = TRUE, digits = 3), "%")
  colnames(ci) <- pct

  for (i in seq_along(parm)) {
    idx <- parm[i]
    bci <- tryCatch(
      boot.ci(object$boot, conf = level, type = type, index = idx, ...),
      error = function(e) NULL
    )

    if (!is.null(bci)) {
      ci_vals <- switch(type,
        perc = bci$percent[4:5],
        bca = bci$bca[4:5],
        norm = bci$normal[2:3],
        basic = bci$basic[4:5]
      )
      ci[i, ] <- ci_vals
    }
  }

  if (!is.null(names(cf))) {
    rownames(ci) <- names(cf)[parm]
  }

  ci
}

#' Compute bootstrap bias estimate
#'
#' Estimates bias from bootstrap replicates:
#' bias = mean(bootstrap estimates) - original estimate
#'
#' @param x A fisher_boot object
#' @param theta Ignored (for compatibility)
#' @param ... Additional arguments (ignored)
#' @return Bias estimate vector
#' @export
bias.fisher_boot <- function(x, theta = NULL, ...) {
  colMeans(x$replicates) - x$par
}

#' Print fisher_boot object
#' @param x A fisher_boot object
#' @param ... Additional arguments (ignored)
#' @return The fisher_boot object, invisibly
#' @export
print.fisher_boot <- function(x, ...) {
  cat("Bootstrap MLE (Fisherian)\n")
  cat("-------------------------\n")
  cat("Bootstrap replicates:", x$R, "\n\n")
  cat("Coefficients:\n")
  print(coef(x))
  cat("\nBootstrap SE:", format(se(x), digits = 4), "\n")
  cat("Bootstrap bias:", format(bias(x), digits = 4), "\n")
  invisible(x)
}


#' Asymptotic sampler for fisher_mle
#'
#' Returns a function that samples from the asymptotic normal distribution
#' of the MLE: N(theta_hat, vcov).
#'
#' @param x A fisher_mle object
#' @param ... Additional arguments (ignored)
#' @return Function that takes n and returns n x p matrix of samples
#' @export
sampler.fisher_mle <- function(x, ...) {
  mu <- x$par
  sigma <- x$vcov

  if (is.null(sigma)) {
    stop("Cannot create sampler: vcov is NULL")
  }

  p <- length(mu)

  if (p > 1 && !requireNamespace("mvtnorm", quietly = TRUE)) {
    stop("Package 'mvtnorm' is required for multivariate sampling. ",
         "Install it with: install.packages('mvtnorm')")
  }

  function(n) {
    if (p == 1) {
      matrix(rnorm(n, mean = mu, sd = sqrt(sigma)), ncol = 1)
    } else {
      mvtnorm::rmvnorm(n, mean = mu, sigma = sigma)
    }
  }
}

#' Bootstrap sampler for fisher_boot
#'
#' Returns a function that resamples from the bootstrap distribution.
#'
#' @param x A fisher_boot object
#' @param ... Additional arguments (ignored)
#' @return Function that takes n and returns n x p matrix of resampled estimates
#' @export
sampler.fisher_boot <- function(x, ...) {
  replicates <- x$replicates

  function(n) {
    idx <- sample.int(nrow(replicates), size = n, replace = TRUE)
    replicates[idx, , drop = FALSE]
  }
}

---
title: "Exponential Lifetime Model"
output:
    rmarkdown::html_vignette:
        toc: true
vignette: >
  %\VignetteIndexEntry{Exponential Lifetime Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)
```

## Introduction

The `exponential_lifetime` model is a reference implementation demonstrating
how to build a specialized likelihood model with:

- **Closed-form MLE**: The `fit()` method computes the MLE directly as
  $\hat\lambda = d/T$, bypassing `optim` entirely.
- **Analytical derivatives**: Score, Hessian, and Fisher information matrix
  in closed form.
- **Right-censoring**: Natural handling via the sufficient statistic $(d, T)$.
- **`rdata()` method**: For Monte Carlo validation and bootstrap studies.

The log-likelihood for Exponential($\lambda$) data with right-censoring is:

$$\ell(\lambda) = d \log \lambda - \lambda T$$

where $d$ is the number of exact (uncensored) observations and $T$ is the
total observation time (sum of all times, whether censored or not).

```{r load, message=FALSE, warning=FALSE}
library(likelihood.model)
```

## Uncensored Data

The simplest case: all observations are exact.

```{r uncensored}
set.seed(42)
true_lambda <- 2.5
df <- data.frame(t = rexp(200, rate = true_lambda))

model <- exponential_lifetime("t")
assumptions(model)
```

### Closed-Form MLE

Unlike most models that require `optim`, the exponential model computes the
MLE directly. No initial guess is needed:

```{r fit-uncensored}
mle <- fit(model)(df)

cat("MLE:", coef(mle), "(true:", true_lambda, ")\n")
cat("SE:", se(mle), "\n")
cat("95% CI:", confint(mle)[1, ], "\n")
cat("Score at MLE:", score_val(mle), "(exactly zero by construction)\n")
```

### How the Closed-Form MLE Works

The MLE is $\hat\lambda = n / \sum x_i$ for uncensored data. We can verify
this against the manual calculation:

```{r verify-mle}
n <- nrow(df)
total_time <- sum(df$t)
manual_mle <- n / total_time

cat("fit() result:   ", coef(mle), "\n")
cat("Manual n/T:     ", manual_mle, "\n")
cat("Match:", all.equal(unname(coef(mle)), manual_mle), "\n")
```

## Right-Censored Data

In reliability and survival analysis, observations are often right-censored:
we know the item survived past a certain time, but not when it actually failed.

```{r censored-data}
set.seed(42)
true_lambda <- 2.0
censor_time <- 0.5

# Generate latent failure times
x <- rexp(300, rate = true_lambda)
censored <- x > censor_time

df_cens <- data.frame(
  t = pmin(x, censor_time),
  status = ifelse(censored, "right", "exact")
)

cat("Sample size:", nrow(df_cens), "\n")
cat("Censoring rate:", round(mean(censored) * 100, 1), "%\n")
cat("Exact observations (d):", sum(!censored), "\n")
cat("Total time (T):", round(sum(df_cens$t), 2), "\n")
```

### Fitting with Censoring

```{r fit-censored}
model_cens <- exponential_lifetime("t", censor_col = "status")
assumptions(model_cens)

mle_cens <- fit(model_cens)(df_cens)

cat("\nMLE:", coef(mle_cens), "(true:", true_lambda, ")\n")
cat("SE:", se(mle_cens), "\n")
cat("95% CI:", confint(mle_cens)[1, ], "\n")
```

### Ignoring Censoring Leads to Bias

A common mistake is to treat censored times as if they were exact observations.
This biases the rate estimate upward (equivalently, the mean lifetime downward):

```{r censoring-bias}
# Wrong: treat all observations as exact
model_wrong <- exponential_lifetime("t")
mle_wrong <- fit(model_wrong)(df_cens)

cat("Comparison:\n")
cat("  True lambda:          ", true_lambda, "\n")
cat("  MLE (correct):        ", coef(mle_cens), "\n")
cat("  MLE (ignoring censor):", coef(mle_wrong), "\n")
cat("\nIgnoring censoring overestimates the failure rate!\n")
```

## Analytical Derivatives

The model provides score and Hessian in closed form, which we can verify
against numerical differentiation:

```{r derivatives}
set.seed(99)
df_small <- data.frame(t = rexp(50, rate = 3))
model_small <- exponential_lifetime("t")
lambda_test <- 2.5

# Analytical score
score_fn <- score(model_small)
analytical_score <- score_fn(df_small, lambda_test)

# Numerical score (via numDeriv)
ll_fn <- loglik(model_small)
numerical_score <- numDeriv::grad(
  function(p) ll_fn(df_small, p), lambda_test
)

cat("Score at lambda =", lambda_test, ":\n")
cat("  Analytical:", analytical_score, "\n")
cat("  Numerical: ", numerical_score, "\n")
cat("  Match:", all.equal(unname(analytical_score), numerical_score), "\n")

# Analytical Hessian
hess_fn <- hess_loglik(model_small)
analytical_hess <- hess_fn(df_small, lambda_test)

numerical_hess <- numDeriv::hessian(
  function(p) ll_fn(df_small, p), lambda_test
)

cat("\nHessian at lambda =", lambda_test, ":\n")
cat("  Analytical:", analytical_hess[1, 1], "\n")
cat("  Numerical: ", numerical_hess[1, 1], "\n")
cat("  Match:", all.equal(analytical_hess[1, 1], numerical_hess[1, 1]), "\n")
```

## Fisher Information Matrix

The expected Fisher information for Exponential($\lambda$) is
$I(\lambda) = n / \lambda^2$. The model provides this analytically:

```{r fim}
fim_fn <- fim(model_small)
n_obs <- nrow(df_small)
lambda_hat <- coef(fit(model_small)(df_small))

fim_analytical <- fim_fn(lambda_hat, n_obs)

cat("FIM at MLE (lambda =", lambda_hat, "):\n")
cat("  Analytical n/lambda^2:", n_obs / lambda_hat^2, "\n")
cat("  fim() result:         ", fim_analytical[1, 1], "\n")
cat("  Match:", all.equal(n_obs / unname(lambda_hat)^2, fim_analytical[1, 1]), "\n")
```

## Fisherian Likelihood Inference

The package supports pure likelihood-based inference. Instead of confidence
intervals (which make probability statements about parameters), likelihood
intervals identify the set of parameter values that are "well supported"
by the data.

```{r fisherian}
set.seed(42)
df_fish <- data.frame(t = rexp(100, rate = 2.0))
model_fish <- exponential_lifetime("t")
mle_fish <- fit(model_fish)(df_fish)

# Support function: S(lambda) = logL(lambda) - logL(lambda_hat)
# Support at MLE is always 0
s_mle <- support(mle_fish, coef(mle_fish), df_fish, model_fish)
cat("Support at MLE:", s_mle, "\n")

# At a different value, support is negative
s_alt <- support(mle_fish, c(lambda = 1.5), df_fish, model_fish)
cat("Support at lambda=1.5:", s_alt, "\n")

# Relative likelihood: R(lambda) = L(lambda)/L(lambda_hat) = exp(S)
rl_alt <- relative_likelihood(mle_fish, c(lambda = 1.5), df_fish, model_fish)
cat("Relative likelihood at lambda=1.5:", rl_alt, "\n")
```

### Likelihood Intervals

A 1/k likelihood interval contains all $\lambda$ where
$R(\lambda) \geq 1/k$. The 1/8 interval is roughly analogous to a 95%
confidence interval:

```{r likelihood-interval}
li <- likelihood_interval(mle_fish, df_fish, model_fish, k = 8)
print(li)

cat("\nWald 95% CI for comparison:\n")
print(confint(mle_fish))
```

### Profile Log-Likelihood

```{r profile, fig.width=6, fig.height=4}
prof <- profile_loglik(mle_fish, df_fish, model_fish, param = 1, n_grid = 100)

plot(prof$lambda, prof$support, type = "l", lwd = 2,
     xlab = expression(lambda), ylab = "Support",
     main = "Profile Support Function")
abline(h = -log(8), lty = 2, col = "red")
abline(v = coef(mle_fish), lty = 3, col = "blue")
legend("topright",
       legend = c("Support S(lambda)", "1/8 cutoff", "MLE"),
       lty = c(1, 2, 3), col = c("black", "red", "blue"), lwd = c(2, 1, 1))
```

## Monte Carlo Validation with `rdata()`

The `rdata()` method generates synthetic data from the model, enabling
Monte Carlo studies. Here we verify that the MLE is unbiased and that the
asymptotic standard error matches the empirical standard deviation:

```{r monte-carlo, cache=TRUE}
set.seed(42)
true_lambda <- 3.0
n_obs <- 100
n_sim <- 1000

model_mc <- exponential_lifetime("t")
gen <- rdata(model_mc)

# Simulate n_sim datasets and fit each
mle_vals <- replicate(n_sim, {
  sim_df <- gen(true_lambda, n_obs)
  coef(fit(model_mc)(sim_df))
})

cat("Monte Carlo results (", n_sim, "simulations, n=", n_obs, "):\n")
cat("  True lambda:       ", true_lambda, "\n")
cat("  Mean of MLEs:      ", mean(mle_vals), "\n")
cat("  Bias:              ", mean(mle_vals) - true_lambda, "\n")
cat("  Empirical SE:      ", sd(mle_vals), "\n")
cat("  Theoretical SE:    ", true_lambda / sqrt(n_obs), "\n")
cat("  (SE = lambda/sqrt(n) from Fisher information)\n")
```

## Summary

The `exponential_lifetime` model demonstrates several design patterns
available in the `likelihood.model` framework:

| Feature | How it's implemented |
|---|---|
| Closed-form MLE | Override `fit()` to return `fisher_mle` directly |
| Analytical derivatives | Implement `score()`, `hess_loglik()`, `fim()` |
| Right-censoring | Sufficient statistics $(d, T)$ handle censoring naturally |
| Data generation | `rdata()` method for Monte Carlo validation |
| Fisherian inference | Works out of the box via `support()`, `likelihood_interval()`, etc. |

These patterns apply to any distribution where you want tighter integration
than a generic wrapper provides. For contribution-based models with
heterogeneous observation types, see the `likelihood.contr` package.

## Session Info

```{r session}
sessionInfo()
```

```{r cleanup, include=FALSE}
options(old_opts)
```

---
title: "Integration with algebraic.mle and algebraic.dist"
output:
    rmarkdown::html_vignette:
        toc: true
vignette: >
  %\VignetteIndexEntry{Integration with algebraic.mle and algebraic.dist}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)
```

## The Three-Package Ecosystem

The `likelihood.model` package is part of an ecosystem of three interoperable
packages:

- **`algebraic.dist`** -- probability distribution objects with a common interface
  (`params`, `sampler`, `expectation`, etc.)
- **`algebraic.mle`** -- maximum likelihood estimation infrastructure, defining
  the `mle` class and generics like `params`, `nparams`, `observed_fim`, `rmap`,
  `marginal`, and `expectation`
- **`likelihood.model`** -- likelihood model specification and Fisherian inference

When you fit a model with `likelihood.model`, the result is a `fisher_mle`
object that inherits from `algebraic.mle::mle`. This means all of the
`algebraic.mle` generics work on it directly:

```{r load, message=FALSE, warning=FALSE}
library(likelihood.model)
library(algebraic.mle)
```

## Basic Interface: `params`, `nparams`, `observed_fim`

Let's fit an exponential model and explore the MLE through the `algebraic.mle`
interface:

```{r basic-fit}
set.seed(42)
true_lambda <- 2.0
n <- 200
df <- data.frame(t = rexp(n, rate = true_lambda))

model <- exponential_lifetime("t")
mle_result <- fit(model)(df)

# algebraic.mle generics -- these work because fisher_mle inherits from mle
params(mle_result)
nparams(mle_result)
```

The MLE estimate is close to the true value (2.0), demonstrating good recovery
with $n = 200$ observations. The `params()` and `nparams()` generics work
identically to how they work on native `algebraic.mle::mle` objects -- because
`fisher_mle` inherits from that class, the entire `algebraic.mle` interface is
available without any adapter code.

The `observed_fim()` function returns the observed Fisher information matrix,
computed as the negative Hessian of the log-likelihood at the MLE:

```{r observed-fim}
observed_fim(mle_result)
```

For the exponential model, this is a 1x1 matrix equal to $n/\hat\lambda^2$.
It should be positive at the MLE:

```{r fim-pd}
cat("Observed FIM:", observed_fim(mle_result)[1,1], "\n")
cat("Positive:", observed_fim(mle_result)[1,1] > 0, "\n")
```

For comparison, `vcov()` is the inverse of the observed FIM:

```{r vcov-vs-fim}
vcov(mle_result)
1 / observed_fim(mle_result)
```

The numerical equality confirms that `vcov()` is computed as
`solve(observed_fim())`, the classical relationship
$\widehat{\text{Var}}(\hat\theta) = I(\hat\theta)^{-1}$.

## Parameter Transformations via `rmap`

One powerful feature of `algebraic.mle` is the `rmap()` function, which
transforms MLE parameters using the delta method. This propagates uncertainty
through nonlinear transformations.

For an Exponential($\lambda$) distribution, the mean lifetime is $1/\lambda$.
We can transform our rate MLE to get an MLE for the mean lifetime:

```{r rmap}
# Transform rate -> mean lifetime
mean_life_mle <- rmap(mle_result, function(p) {
  c(mean_lifetime = 1 / p[1])
})

params(mean_life_mle)
se(mean_life_mle)
```

Compare to the true mean lifetime:

```{r rmap-compare}
true_mean <- 1 / true_lambda
cat("True mean lifetime:", true_mean, "\n")
cat("Estimated mean lifetime:", params(mean_life_mle), "\n")
cat("95% CI:", confint(mean_life_mle), "\n")
```

We can also transform to multiple derived quantities at once:

```{r rmap-multi}
# Derive mean, variance, and median of the exponential distribution
derived_mle <- rmap(mle_result, function(p) {
  lam <- p[1]
  c(
    mean   = 1 / lam,
    var    = 1 / lam^2,
    median = log(2) / lam
  )
})

params(derived_mle)
se(derived_mle)
```

Notice that the variance has the largest SE relative to its estimate -- second
moments are inherently harder to estimate than first moments.

## Monte Carlo Expectations

The `expectation()` function computes Monte Carlo expectations over the MLE's
asymptotic sampling distribution.

```{r expectation}
set.seed(123)
# E[lambda^2] under the asymptotic distribution
e_lam_sq <- expectation(mle_result, function(p) p[1]^2,
                         control = list(n = 10000L))
cat("E[lambda^2]:", e_lam_sq, "\n")
cat("lambda^2 at MLE:", params(mle_result)[1]^2, "\n")

# Probability that lambda > 1.5 (under asymptotic distribution)
pr_lam_gt <- expectation(mle_result, function(p) as.numeric(p[1] > 1.5),
                          control = list(n = 10000L))
cat("P(lambda > 1.5):", pr_lam_gt, "\n")
```

## Mean Squared Error

Under standard MLE asymptotics, bias is zero and MSE equals the
variance-covariance matrix:

```{r mse}
mse(mle_result)
all.equal(mse(mle_result), vcov(mle_result))
```

The `TRUE` result confirms that MSE = Vcov under the assumption of zero
asymptotic bias.

## Bootstrap Integration

The `sampler()` generic creates a bootstrap sampling distribution. The resulting
`fisher_boot` object also satisfies the `mle` interface:

```{r bootstrap, cache=TRUE}
set.seed(42)
boot_sampler <- sampler(model, df = df, par = c(lambda = 2))
boot_result <- boot_sampler(n = 200)

# Same algebraic.mle generics work
params(boot_result)
nparams(boot_result)
se(boot_result)
bias(boot_result)
```

The bootstrap bias is negligible compared to the standard error, consistent with
the MLE being asymptotically unbiased.

Compare bootstrap and asymptotic confidence intervals:

```{r boot-compare}
cat("Asymptotic 95% CI:\n")
confint(mle_result)

cat("\nBootstrap percentile 95% CI:\n")
confint(boot_result, type = "perc")
```

## Using `algebraic.dist` Distributions

```{r dist-check, include=FALSE}
has_dist <- requireNamespace("algebraic.dist", quietly = TRUE)
```

```{r dist-section, eval=has_dist}
library(algebraic.dist)
```

The `algebraic.dist` package provides distribution objects with a consistent
interface. We can compare the MLE's asymptotic sampling distribution to
theoretical distribution objects.

For example, the MLE of an exponential rate parameter $\hat\lambda$ is
asymptotically normal with variance $\lambda^2/n$. We can create this
theoretical distribution:

```{r dist-compare, eval=has_dist}
set.seed(42)

cat("MLE:", params(mle_result), "\n")
cat("SE:", se(mle_result), "\n")

# Theoretical asymptotic distribution of the MLE
asymp_var <- true_lambda^2 / n
asymp_dist <- normal(mu = true_lambda, var = asymp_var)
cat("\nTheoretical asymptotic distribution:\n")
cat("  Mean:", params(asymp_dist)[1], "\n")
cat("  Variance:", params(asymp_dist)[2], "\n")

# Compare: sample from the MLE's estimated distribution
mle_sampler <- sampler(mle_result)
set.seed(1)
mle_samples <- mle_sampler(5000)

# vs. sample from the theoretical distribution
dist_sampler <- sampler(asymp_dist)
set.seed(1)
dist_samples <- dist_sampler(5000)

cat("\nMLE sampler mean:", mean(mle_samples), "\n")
cat("Theoretical sampler mean:", mean(dist_samples), "\n")
cat("MLE sampler sd:", sd(mle_samples), "\n")
cat("Theoretical sampler sd:", sd(dist_samples), "\n")
```

## Summary

The `fisher_mle` objects returned by `likelihood.model` are fully compatible
with the `algebraic.mle` interface:

| Generic | What it does on `fisher_mle` |
|---------|------------------------------|
| `params()` | Parameter estimates (same as `coef()`) |
| `nparams()` | Number of parameters |
| `se()` | Standard errors |
| `vcov()` | Variance-covariance matrix |
| `observed_fim()` | Observed Fisher information ($-H$) |
| `bias()` | Asymptotic bias (zero) |
| `mse()` | Mean squared error (equals `vcov` asymptotically) |
| `AIC()` | Akaike information criterion |
| `confint()` | Wald confidence intervals |
| `rmap()` | Delta method parameter transformations |
| `marginal()` | Marginal sampling distributions |
| `expectation()` | Monte Carlo expectations over sampling distribution |
| `sampler()` | Asymptotic or bootstrap sampling function |

```{r cleanup, include=FALSE}
options(old_opts)
```

---
title: "Getting Started with likelihood.model"
output:
    rmarkdown::html_vignette:
        toc: true
vignette: >
  %\VignetteIndexEntry{Getting Started with likelihood.model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width = 6,
    fig.height = 4
)
old_opts <- options(digits = 4)
```

## Introduction

The `likelihood.model` package provides a flexible framework for specifying and
using likelihood models for statistical inference in R. It supports:
- Generic likelihood model interface (`loglik`, `score`, `hess_loglik`, `fim`)
- Maximum likelihood estimation with confidence intervals
- Bootstrap sampling distributions
- Pure likelihood-based (Fisherian) inference
- Likelihood ratio tests

This vignette provides a quick introduction to the main features using the
built-in `exponential_lifetime` model.

For contribution-based models with heterogeneous observation types (exact,
censored, etc.), see the companion package `likelihood.contr`.

## Installation

```{r install, eval=FALSE}
# Install from GitHub
if (!require(devtools)) install.packages("devtools")
devtools::install_github("queelius/likelihood.model")
```

## Loading Packages

```{r load, message=FALSE, warning=FALSE}
library(likelihood.model)
```

## Example 1: Closed-Form MLE with Exponential Lifetime Model

The `exponential_lifetime` model demonstrates a key design feature: specialized
models can override `fit()` to bypass `optim` entirely when the MLE has a
closed-form solution.

For Exponential(lambda) data, the MLE is simply lambda_hat = d / T, where d
is the number of uncensored observations and T is the total observation time.

```{r exponential-example}
# Generate exponential survival data
set.seed(99)
df_exp <- data.frame(t = rexp(200, rate = 3.0))

# Create model and fit -- no initial guess needed!
model_exp <- exponential_lifetime("t")

# View model assumptions
assumptions(model_exp)

# Fit the model
mle_exp <- fit(model_exp)(df_exp)

# View results
summary(mle_exp)
```

### Extracting Results

```{r exponential-results}
# Parameter estimates
cat("Estimated parameters:\n")
print(coef(mle_exp))

# Confidence intervals (Wald-based)
cat("\n95% Confidence Intervals:\n")
print(confint(mle_exp))

# Standard errors
cat("\nStandard Errors:\n")
print(se(mle_exp))

# Log-likelihood value
cat("\nLog-likelihood:", as.numeric(logLik(mle_exp)), "\n")

# AIC for model selection
cat("AIC:", AIC(mle_exp), "\n")

# The score at the MLE is exactly zero (by construction)
cat("Score at MLE:", score_val(mle_exp), "\n")
```

## Example 2: Right-Censored Exponential Data

The model handles right-censoring naturally through the sufficient statistics.

```{r exponential-censored}
# Generate data with right-censoring at time 0.5
set.seed(99)
true_lambda <- 3.0
x <- rexp(200, rate = true_lambda)
censored <- x > 0.5
df_cens <- data.frame(
  t = pmin(x, 0.5),
  status = ifelse(censored, "right", "exact")
)

cat("Censoring rate:", mean(censored) * 100, "%\n")

model_cens <- exponential_lifetime("t", censor_col = "status")
mle_cens <- fit(model_cens)(df_cens)

cat("MLE (censored):", coef(mle_cens), "(true:", true_lambda, ")\n")
cat("95% CI:", confint(mle_cens)[1, ], "\n")
```

## Example 3: Verifying the Score at MLE

At the MLE, the score function (gradient of log-likelihood) should be
approximately zero. This provides a useful diagnostic.

```{r score-check}
model <- exponential_lifetime("t")
df <- data.frame(t = rexp(100, rate = 2.0))

# Fit MLE
mle <- fit(model)(df)

# Evaluate score at MLE
score_func <- score(model)
score_at_mle <- score_func(df, coef(mle))

cat("Score at MLE (should be near zero):\n")
print(score_at_mle)

# The score is also stored in the MLE object
cat("\nScore stored in MLE object:\n")
print(score_val(mle))
```

The score is exactly zero at the MLE for `exponential_lifetime` because
the closed-form solution satisfies the score equation by construction.

## Example 4: Fisherian Likelihood Inference

The package supports pure likelihood-based inference in the Fisherian tradition.
Instead of confidence intervals, we can compute likelihood intervals that make
no probability statements about parameters.

```{r fisherian-example}
# Fit a model
model <- exponential_lifetime("t")
df <- data.frame(t = rexp(200, rate = 2.0))
mle <- fit(model)(df)

# Support function: log relative likelihood
# S(theta) = logL(theta) - logL(theta_hat)
# Support at MLE is always 0
s_at_mle <- support(mle, coef(mle), df, model)
cat("Support at MLE:", s_at_mle, "\n")

# Support at alternative parameter values (negative = less supported)
s_alt <- support(mle, c(lambda = 1.0), df, model)
cat("Support at lambda=1.0:", s_alt, "\n")

# Relative likelihood = exp(support)
rl <- relative_likelihood(mle, c(lambda = 1.0), df, model)
cat("Relative likelihood at lambda=1.0:", rl, "\n")
```

### Likelihood Intervals

```{r likelihood-intervals}
# Compute 1/8 likelihood interval (roughly equivalent to 95% CI)
# This is the set of theta where R(theta) >= 1/8
li <- likelihood_interval(mle, df, model, k = 8)
print(li)

# Compare with Wald confidence interval
cat("\nWald 95% CI:\n")
print(confint(mle))
```

## Example 5: Evidence Function

The evidence function computes the log-likelihood ratio between two parameter
values, providing a pure likelihood-based measure of support.

```{r evidence-example}
model <- exponential_lifetime("t")
set.seed(42)
df <- data.frame(t = rexp(100, rate = 2.0))

# Evidence for lambda=2 vs lambda=1
ev <- evidence(model, df, c(lambda = 2), c(lambda = 1))
cat("Evidence for lambda=2 vs lambda=1:", ev, "\n")

# Positive evidence supports the first hypothesis
if (ev > log(8)) {
  cat("Strong evidence favoring lambda=2\n")
}
```

## Example 6: Bootstrap Sampling Distribution

For more robust inference, especially with small samples, we can use bootstrap
methods to estimate the sampling distribution of the MLE.

```{r bootstrap-example, cache=TRUE}
model <- exponential_lifetime("t")
set.seed(42)
df <- data.frame(t = rexp(100, rate = 2.0))

# Create bootstrap sampler
boot_sampler <- sampler(model, df = df, par = c(lambda = 2))

# Generate bootstrap samples (100 replicates for speed)
boot_result <- boot_sampler(n = 100)

# Compare asymptotic vs bootstrap standard errors
mle <- fit(model)(df)
asymp_se <- se(mle)
boot_se <- se(boot_result)

cat("Standard Error Comparison:\n")
cat("           Asymptotic  Bootstrap\n")
cat(sprintf("Lambda:    %10.4f  %9.4f\n", asymp_se[1], boot_se[1]))

# Bootstrap bias estimate
cat("\nBootstrap Bias Estimate:\n")
print(bias(boot_result))
```

## Summary of Key Functions

### Model Creation
| Function | Description |
|----------|-------------|
| `exponential_lifetime()` | Exponential model with closed-form MLE and optional censoring |

For contribution-based models and standard distribution wrappers, see the
`likelihood.contr` package.

### Likelihood Calculus
| Function | Description |
|----------|-------------|
| `loglik()` | Get log-likelihood function |
| `score()` | Get score (gradient) function |
| `hess_loglik()` | Get Hessian of log-likelihood |
| `fim()` | Fisher information matrix |

### Estimation and Inference
| Function | Description |
|----------|-------------|
| `fit()` | Create MLE solver (returns `fisher_mle`) |
| `sampler()` | Create bootstrap sampler (returns `fisher_boot`) |
| `coef()` | Extract parameter estimates |
| `vcov()` | Variance-covariance matrix |
| `confint()` | Confidence intervals |
| `se()` | Standard errors |

### Fisherian Inference
| Function | Description |
|----------|-------------|
| `support()` | Log relative likelihood: S(theta) = logL(theta) - logL(theta_hat) |
| `relative_likelihood()` | Likelihood ratio: R(theta) = L(theta)/L(theta_hat) |
| `likelihood_interval()` | Interval where R(theta) >= 1/k |
| `profile_loglik()` | Profile log-likelihood |
| `evidence()` | Evidence for theta_1 vs theta_2 |

### Model Comparison
| Function | Description |
|----------|-------------|
| `lrt()` | Likelihood ratio test |
| `AIC()` | Akaike Information Criterion |
| `BIC()` | Bayesian Information Criterion |
| `deviance()` | Deviance statistic |

## Next Steps

For more details, see the other vignettes:

- **Exponential Lifetime Model**: Detailed coverage of `exponential_lifetime`
  with analytical derivatives and right-censoring
- **Integration with algebraic.mle**: Three-package ecosystem demonstration

## Session Info

```{r session}
sessionInfo()
```

```{r cleanup, include=FALSE}
options(old_opts)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{confint.fisher_mle}
\alias{confint.fisher_mle}
\title{Compute confidence intervals for fisher_mle parameters}
\usage{
\method{confint}{fisher_mle}(object, parm = NULL, level = 0.95, ...)
}
\arguments{
\item{object}{A fisher_mle object}

\item{parm}{Parameter names or indices (NULL for all)}

\item{level}{Confidence level (default 0.95)}

\item{...}{Additional arguments (ignored)}
}
\value{
Matrix with columns for lower and upper bounds
}
\description{
Computes asymptotic Wald confidence intervals based on the
estimated variance-covariance matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fit.R
\name{fit}
\alias{fit}
\title{Fit a model}
\description{
Re-exported from \pkg{generics}. See \code{\link[generics:fit]{generics::fit()}} for details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{rdata.exponential_lifetime}
\alias{rdata.exponential_lifetime}
\title{Random data generation for exponential_lifetime}
\usage{
\method{rdata}{exponential_lifetime}(model, ...)
}
\arguments{
\item{model}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
A function(theta, n, censor_time, ...) that returns a data frame
}
\description{
Generates exponential data, optionally with right-censoring at a fixed
censoring time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{params.fisher_mle}
\alias{params.fisher_mle}
\title{Extract parameter estimates from fisher_mle}
\usage{
\method{params}{fisher_mle}(x)
}
\arguments{
\item{x}{A fisher_mle object}
}
\value{
Named numeric vector of parameter estimates
}
\description{
Extract parameter estimates from fisher_mle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{fisher_boot}
\alias{fisher_boot}
\title{Bootstrap MLE Estimate}
\usage{
fisher_boot(boot_result, original_mle)
}
\arguments{
\item{boot_result}{Result from boot::boot()}

\item{original_mle}{The original fisher_mle object}
}
\value{
An object of class \code{c("fisher_boot", "fisher_mle", "mle_fit", "boot")}
}
\description{
Creates a \code{fisher_boot} object representing bootstrap-based inference
for maximum likelihood estimates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{sampler.fisher_mle}
\alias{sampler.fisher_mle}
\title{Asymptotic sampler for fisher_mle}
\usage{
\method{sampler}{fisher_mle}(x, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
Function that takes n and returns n x p matrix of samples
}
\description{
Returns a function that samples from the asymptotic normal distribution
of the MLE: N(theta_hat, vcov).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{rdata}
\alias{rdata}
\title{Random data generation method}
\usage{
rdata(model, ...)
}
\arguments{
\item{model}{A likelihood model}

\item{...}{Additional arguments}
}
\value{
Function that takes (theta, n, ...) and returns a data frame
}
\description{
Returns a function that generates random data from the model's
data-generating process (DGP) at a given parameter value.
}
\details{
This is used by the default \code{fim} method for Monte Carlo estimation
of the Fisher information matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{logLik.fisher_mle}
\alias{logLik.fisher_mle}
\title{Extract log-likelihood from fisher_mle object}
\usage{
\method{logLik}{fisher_mle}(object, ...)
}
\arguments{
\item{object}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
A logLik object
}
\description{
Extract log-likelihood from fisher_mle object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{score_val.fisher_mle}
\alias{score_val.fisher_mle}
\title{Extract score vector from fisher_mle}
\usage{
\method{score_val}{fisher_mle}(x, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
The score vector at the MLE (should be near zero)
}
\description{
Extract score vector from fisher_mle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{profile_loglik}
\alias{profile_loglik}
\alias{profile_loglik.fisher_mle}
\title{Profile Log-Likelihood}
\usage{
profile_loglik(x, ...)

\method{profile_loglik}{fisher_mle}(
  x,
  data,
  model,
  param,
  grid = NULL,
  n_grid = 50,
  range_mult = 4,
  ...
)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments passed to loglik}

\item{data}{Data frame used for likelihood computation}

\item{model}{The likelihood model used for fitting}

\item{param}{Index or name of parameter(s) to profile}

\item{grid}{Optional grid of values to evaluate (vector or matrix)}

\item{n_grid}{Number of grid points if grid not specified (default 50)}

\item{range_mult}{Multiplier for grid range based on SE (default 4)}
}
\value{
A data frame with parameter values and profile log-likelihood
}
\description{
Computes the profile log-likelihood for a subset of parameters.
For each value of the parameters of interest, the remaining
(nuisance) parameters are optimized out.
}
\details{
The profile likelihood is useful for:
\itemize{
\item Visualizing the likelihood surface
\item Computing likelihood intervals
\item Eliminating nuisance parameters
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{hess_loglik.exponential_lifetime}
\alias{hess_loglik.exponential_lifetime}
\title{Hessian of the log-likelihood for exponential_lifetime}
\usage{
\method{hess_loglik}{exponential_lifetime}(model, ...)
}
\arguments{
\item{model}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
A function(df, par, ...) computing the Hessian matrix
}
\description{
Returns a function computing the 1x1 Hessian: -d/lambda^2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{print.likelihood_interval}
\alias{print.likelihood_interval}
\title{Print likelihood interval}
\usage{
\method{print}{likelihood_interval}(x, ...)
}
\arguments{
\item{x}{A likelihood_interval object}

\item{...}{Additional arguments (ignored)}
}
\value{
The likelihood_interval object, invisibly
}
\description{
Print likelihood interval
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{sampler.fisher_boot}
\alias{sampler.fisher_boot}
\title{Bootstrap sampler for fisher_boot}
\usage{
\method{sampler}{fisher_boot}(x, ...)
}
\arguments{
\item{x}{A fisher_boot object}

\item{...}{Additional arguments (ignored)}
}
\value{
Function that takes n and returns n x p matrix of resampled estimates
}
\description{
Returns a function that resamples from the bootstrap distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{print.fisher_mle}
\alias{print.fisher_mle}
\title{Print fisher_mle object}
\usage{
\method{print}{fisher_mle}(x, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
The fisher_mle object, invisibly
}
\description{
Print fisher_mle object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{fim}
\alias{fim}
\title{Fisher information matrix method}
\usage{
fim(model, ...)
}
\arguments{
\item{model}{A likelihood model}

\item{...}{Additional arguments}
}
\value{
Function that computes the FIM given a parameter vector and sample size
}
\description{
This function calculates the Fisher information matrix (FIM), an expectation
over the data-generating process (DGP). The FIM is a crucial concept in
statistics because it provides information about the precision of estimates
and the amount of information that data carries about an unknown parameter.
}
\details{
FIM is a function of the parameters, and is used to compute the standard
errors of the parameters. It is also used to compute the covariance matrix of
the parameters, which is in turn used to compute standard errors of the
parameters.

Additionally, FIM is used to compute the Cramer-Rao lower bound (CRLB),
the inverse of the FIM. CRLB represents the lower limit of the variance that
an unbiased estimator can attain. This is used to compute the asymptotic
relative efficiency (ARE) of an estimator of the parameters, which is the
ratio of the variance of the estimator to the CRLB.

The function computes FIM(x)(theta), the FIM of the likelihood model \code{x}, is
based on the following formulas:

\if{html}{\out{<div class="sourceCode">}}\preformatted{FIM(x)(theta) = E[-loglik_hessian(x)(ob,theta)]
FIM(x)(theta) = E[score(x)(ob,theta) \%*\% t(score(x)(ob,theta))]
}\if{html}{\out{</div>}}

where the expectation is taken with respect to ob ~ DGP. The first formula
is the expected hessian of the log-likelihood function, and the second formula
is the expected outer product of the score function. The two formulas are
equivalent.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-lrt.R
\name{lrt}
\alias{lrt}
\title{Likelihood ratio test}
\usage{
lrt(null, alt, data, null_par, alt_par, dof = NULL, ...)
}
\arguments{
\item{null}{the likelihood model for the simpler (null) hypothesis nested
within the alternative model}

\item{alt}{the likelihood model for the more complicated (alternative)
hypothesis}

\item{data}{a data frame}

\item{null_par}{parameter values under the null model}

\item{alt_par}{parameter values under the alternative model}

\item{dof}{degrees of freedom (computed automatically if NULL)}

\item{...}{additional arguments passed to loglik}
}
\value{
An \code{lrt_result} object with components \code{stat}, \code{p.value}, and \code{dof}
}
\description{
Computes the likelihood ratio test statistic and p-value for
nested models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\docType{package}
\name{likelihood.model-package}
\alias{likelihood.model}
\alias{likelihood.model-package}
\title{likelihood.model: Likelihood-Based Inference in the Fisherian Tradition}
\description{
The \code{likelihood.model} package provides a framework for likelihood-based
inference. The package is organized in layers:

\strong{Core Concept} (\code{core-generics.R}):
The \code{likelihood_model} "concept" -- an abstract interface that any model
can implement. At minimum, implement \code{\link[=loglik]{loglik()}}. Optionally provide
\code{\link[=score]{score()}} and \code{\link[=hess_loglik]{hess_loglik()}} for analytical derivatives; defaults use
numerical differentiation via \code{numDeriv}.

\strong{Core Infrastructure}:
\itemize{
\item \link{fisher_mle} / \link{fisher_boot}: Result objects from MLE fitting, with
methods for \code{\link[=coef]{coef()}}, \code{\link[=vcov]{vcov()}}, \code{\link[=confint]{confint()}}, \code{\link[algebraic.mle:se]{algebraic.mle::se()}}, \code{\link[stats:AIC]{stats::AIC()}}, \code{\link[stats:AIC]{stats::BIC()}},
\code{\link[=summary]{summary()}}.
\item \code{\link[=fit]{fit()}}: Default MLE solver using \code{\link[=optim]{optim()}}. Models can override this
with closed-form solutions (see \link{exponential_lifetime} for an example).
\item Fisherian inference: \code{\link[=support]{support()}}, \code{\link[=relative_likelihood]{relative_likelihood()}},
\code{\link[=likelihood_interval]{likelihood_interval()}}, \code{\link[=profile_loglik]{profile_loglik()}}, \code{\link[=evidence]{evidence()}} -- pure
likelihood-based inference without probability statements.
\item \code{\link[=lrt]{lrt()}}: Likelihood ratio test for nested models.
}

\strong{Example Implementation}:
\link{exponential_lifetime}: Exponential with right-censoring support.
Demonstrates closed-form MLE (no optim needed), analytical FIM,
and \code{\link[=rdata]{rdata()}} for Monte Carlo validation.

For contribution-based models with heterogeneous observation types,
see the companion package \pkg{likelihood.contr}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/queelius/likelihood.model}
  \item \url{https://queelius.github.io/likelihood.model/}
  \item Report bugs at \url{https://github.com/queelius/likelihood.model/issues}
}

}
\author{
\strong{Maintainer}: Alexander Towell \email{lex@metafunctor.com} (\href{https://orcid.org/0000-0001-6443-9897}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{assumptions}
\alias{assumptions}
\title{Retrieve the assumptions the likelihood model makes about the data.}
\usage{
assumptions(model, ...)
}
\arguments{
\item{model}{The likelihood model}

\item{...}{Additional arguments}
}
\value{
A list of assumptions
}
\description{
Retrieve the assumptions the likelihood model makes about the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{obs.fisher_mle}
\alias{obs.fisher_mle}
\title{Extract observed data from fisher_mle}
\usage{
\method{obs}{fisher_mle}(x)
}
\arguments{
\item{x}{A fisher_mle object}
}
\value{
Always \code{NULL}. \code{fisher_mle} objects do not store the observed data.
}
\description{
\code{fisher_mle} objects do not store observed data by design, so this
always returns NULL.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{assumptions.exponential_lifetime}
\alias{assumptions.exponential_lifetime}
\title{Assumptions for exponential_lifetime}
\usage{
\method{assumptions}{exponential_lifetime}(model, ...)
}
\arguments{
\item{model}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
Character vector of assumptions
}
\description{
Assumptions for exponential_lifetime
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{print.fisher_boot}
\alias{print.fisher_boot}
\title{Print fisher_boot object}
\usage{
\method{print}{fisher_boot}(x, ...)
}
\arguments{
\item{x}{A fisher_boot object}

\item{...}{Additional arguments (ignored)}
}
\value{
The fisher_boot object, invisibly
}
\description{
Print fisher_boot object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{hess_loglik}
\alias{hess_loglik}
\title{Hessian of log-likelihood method}
\usage{
hess_loglik(model, ...)
}
\arguments{
\item{model}{The likelihood model}

\item{...}{Additional arguments}
}
\value{
A function to compute the hessian of the log-likelihood given a
data frame and parameters
}
\description{
This function returns the hessian of the log-likelihood function of a model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{fisherian}
\alias{fisherian}
\title{Fisherian Likelihood Inference}
\description{
Functions for pure likelihood-based inference in the Fisherian tradition.
These emphasize the likelihood function itself as the primary object of
inference, without requiring probability statements about parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{support}
\alias{support}
\alias{support.fisher_mle}
\title{Support Function (Log Relative Likelihood)}
\usage{
support(x, ...)

\method{support}{fisher_mle}(x, theta, data, model, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments passed to loglik}

\item{theta}{Parameter value(s) to evaluate}

\item{data}{Data frame used for likelihood computation}

\item{model}{The likelihood model used for fitting}
}
\value{
Support value(s): logL(theta) - logL(theta_hat)
}
\description{
Computes the support for parameter value theta relative to the MLE:
S(theta) = logL(theta) - logL(theta_hat)
}
\details{
The support function is always <= 0, with maximum at the MLE.
Values of theta with support > -2 are considered well-supported.
Values with support > -log(8) ~ -2.08 correspond roughly to a
95\% likelihood interval.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{evidence}
\alias{evidence}
\alias{evidence.likelihood_model}
\title{Likelihood-Based Evidence}
\usage{
evidence(model, ...)

\method{evidence}{likelihood_model}(model, data, theta1, theta2, ...)
}
\arguments{
\item{model}{The likelihood model}

\item{...}{Additional arguments}

\item{data}{Data frame for likelihood computation}

\item{theta1}{First parameter value}

\item{theta2}{Second parameter value}
}
\value{
Evidence value (log likelihood ratio)
}
\description{
Computes the strength of evidence for theta1 vs theta2:
E(theta1, theta2) = logL(theta1) - logL(theta2)
}
\details{
Positive values favor theta1, negative values favor theta2.

Conventional interpretation (following Royall):
\itemize{
\item |E| > log(8) ~ 2.08: Strong evidence
\item |E| > log(32) ~ 3.47: Very strong evidence
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{fit.exponential_lifetime}
\alias{fit.exponential_lifetime}
\title{Closed-form MLE for exponential_lifetime}
\usage{
\method{fit}{exponential_lifetime}(object, ...)
}
\arguments{
\item{object}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
A solver function that takes (df, par, ...) and returns a
\code{fisher_mle} object. The \code{par} argument is accepted but ignored
since the MLE is computed in closed form.
}
\description{
Computes the MLE directly as lambda_hat = d / T, bypassing \code{optim}.
This demonstrates that specialized models can provide exact solutions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-lrt.R
\name{print.lrt_result}
\alias{print.lrt_result}
\title{Print method for likelihood ratio test results}
\usage{
\method{print}{lrt_result}(x, ...)
}
\arguments{
\item{x}{An lrt_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
The lrt_result object, invisibly
}
\description{
Print method for likelihood ratio test results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{.exponential_sufficient_stats}
\alias{.exponential_sufficient_stats}
\title{Compute sufficient statistics for the exponential model}
\usage{
.exponential_sufficient_stats(df, model)
}
\arguments{
\item{df}{Data frame}

\item{model}{An exponential_lifetime model}
}
\value{
List with d (number of exact obs), total_time (sum of all times),
n (total observations)
}
\description{
Compute sufficient statistics for the exponential model
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{is_likelihood_model}
\alias{is_likelihood_model}
\title{Check if an object is a likelihood model}
\usage{
is_likelihood_model(x)
}
\arguments{
\item{x}{An object to test}
}
\value{
Logical indicating whether x is a likelihood model
}
\description{
Tests whether an object satisfies the \code{likelihood_model} concept by
checking if \code{"likelihood_model"} is in its class hierarchy. To be a
likelihood model, at a minimum the object must implement \code{\link[=loglik]{loglik()}}.
For optimal results, it may also provide \code{\link[=score]{score()}} and \code{\link[=hess_loglik]{hess_loglik()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{print.profile_loglik}
\alias{print.profile_loglik}
\title{Print profile log-likelihood}
\usage{
\method{print}{profile_loglik}(x, ...)
}
\arguments{
\item{x}{A profile_loglik object}

\item{...}{Additional arguments (ignored)}
}
\value{
The profile_loglik object, invisibly
}
\description{
Print profile log-likelihood
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{likelihood_interval}
\alias{likelihood_interval}
\alias{likelihood_interval.fisher_mle}
\title{Likelihood Interval}
\usage{
likelihood_interval(x, ...)

\method{likelihood_interval}{fisher_mle}(x, data, model, k = 8, param = NULL, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments passed to optimization}

\item{data}{Data frame used for likelihood computation}

\item{model}{The likelihood model used for fitting}

\item{k}{Likelihood ratio cutoff (default 8, giving 1/8 interval)}

\item{param}{Index or name of parameter for profile interval (NULL for all)}
}
\value{
Matrix with lower and upper bounds for each parameter
}
\description{
Computes the likelihood interval for a parameter:
\code{LI(k) = \{theta : R(theta) >= 1/k\} = \{theta : S(theta) >= -log(k)\}}
}
\details{
Unlike confidence intervals, likelihood intervals make no probability
statements about the parameter. They simply identify the set of
parameter values that are well-supported by the data.

Common choices for k:
\itemize{
\item k = 8: 1/8 likelihood interval (~95\% CI equivalent)
\item k = 15: 1/15 likelihood interval (~99\% CI equivalent)
\item k = 32: 1/32 likelihood interval (~99.9\% CI equivalent)
}

For multivariate parameters, specify \code{param} to get a profile likelihood
interval for that parameter (profiling over the others).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{exponential_lifetime}
\alias{exponential_lifetime}
\title{Exponential lifetime model with right-censoring support}
\usage{
exponential_lifetime(ob_col, censor_col = NULL)
}
\arguments{
\item{ob_col}{The name of the column containing observation times.}

\item{censor_col}{Optional column name indicating censoring status.
When provided, values should be \code{"exact"} for uncensored observations
and \code{"right"} for right-censored observations. When \code{NULL}, all
observations are treated as exact.}
}
\value{
An \code{exponential_lifetime} likelihood model object
}
\description{
A likelihood model for the Exponential(lambda) distribution with
optional right-censoring. This is a reference implementation
demonstrating:
\itemize{
\item \strong{Closed-form MLE}: Overrides \code{fit()} to compute lambda_hat = d/T
directly, bypassing \code{optim} entirely.
\item \strong{Analytical derivatives}: score, Hessian, and FIM in closed form.
\item \strong{Right-censoring}: Natural handling via the sufficient statistic
(d, T) where d = number of exact observations and T = total time.
\item \strong{\code{rdata()} method}: For Monte Carlo validation and FIM estimation.
}

The log-likelihood is:
\deqn{\ell(\lambda) = d \log \lambda - \lambda T}
where d is the number of exact (uncensored) observations and T is the
total observation time (sum of all times, whether censored or not).
}
\examples{
# Uncensored exponential data
model <- exponential_lifetime("t")
df <- data.frame(t = rexp(100, rate = 2))
mle <- fit(model)(df)
coef(mle)  # should be close to 2

# Right-censored data
model_c <- exponential_lifetime("t", censor_col = "status")
df_c <- data.frame(
  t = c(rexp(80, 2), rep(0.5, 20)),
  status = c(rep("exact", 80), rep("right", 20))
)
mle_c <- fit(model_c)(df_c)
coef(mle_c)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{hess_loglik.likelihood_model}
\alias{hess_loglik.likelihood_model}
\title{Default method to compute the hessian of the log-likelihood.}
\usage{
\method{hess_loglik}{likelihood_model}(model, control = list(), ...)
}
\arguments{
\item{model}{The likelihood model}

\item{control}{Custom arguments to pass to \code{numDeriv::hessian}.}

\item{...}{Additional arguments (to pass into \code{loglik})}
}
\value{
A function(df, par, ...) that computes the Hessian matrix of the
log-likelihood evaluated at \code{par} given data \code{df}.
}
\description{
In case a \code{hess_loglik} method is not provided, this function will be
used.
It computes the hessian of the log-likelihood function using numerical
differentiation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{bias.fisher_mle}
\alias{bias.fisher_mle}
\title{Bias for fisher_mle}
\usage{
\method{bias}{fisher_mle}(x, theta = NULL, model = NULL, n_sim = 1000, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{theta}{True parameter value (for simulation studies)}

\item{model}{A likelihood model with \code{rdata} and \code{fit} methods (optional)}

\item{n_sim}{Number of Monte Carlo replicates (default 1000)}

\item{...}{Additional arguments (ignored)}
}
\value{
Bias estimate vector
}
\description{
Estimates the bias of the MLE. Without a model and true parameter value,
returns zeros (asymptotic bias is zero under regularity conditions).
When both \code{theta} and \code{model} are provided, performs a Monte Carlo
simulation study to estimate finite-sample bias.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fit.R
\name{sampler.likelihood_model}
\alias{sampler.likelihood_model}
\title{Estimate the sampling distribution of the MLE for a likelihood model.}
\usage{
\method{sampler}{likelihood_model}(x, df, par, ..., nthreads = 1L)
}
\arguments{
\item{x}{The likelihood model}

\item{df}{Data frame to bootstrap from}

\item{par}{Initial parameter values}

\item{...}{Additional arguments to pass into the likelihood model}

\item{nthreads}{The number of threads to use for parallelization}
}
\value{
A function that returns a bootstrapped sampling distribution of an
MLE (fisher_boot object).
}
\description{
We use the bootstrap method. In other words, we treat the data as an
empirical distribution and sample from it to get a new dataset, then
we fit the model to that dataset and return the MLE. We do this R
times and return the R MLEs.
}
\details{
This is the default method, but if you want to use a different method,
you should define your own method for your likelihood model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{se.fisher_mle}
\alias{se.fisher_mle}
\title{Extract standard errors from fisher_mle}
\usage{
\method{se}{fisher_mle}(x, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
Numeric vector of standard errors, or NA values if vcov is NULL
}
\description{
Computes standard errors as the square root of the diagonal
of the variance-covariance matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{bias.fisher_boot}
\alias{bias.fisher_boot}
\title{Compute bootstrap bias estimate}
\usage{
\method{bias}{fisher_boot}(x, theta = NULL, ...)
}
\arguments{
\item{x}{A fisher_boot object}

\item{theta}{Ignored (for compatibility)}

\item{...}{Additional arguments (ignored)}
}
\value{
Bias estimate vector
}
\description{
Estimates bias from bootstrap replicates:
bias = mean(bootstrap estimates) - original estimate
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{loglik}
\alias{loglik}
\title{Log-likelihood method}
\usage{
loglik(model, ...)
}
\arguments{
\item{model}{The likelihood model}

\item{...}{Additional arguments}
}
\value{
A log-likelihood function to compute the log-likelihood given a data
frame and parameters.
}
\description{
This function returns the log-likelihood function of a model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{fisher_mle}
\alias{fisher_mle}
\title{Maximum Likelihood Estimate (Fisherian)}
\usage{
fisher_mle(
  par,
  vcov = NULL,
  loglik_val,
  hessian = NULL,
  score_val = NULL,
  nobs = NULL,
  converged = TRUE,
  optim_result = NULL
)
}
\arguments{
\item{par}{Numeric vector of parameter estimates (may be named)}

\item{vcov}{Variance-covariance matrix of the estimates}

\item{loglik_val}{Log-likelihood value at the MLE}

\item{hessian}{Hessian matrix of the log-likelihood at the MLE}

\item{score_val}{Optional score vector at the MLE (should be near zero)}

\item{nobs}{Number of observations used in estimation}

\item{converged}{Logical indicating if optimization converged}

\item{optim_result}{Raw result from optim() for diagnostics}
}
\value{
An object of class \code{c("fisher_mle", "mle_fit")}
}
\description{
Creates a \code{fisher_mle} object representing a maximum likelihood estimate
with methods for standard inference. This class emphasizes the Fisherian
approach to likelihood-based inference.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{confint.fisher_boot}
\alias{confint.fisher_boot}
\title{Confidence intervals from bootstrap}
\usage{
\method{confint}{fisher_boot}(
  object,
  parm = NULL,
  level = 0.95,
  type = c("perc", "bca", "norm", "basic"),
  ...
)
}
\arguments{
\item{object}{A fisher_boot object}

\item{parm}{Parameter names or indices (NULL for all)}

\item{level}{Confidence level (default 0.95)}

\item{type}{Type of bootstrap CI: "perc", "bca", "norm", or "basic"}

\item{...}{Additional arguments passed to boot::boot.ci}
}
\value{
Matrix with columns for lower and upper bounds
}
\description{
Computes bootstrap confidence intervals using various methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{mse.fisher_mle}
\alias{mse.fisher_mle}
\title{Mean squared error for fisher_mle}
\usage{
\method{mse}{fisher_mle}(x, theta = NULL, ..., model = NULL, n_sim = 1000)
}
\arguments{
\item{x}{A fisher_mle object}

\item{theta}{True parameter value (for simulation studies)}

\item{...}{Additional arguments (ignored)}

\item{model}{A likelihood model (optional, enables MC bias estimation)}

\item{n_sim}{Number of MC replicates for bias estimation (default 1000)}
}
\value{
MSE matrix or scalar
}
\description{
Computes MSE = Var + Bias^2 (scalar) or Vcov + bias \%*\% t(bias) (matrix).
Under regularity conditions, asymptotic bias is zero, so MSE equals the
variance-covariance matrix. When \code{model} is provided, uses Monte Carlo
bias estimation via \code{\link[=bias.fisher_mle]{bias.fisher_mle()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{print.summary_fisher_mle}
\alias{print.summary_fisher_mle}
\title{Print summary of fisher_mle}
\usage{
\method{print}{summary_fisher_mle}(x, ...)
}
\arguments{
\item{x}{A summary_fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
The summary_fisher_mle object, invisibly
}
\description{
Print summary of fisher_mle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{loglik.likelihood_model}
\alias{loglik.likelihood_model}
\title{Default loglik method}
\usage{
\method{loglik}{likelihood_model}(model, ...)
}
\arguments{
\item{model}{A likelihood model}

\item{...}{Additional arguments}
}
\value{
Never returns; always errors.
}
\description{
Provides a clear error when a model class does not implement \code{loglik}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{fim.likelihood_model}
\alias{fim.likelihood_model}
\title{Default FIM method using Monte Carlo estimation}
\usage{
\method{fim}{likelihood_model}(model, ...)
}
\arguments{
\item{model}{A likelihood model}

\item{...}{Additional arguments (currently unused; reserved for future
extension at closure-construction time)}
}
\value{
Function that takes (theta, n_obs, n_samples, ...) and returns
FIM matrix. The inner \code{...} is forwarded to \code{rdata} only.
}
\description{
Computes the Fisher information matrix by Monte Carlo simulation using
the negative expected Hessian approach. For each of \code{n_samples} replicates,
generates a single-observation dataset via \code{rdata}, computes
\code{-hess_loglik(single_obs, theta)}, and averages. The result is
\code{n_obs * I_1(theta)} where \code{I_1(theta)} is the per-observation FIM.
}
\details{
This default requires the model to implement \code{rdata} and \code{hess_loglik}
(or \code{loglik}, since \code{hess_loglik} falls back to numerical differentiation).

Extra arguments via \code{...} to the returned FIM function are forwarded
to \code{rdata} only (i.e., they parameterize the data-generating process:
censoring time, masking probability, observation functor, etc.) and
are NOT passed to the likelihood evaluator \code{hess_loglik}. This honors
the separation between the DGP layer and the likelihood layer: the
likelihood is computed on data, not on the DGP knobs that produced it.
Forwarding DGP kwargs into the likelihood layer was both an axiom
violation (under masking condition C3 the likelihood does not depend
on the masking probability) and a partial-matching footgun (a kwarg
named \code{p} would collide with the formal \code{par}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{observed_fim.fisher_mle}
\alias{observed_fim.fisher_mle}
\title{Observed Fisher information matrix from fisher_mle}
\usage{
\method{observed_fim}{fisher_mle}(x, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
A matrix, or NULL if the Hessian was not computed
}
\description{
Returns the negative Hessian of the log-likelihood evaluated at the
MLE, which estimates the Fisher information matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fit.R
\name{fit.likelihood_model}
\alias{fit.likelihood_model}
\title{Default MLE solver for subclasses of \code{likelihood_model}.}
\usage{
\method{fit}{likelihood_model}(object, ...)
}
\arguments{
\item{object}{The \code{likelihood_model} object}

\item{...}{Additional arguments to pass into the likelihood model's
\code{loglik}, \code{score}, and \code{hess_loglik} constructors.}
}
\value{
An MLE solver (function) that returns an MLE object and accepts as
arguments:
\itemize{
\item \code{df}: The data frame
\item \code{par}: The initial guess for the parameters
\item \code{control}: Control parameters for the optimization algorithm
\item \code{...}: Additional arguments to pass into the likelihood model's
constructed functions from \code{loglik}, \code{score}, and \code{hess_loglik}.
}
}
\description{
Note that \code{likelihood_model} is not a class, but a concept,
that other likelihood models implement. They should add
\code{likelihood_model} to their class definition, and then
they can use this function to compute the MLE.
}
\details{
This function uses the \code{optim} function to find the MLE of the
parameters of a likelihood model. It uses the \code{loglik} and \code{score}
methods to compute the log-likelihood and score function, respectively.

There are a few interesting options for the \code{control} argument:
\itemize{
\item \code{method}: The optimization method to use. The default is \code{Nelder-Mead},
which is a derivative-free method. Other options like include \code{BFGS}
are gradient-based methods, which may be preferable if you provide
a \code{score} function (rather than using the default finite-difference). There
is also the \code{SANN} method, which is a simulated annealing method. This
method is useful for multimodal likelihood functions, where the MLE
may be sensitive to the initial guess. The \code{SANN} method is a more global
method, but it is slower and may require some tweaking. Regardless,
if you do use \code{SANN}, you should follow it up with a local search
method like \code{Nelder-Mead} to refine the solution.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisher_mle.R
\name{summary.fisher_mle}
\alias{summary.fisher_mle}
\title{Summarize fisher_mle object}
\usage{
\method{summary}{fisher_mle}(object, ...)
}
\arguments{
\item{object}{A fisher_mle object}

\item{...}{Additional arguments (ignored)}
}
\value{
A summary_fisher_mle object
}
\description{
Summarize fisher_mle object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{score}
\alias{score}
\title{Score method}
\usage{
score(model, ...)
}
\arguments{
\item{model}{The likelihood model}

\item{...}{Additional arguments}
}
\value{
A function to compute the score given a data frame and parameters
}
\description{
This function returns the score function of a model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{relative_likelihood}
\alias{relative_likelihood}
\alias{relative_likelihood.fisher_mle}
\title{Relative Likelihood}
\usage{
relative_likelihood(x, ...)

\method{relative_likelihood}{fisher_mle}(x, theta, data, model, ...)
}
\arguments{
\item{x}{A fisher_mle object}

\item{...}{Additional arguments passed to loglik}

\item{theta}{Parameter value(s) to evaluate}

\item{data}{Data frame used for likelihood computation}

\item{model}{The likelihood model used for fitting}
}
\value{
Relative likelihood value(s): L(theta)/L(theta_hat)
}
\description{
Computes the relative likelihood (likelihood ratio) for theta:
R(theta) = L(theta) / L(theta_hat) = exp(S(theta))
}
\details{
The relative likelihood is always between 0 and 1, with maximum 1 at the MLE.
Common cutoff values:
\itemize{
\item R >= 0.15 (k=8): roughly equivalent to 95\% confidence
\item R >= 0.10 (k=10): more conservative
\item R >= 0.05 (k=20): very conservative
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{print.likelihood_model}
\alias{print.likelihood_model}
\title{Print method for likelihood models}
\usage{
\method{print}{likelihood_model}(x, show.loglik = FALSE, ...)
}
\arguments{
\item{x}{A likelihood model}

\item{show.loglik}{Logical; if TRUE, print the log-likelihood function}

\item{...}{Additional arguments (ignored)}
}
\value{
The likelihood model object, invisibly
}
\description{
Print method for likelihood models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{fim.exponential_lifetime}
\alias{fim.exponential_lifetime}
\title{Fisher information matrix for exponential_lifetime}
\usage{
\method{fim}{exponential_lifetime}(model, ...)
}
\arguments{
\item{model}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
A function(theta, n_obs, ...) computing the FIM
}
\description{
Returns the analytical FIM: n / lambda^2 (1x1 matrix).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example-exponential.R
\name{score.exponential_lifetime}
\alias{score.exponential_lifetime}
\title{Score for exponential_lifetime}
\usage{
\method{score}{exponential_lifetime}(model, ...)
}
\arguments{
\item{model}{An exponential_lifetime model}

\item{...}{Additional arguments (ignored)}
}
\value{
A function(df, par, ...) computing the score vector
}
\description{
Returns a function computing d/lambda - T.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-generics.R
\name{score.likelihood_model}
\alias{score.likelihood_model}
\title{Default score method}
\usage{
\method{score}{likelihood_model}(model, control = list(), ...)
}
\arguments{
\item{model}{The likelihood model}

\item{control}{Custom arguments to pass to \code{numDeriv::grad}.}

\item{...}{Additional arguments (to pass into \code{loglik})}
}
\value{
A function to compute the score given a data frame and parameters
}
\description{
In case a \code{score} method is not provided, this function will be used.
It computes the score by numerical differentiation of the log-likelihood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core-fisherian.R
\name{deviance.fisher_mle}
\alias{deviance.fisher_mle}
\title{Deviance for likelihood models}
\usage{
\method{deviance}{fisher_mle}(object, null_model = NULL, ...)
}
\arguments{
\item{object}{A fisher_mle object}

\item{null_model}{Optional reduced/null model for comparison}

\item{...}{Additional arguments (ignored)}
}
\value{
Deviance value
}
\description{
Computes the deviance, which is useful for model comparison.
}
\details{
When called with a single model, returns -2 * logL (the deviance
relative to a saturated model).

When called with two models, returns the deviance difference:
D = 2 * (logL_full - logL_reduced)

Under the null hypothesis that the reduced model is correct,
D is asymptotically chi-squared with df = p_full - p_reduced.
}
